REM The Dungeon Maze Adventure Game v12.0 r3.0
REM   Main Program Source Module for QB64
REM
REM ********
REM   The Dungeon QB45 source conversion to QB64 changes:
REM      Removed all .asm inline code (Keytrap and Swapbas)
REM      Removed all calls to Interrupt
REM      Adds _TITLE and _LIMIT
REM      Changes all arrays in UDTs to strings
REM      Consolidates all .bas into Dungeon.bas including:
REM        Ansi, Edit, Help, Util, Mapedit
REM      Changes keys:
REM        Alt-Backspace to Ctrl-Delete for Dos Shell
REM        Alt-Enter To Alt-Up for Map Editor
REM        Alt-Tab To Alt-Left for Make Globe
REM      Adds Alt-Right for Brief Help
REM      Adds -ansi to command line
REM      Adds -util to command line
REM      Adds -edit to command line
REM      Adds -mapedit to command line
REM ********

_TITLE "DUNGEON"

' declare arrays at runtime
REM $DYNAMIC

' declare all variables as integer
DEFINT A-Z

' standard error routine
ON ERROR GOTO Error.Routine

' declares variable storage for dungeon record structure type
TYPE DungeonType
    LevelData AS STRING * 3840 ' 24x80x2
    ScreenData AS STRING * 3840 ' 24x80x2
    Stats AS STRING * 64 ' int
END TYPE

' declares variable storage for user record structure type
TYPE UserType
    CodeName AS STRING * 32
    PassWord AS STRING * 32
    Spell AS STRING * 80 ' int
    Wand AS STRING * 20 ' int
    Potion AS STRING * 20 ' int
    Staff AS STRING * 20 ' int
    Ring AS STRING * 20 ' int
    Globe AS STRING * 20 ' int
    Stats1 AS STRING * 48 ' double
    Stats2 AS STRING * 34 ' int
END TYPE

' declare structures common to main program
DIM SHARED DungeonRecord AS DungeonType
DIM SHARED UserRecord AS UserType

' declare arrays common to main program
DIM SHARED Maze1(30, 20) AS INTEGER, Maze2(30, 20) AS INTEGER
DIM SHARED Monsters(24, 6) AS DOUBLE, MonsterNames(101) AS STRING

' define global variables
DIM SHARED Futil AS INTEGER

KEY OFF ' disable function keys
SCREEN 0 ' reset to default text screen
RANDOMIZE TIMER ' seed random number generator
CONST Util.Filename$ = "ranklist.dat"

' parse command line
X$ = COMMAND$
IF LEN(X$) THEN
    IF LCASE$(X$) = "-ansi" THEN
        _TITLE "DUNGEON -ansi"
        CALL ANSI
        COLOR 15, 1 ' change color
        LOCATE 24, 27 ' locate message
        PRINT "Press any key to continue:";
        X$ = "" ' reset input
        WHILE X$ = "" ' wait for input
            _LIMIT 100
            X$ = INKEY$ ' get input
        WEND ' end wait
        COLOR 7, 0
        END
    END IF
    IF LCASE$(X$) = "-help" THEN
        PRINT "Dungeon command line options:"
        PRINT "  -ansi  display splash screen."
        PRINT "  -util  run report maker."
        PRINT "  -edit  start user editor."
        PRINT "  -mapedit  start user map editor."
        END
    END IF
    IF LCASE$(X$) = "-util" THEN
        _TITLE "DUNGEON -util"
        Futil = -1
        GOSUB Open.User.File
        CALL User.List.Menu ' run utility program
        PRINT "User list report written to file:"
        PRINT "  "; Util.Filename$
        END
    END IF
    IF LCASE$(X$) = "-edit" THEN
        _TITLE "DUNGEON -edit"
        GOSUB Open.User.File
        CLS ' clear screen
        CALL Edit.User.Menu ' run edit program
        END
    END IF
    IF LCASE$(X$) = "-mapedit" THEN
        _TITLE "DUNGEON -mapedit"
        GOSUB Open.User.File
        CLS ' clear screen
        CALL Map.Editor ' run utility program
        END
    END IF
    PRINT "Command line error."
    END
END IF

' start program
CALL ANSI
COLOR 15, 1 ' change color
LOCATE 24, 7 ' locate message
PRINT "Press <enter> to continue to The Dungeon, or press <space> to Quit";
DO ' input loop
    X$ = "" ' reset input
    WHILE X$ = "" ' wait for input
        _LIMIT 100
        X$ = INKEY$ ' get input
    WEND ' end wait
    SELECT CASE X$ ' choose input
        CASE " " ' space
            COLOR 7, 0 ' reset color
            CLS ' clear screen
            END ' end program
        CASE CHR$(13) ' return pressed
            EXIT DO ' exit input loop
    END SELECT ' end choose input
LOOP ' end input loop
GOSUB Open.User.File ' open the users data file
RESTORE MonsterData ' read names of monsters from data
FOR i = 1 TO 101 ' 100 monsters plus 1 demon
    READ MonsterNames(i) ' load data into monster name array
NEXT ' end load loop
DO ' main program loop
    ' user presses enter to play,
    ' or another key to exit program.
    COLOR 14, 1 ' blue background
    CLS ' clear screen
    LOCATE 10, 27, 0 ' locate for message
    COLOR 14 ' yellow color
    PRINT CHR$(201); ' print ascii character
    PRINT STRING$(25, CHR$(205)); ' print ascii character
    PRINT CHR$(187); ' print ascii character
    LOCATE 11, 27, 0 ' position cursor
    PRINT CHR$(186); ' print ascii character
    COLOR 15 ' change color
    PRINT " Welcome to the ";
    COLOR 12 ' change color
    PRINT "Dungeon! ";
    COLOR 14 ' change color
    PRINT CHR$(186); ' print ascii character
    LOCATE 12, 27, 0 ' position cursor
    PRINT CHR$(186); ' print ascii character
    COLOR 15 ' change color
    PRINT " Hit ";
    COLOR 10 ' change color
    PRINT "<enter>";
    COLOR 15 ' change color
    PRINT " to play or  ";
    COLOR 14 ' change color
    PRINT CHR$(186); ' print ascii character
    LOCATE 13, 27, 0 ' position cursor
    PRINT CHR$(186); ' print ascii character
    COLOR 15 ' change color
    PRINT " any other key to quit.. ";
    COLOR 14 ' change color
    PRINT CHR$(186); ' print ascii character
    LOCATE 14, 27, 0 ' position cursor
    PRINT CHR$(200); ' print ascii character
    PRINT STRING$(25, CHR$(205)); ' print ascii character
    PRINT CHR$(188); ' print ascii character
    LOCATE 15, 33, 0 ' position cursor
    COLOR 15 ' change color
    PRINT "Turn on Numlock";
    COLOR 7 ' change color
    X$ = "" ' get keystroke
    WHILE X$ = "" ' wait for input
        _LIMIT 100
        X$ = INKEY$ ' get keystroke
    WEND ' end wait
    IF X$ <> CHR$(13) THEN
        EXIT DO
    END IF
    GOSUB Clear.Monsters
    DO ' loop to get user codename into CodeName$ and password into PassWord$
        COLOR 14, 1 ' blue background
        CLS ' clear screen
        LOCATE 11, 25, 0 ' locate for message
        COLOR 14 ' yellow color
        PRINT "Before you enter the ";
        COLOR 13 ' change color
        PRINT "Dungeon,";
        LOCATE 12, 25, 0 ' position cursor
        COLOR 14 ' change color
        PRINT "you must inscibe your name in";
        LOCATE 13, 25, 0 ' position cursor
        PRINT "   the book of adventurers.";
        COLOR 10 ' change color
        LOCATE 15, 28, 0 ' position cursor
        PRINT CHR$(201); ' print ascii character
        PRINT STRING$(20, CHR$(205)); ' print ascii character
        PRINT CHR$(187); ' print ascii character
        LOCATE 16, 49, 0 ' position cursor
        PRINT CHR$(186); ' block character
        LOCATE 17, 49, 0 ' position cursor
        PRINT CHR$(188); ' block character
        FOR i = 48 TO 29 STEP -1
            LOCATE 17, i, 0 ' position cursor
            PRINT CHR$(205); ' block character
        NEXT
        LOCATE 17, 28, 0 ' position cursor
        PRINT CHR$(200); ' block character
        LOCATE 16, 28, 0 ' position cursor
        PRINT CHR$(186); ' block character
        LOCATE 16, 29, 1 ' position cursor
        COLOR 15 ' change color
        ' loop to retrieve CodeName$ character by character.
        '   backspace, enter, and characters A to Z are accepted,
        '   or ignore for enter with no input. Maximum 20 characters.
        CodeName$ = "" ' clear CodeName$
        DO
            X$ = "" ' get keystroke
            WHILE X$ = "" ' wait for input
                _LIMIT 100
                X$ = INKEY$ ' get keystroke
            WEND ' end wait
            SELECT CASE X$
                CASE CHR$(13) ' enter key
                    IF LEN(CodeName$) THEN
                        EXIT DO ' exit loop
                    END IF
                CASE CHR$(8) ' backspace
                    IF LEN(CodeName$) THEN
                        LOCATE 16, POS(0) - 1, 1 ' position cursor
                        PRINT " ";
                        LOCATE 16, POS(0) - 1, 1 ' position cursor
                        CodeName$ = LEFT$(CodeName$, LEN(CodeName$) - 1) ' truncate input one character
                    END IF
                CASE "a" TO "z", "A" TO "Z", " " ' check valid character
                    IF LEN(CodeName$) < 20 THEN ' verify length of string
                        CodeName$ = CodeName$ + X$ ' append to CodeName$
                        PRINT X$;
                    END IF
            END SELECT
        LOOP
        LOCATE 19, 25, 0 ' position cursor
        PRINT "Now inscribe your secret name.";
        COLOR 10 ' change color
        LOCATE 21, 28, 0 ' position cursor
        PRINT CHR$(201); ' print ascii character
        PRINT STRING$(20, CHR$(205)); ' print ascii character
        PRINT CHR$(187); ' print ascii character
        LOCATE 22, 49, 0 ' position cursor
        PRINT CHR$(186); ' block character
        LOCATE 23, 49, 0 ' position cursor
        PRINT CHR$(188); ' block character
        FOR i = 48 TO 29 STEP -1
            LOCATE 23, i, 0 ' position cursor
            PRINT CHR$(205); ' block character
        NEXT
        LOCATE 23, 28, 0 ' position cursor
        PRINT CHR$(200); ' block character
        LOCATE 22, 28, 0 ' position cursor
        PRINT CHR$(186); ' block character
        LOCATE 22, 29, 1 ' position cursor
        COLOR 15 ' change color
        ' loop to retrieve PassWord$ character by character.
        '   backspace, enter, and characters A to Z are accepted,
        '   or nothing for enter with no input. Maximum 20 characters.
        PassWord$ = "" ' clear PassWord$
        DO
            X$ = "" ' get keystroke
            WHILE X$ = "" ' wait for input
                _LIMIT 100
                X$ = INKEY$ ' get keystroke
            WEND ' end wait
            SELECT CASE X$
                CASE CHR$(13) ' enter key
                    IF LEN(PassWord$) THEN
                        COLOR 7 ' change color
                        EXIT DO ' exit loop
                    END IF
                CASE CHR$(8) ' backspace
                    IF LEN(PassWord$) THEN
                        LOCATE 22, POS(0) - 1, 1 ' position cursor
                        PRINT " ";
                        LOCATE 22, POS(0) - 1, 1 ' position cursor
                        PassWord$ = LEFT$(PassWord$, LEN(PassWord$) - 1) ' truncate input one character
                    END IF
                CASE "a" TO "z", "A" TO "Z", " " ' check valid character
                    IF LEN(PassWord$) < 20 THEN ' verify length of string
                        PassWord$ = PassWord$ + X$ ' append to PassWord$
                        PRINT ".";
                    END IF
            END SELECT
        LOOP
        CodeName$ = CodeName$ + STRING$(20 - LEN(CodeName$), 32) ' pad with spaces to the right
        CodeName$ = LCASE$(CodeName$) ' convert codename to lowercase
        PassWord$ = PassWord$ + STRING$(20 - LEN(PassWord$), 32) ' pad with spaces to the right
        PassWord$ = LCASE$(PassWord$) ' convert password to lowercase
        ' find existing codename in users data file, or new user
        login = 0
        new.user = -1
        ' get each users data file record one by one
        FOR User.Index = 1 TO LOF(1) / LEN(UserRecord)
            ' get the data file record
            GET 1, User.Index, UserRecord
            ' get the decrypted codename
            xcrypt$ = fndecrypt$(UserRecord.CodeName)
            ' check for an existing codename
            IF CodeName$ = xcrypt$ THEN
                ' get the decrypted password
                ycrypt$ = fndecrypt$(UserRecord.PassWord)
                ' if existing users password does not match,
                '   set flag for no login.
                IF PassWord$ = ycrypt$ THEN
                    login = -1
                END IF
                ' not a new user if codename found
                new.user = 0
                EXIT FOR ' exit the loop after finding and checking password
            END IF
        NEXT
        ' the codename was not found.. a new user.
        IF new.user THEN
            ' check if max users
            IF User.Index > 999 THEN ' compare largest
                login = 0 ' reset login
                EXIT DO ' exit login loop
            END IF ' end check max value
            ' reset all variables for new user
            dlvl# = 1#
            CALL PutUserRecordStats1(1, 1)
            CALL PutUserRecordStats1(2, 1)
            CALL PutUserRecordStats1(3, 64)
            CALL PutUserRecordStats1(4, 15)
            CALL PutUserRecordStats1(5, 0)
            CALL PutUserRecordStats1(6, 15)
            FOR i = 1 TO 12
                CALL PutUserRecordStats2(i, 0)
            NEXT
            FOR i = 13 TO 16
                CALL PutUserRecordStats2(i, 10)
            NEXT
            CALL PutUserRecordStats2(17, 0)
            FOR i = 1 TO 10
                CALL PutUserRecordWand(i, 0)
                CALL PutUserRecordPotion(i, 0)
                CALL PutUserRecordStaff(i, 0)
                CALL PutUserRecordRing(i, 0)
                CALL PutUserRecordGlobe(i, 0)
            NEXT
            FOR i = 1 TO 40
                CALL PutUserRecordSpell(i, 0)
            NEXT
            COLOR 14, 1 ' blue background
            CLS ' get the new users statistics
            LOCATE 11, 28, 0 ' position cursor
            COLOR 12 ' change color
            PRINT "Str Int Wis Con Dex Cha"; ' display statistic header
            LOCATE 13, 23, 0 ' position cursor
            COLOR 13 ' change color
            PRINT "Rolling Character Stats(5 to keep)";
            ' loop to randomly select 6 statistics,
            '   and exit loop after user presses enter
            DO
                LOCATE 12, 29, 0 ' position cursor
                COLOR 10 ' change color
                FOR x = 1 TO 6
                    m = INT(RND * 9 + 10)
                    CALL PutUserRecordStats2(x, m) ' random statistic
                    PRINT RIGHT$(STR$(m), 2) + "  ";
                NEXT
                DO ' get a keystroke
                    GOSUB Get.Keystroke
                LOOP UNTIL x > 0
                ' exit statistics loop when enter/5 is pressed
                IF x = 5 THEN
                    LOCATE 15, 12, 0 ' position cursor
                    COLOR 15 ' change color
                    PRINT "Press 0 for help when playing, or Q to quit. Now hit a key.";
                    COLOR 7 ' change color
                    GOSUB Get.Key.Press ' get a single keystroke
                    GOSUB Put.User.File ' store users record to data file
                    login = -1 ' set logged in variable
                    EXIT DO
                END IF
            LOOP
        END IF
        ' if user has logged in (by rolling new character statistics, or
        ' by matching password to existing codename)..
        IF login THEN
            GOSUB OpenData.File ' open the users data file
            GET 1, User.Index, UserRecord ' read the user record from data file
            GOSUB Change.Level ' ask user to select level to start on
            DO ' main input loop
                GOSUB Status.Line ' display statistics on line 24
                GOSUB Get.Keystroke ' read a key press
                ' determine key pressed
                SELECT CASE LEN(X$)
                    CASE 1
                        SELECT CASE X$
                            CASE "1" ' attack nearby monsters
                                GOSUB Attack.Nearby.Monsters
                                GOSUB Monster.Attack
                                GOSUB Encounter.Cells
                            CASE "2" ' move player down
                                GOSUB Move.Player.Down
                            CASE "3" ' get user to enter a spell chant to cast
                                GOSUB Cast.Chant
                            CASE "4" ' move player left
                                GOSUB Move.Player.Left
                            CASE "5", CHR$(13) ' enter key/null action/update hits
                                GOSUB Update.Hits
                            CASE "6" ' move player right
                                GOSUB Move.Player.Right
                            CASE "7" ' move player away from nearby monster
                                GOSUB Evade.Monster
                            CASE "8" ' move player up
                                GOSUB Move.Player.Up
                            CASE "9" ' ignore key/null action/update hits
                                GOSUB Update.Hits
                            CASE "0" ' help screen menu
                                GOSUB Display.Help
                            CASE "+" ' display dungeon information
                                GOSUB Display.Info
                            CASE "." ' display extended status line 25
                                GOSUB Extended.Status.Line
                            CASE "-" ' learned spells display screen
                                GOSUB Learned.Spells
                            CASE "\" ' statistic trading menu
                                GOSUB Trade.Stats
                            CASE ";" ' inventory purchasing menu
                                GOSUB Buy.Inventory
                            CASE "=" ' reselect dungeon level number
                                GOSUB Store.Dungeon.Level ' store dungeon screen
                                GOSUB Change.Level
                            CASE "/" ' inventory display screen
                                GOSUB Inventory
                            CASE "'" ' use magic item menu
                                GOSUB Use.Item
                                GOSUB Cell.Action ' compute nearby cells
                            CASE "_" ' edit user
                                GOSUB Edit.User
                            CASE "`" ' user list
                                GOSUB User.List
                            CASE "Q", "q" ' quit the dungeon
                                GOSUB Store.Dungeon.Level ' store dungeon screen
                                GOSUB Put.User.File
                                dlvl# = 0# ' dungeon level zero/exit flag
                                login = 0 ' set logged in variable false
                            CASE CHR$(1) TO CHR$(10)
                                i = ASC(X$) ' use a staff number 1 to 10
                                GOSUB Use.Staff
                            CASE CHR$(17) TO CHR$(26)
                                i = ASC(X$) - 16 ' use a ring number 1 to 10
                                GOSUB Use.Ring
                            CASE ELSE
                                COLOR 13 ' change color
                                a$ = "Type 0 for help, or Q to quit.." ' make display message
                                GOSUB Display.Message ' output message
                        END SELECT
                    CASE 2
                        sp = ASC(MID$(X$, 2))
                        spn = 0
                        SELECT CASE sp
                            ' use a wand extended keys 1 to 10
                            CASE 30
                                i = 1
                                GOSUB Use.Wand
                            CASE 48
                                i = 2
                                GOSUB Use.Wand
                            CASE 46
                                i = 3
                                GOSUB Use.Wand
                            CASE 32
                                i = 4
                                GOSUB Use.Wand
                            CASE 18
                                i = 5
                                GOSUB Use.Wand
                            CASE 33
                                i = 6
                                GOSUB Use.Wand
                            CASE 34
                                i = 7
                                GOSUB Use.Wand
                            CASE 35
                                i = 8
                                GOSUB Use.Wand
                            CASE 23
                                i = 9
                                GOSUB Use.Wand
                            CASE 36
                                i = 10
                                GOSUB Use.Wand
                                ' use a potion extended keys 1 to 10
                            CASE 16
                                i = 1
                                GOSUB Use.Potion
                            CASE 19
                                i = 2
                                GOSUB Use.Potion
                            CASE 31
                                i = 3
                                GOSUB Use.Potion
                            CASE 20
                                i = 4
                                GOSUB Use.Potion
                            CASE 22
                                i = 5
                                GOSUB Use.Potion
                            CASE 47
                                i = 6
                                GOSUB Use.Potion
                            CASE 17
                                i = 7
                                GOSUB Use.Potion
                            CASE 45
                                i = 8
                                GOSUB Use.Potion
                            CASE 21
                                i = 9
                                GOSUB Use.Potion
                            CASE 44
                                i = 10
                                GOSUB Use.Potion
                                ' F1 to F10, cast spell type 1
                            CASE 59 TO 68
                                spn = 1
                                sp = sp - 58
                                GOSUB Cast.Spell.Type1
                                ' Control-F1 to F10, cast spell type 2
                            CASE 94 TO 103
                                spn = 2
                                sp = sp - 93
                                GOSUB Cast.Spell.Type2
                                ' Shift-F1 to F10, cast spell type 3
                            CASE 84 TO 93
                                spn = 3
                                sp = sp - 83
                                GOSUB Cast.Spell.Type3
                                ' Alt-F1 to F10, cast spell type 4
                            CASE 104 TO 113
                                spn = 4
                                sp = sp - 103
                                GOSUB Cast.Spell.Type4
                                ' extended keys equal to numeric keypad entry
                            CASE 15 ' inventory purchasing menu
                                GOSUB Buy.Inventory
                            CASE 75 ' 4, move player left
                                GOSUB Move.Player.Left
                            CASE 76 ' 5, enter/accept
                                GOSUB Update.Hits
                            CASE 77 ' 6, move player right
                                GOSUB Move.Player.Right
                            CASE 71 ' 7, evade monster
                                GOSUB Evade.Monster
                            CASE 72 ' 8, move player up
                                GOSUB Move.Player.Up
                            CASE 73 ' 9, ignore key/null action/update hits
                                GOSUB Update.Hits
                            CASE 79 ' 1, attack nearby monsters
                                GOSUB Attack.Nearby.Monsters
                                GOSUB Monster.Attack
                                GOSUB Encounter.Cells
                            CASE 80 ' 2, move player down
                                GOSUB Move.Player.Down
                            CASE 81 ' 3, cast spell
                                GOSUB Cast.Chant
                            CASE 82 ' help screen menu
                                GOSUB Display.Help
                            CASE 83 ' display extended status line 25
                                GOSUB Extended.Status.Line
                            CASE 115 ' inventory display screen
                                GOSUB Inventory
                            CASE 116 ' display dungeon information
                                GOSUB Display.Info
                            CASE 117 ' reselect dungeon level number
                                GOSUB Store.Dungeon.Level ' store dungeon screen
                                GOSUB Change.Level
                            CASE 118 ' learned spells display screen
                                GOSUB Learned.Spells
                            CASE 119 ' statistic trading menu
                                GOSUB Trade.Stats
                                ' Alt-0 to Alt-9, globe power keys
                            CASE 120 TO 129
                                i = ASC(MID$(X$, 2)) - 119
                                GOSUB Use.Globe
                            CASE 130 ' Alt-- edit user
                                GOSUB Edit.User
                            CASE 131 ' Alt-= user list
                                GOSUB User.List
                            CASE 132 ' use magic item menu
                                GOSUB Use.Item
                                GOSUB Cell.Action ' compute nearby cells
                            CASE 133 ' F11
                                s = -1 ' display all symbols
                                GOSUB Display.Symbols
                            CASE 134 ' F12
                                GOSUB Restore.Symbols
                            CASE 135 ' Shift-F11
                                s = 1 ' display stair symbols
                                GOSUB Display.Symbols
                            CASE 136 ' Shift-F12
                                s = 2 ' display wand symbols
                                GOSUB Display.Symbols
                            CASE 137 ' Ctrl-F11
                                s = 3 ' display potion symbols
                                GOSUB Display.Symbols
                            CASE 138 ' Ctrl-F12
                                s = 4 ' display staff symbols
                                GOSUB Display.Symbols
                            CASE 139 ' Alt-F11
                                s = 5 ' display ring symbols
                                GOSUB Display.Symbols
                            CASE 140 ' Alt-F12
                                s = 6 ' display chest symbols
                                GOSUB Display.Symbols
                            CASE 148 ' Ctrl-Tab
                                GOSUB Display.Dungeon.Symbols
                            CASE 152 ' Alt-Up
                                GOSUB Map.Edit
                            CASE 155 ' Alt-Left
                                GOSUB Make.Globe
                            CASE 157 ' Alt-Right
                                GOSUB Display.Help2
                            CASE 147 ' Ctrl-Delete
                                GOSUB DOS.Shell
                        END SELECT
                END SELECT
            LOOP UNTIL login = 0
        END IF
    LOOP UNTIL dlvl# = 0#
LOOP
COLOR 7, 0 ' reset color
CLS ' clear screen
END ' terminate program

' display message containing the spell cast
Cast.Spell.Message:
RESTORE SpellData ' reset the data pointer to the spell names
' loop to the spell number (sp=spell number, spn=spell type 1 to 4)
FOR i = 1 TO sp + (spn - 1) * 10
    READ a$, c$ ' read the spell chant and name
NEXT
COLOR 15 ' change color
a$ = "You cast a " + c$ + " spell!" ' make display message
GOSUB Display.Message ' output message
COLOR 7 ' change color
RETURN ' exit routine

Move.Player.Down:
d = GetDungeonRecordLevelData(xco + 1, yco)
' if the player tries to move through a dungeon
' wall and is not in passwall mode then exit
IF d = 176 THEN
    IF pwl = 0 THEN
        RETURN ' exit routine
    END IF
END IF
' if the player moves to a cell occupied by a monster,
' use the move as an attack on the monster
IF d = 35 THEN
    mxco = xco + 1 ' set monster coordinates
    myco = yco
    GOSUB Attack.Monster
    GOSUB Monster.Attack
    GOSUB Encounter.Cells
    RETURN ' exit routine
END IF
' otherwise move player down
IF xco < 23 THEN
    xco = xco + 1 ' update player x-coordinate
    GOSUB Update.Player ' update nearby cell actions
END IF
RETURN ' exit routine

Move.Player.Left:
d = GetDungeonRecordLevelData(xco, yco - 1)
' if the player tries to move through a dungeon
' wall and is not in passwall mode then exit
IF d = 176 THEN
    IF pwl = 0 THEN
        RETURN ' exit routine
    END IF
END IF
' if the player moves to a cell occupied by a monster,
' use the move as an attack on the monster
IF d = 35 THEN
    mxco = xco ' set monster coordinates
    myco = yco - 1
    GOSUB Attack.Monster
    GOSUB Monster.Attack
    GOSUB Encounter.Cells
    RETURN ' exit routine
END IF
' otherwise move player left
IF yco > 1 THEN
    yco = yco - 1 ' update player y-coordinate
    GOSUB Update.Player ' update nearby cell actions
END IF
RETURN ' exit routine

Move.Player.Right:
d = GetDungeonRecordLevelData(xco, yco + 1)
' if the player tries to move through a dungeon
' wall and is not in passwall mode then exit
IF d = 176 THEN
    IF pwl = 0 THEN
        RETURN ' exit routine
    END IF
END IF
' if the player moves to a cell occupied by a monster,
' use the move as an attack on the monster
IF d = 35 THEN
    mxco = xco ' set monster coordinates
    myco = yco + 1
    GOSUB Attack.Monster
    GOSUB Monster.Attack
    GOSUB Encounter.Cells
    RETURN ' exit routine
END IF
' otherwise move player right
IF yco < 79 THEN
    yco = yco + 1 ' update player y-coordinate
    GOSUB Update.Player ' update nearby cell actions
END IF
RETURN ' exit routine

Move.Player.Up:
d = GetDungeonRecordLevelData(xco - 1, yco)
' if the player tries to move through a dungeon
' wall and is not in passwall mode then exit
IF d = 176 THEN
    IF pwl = 0 THEN
        RETURN ' exit routine
    END IF
END IF
' if the player moves to a cell occupied by a monster,
' use the move as an attack on the monster
IF d = 35 THEN
    mxco = xco - 1 ' set monster coordinates
    myco = yco
    GOSUB Attack.Monster
    GOSUB Monster.Attack
    GOSUB Encounter.Cells
    RETURN ' exit routine
END IF
' otherwise move player up
IF xco > 1 THEN
    xco = xco - 1 ' update player x-coordinate
    GOSUB Update.Player ' update nearby cell actions
END IF
RETURN ' exit routine

' actions for cells next to a player when he moves
Update.Player:
' reset any player increment counters
GOSUB Update.Player.Counters
' surrounding cell monster deaths
IF gd THEN
    GOSUB Cell.Monster.Attack ' all nearby monsters die
END IF
' further cell action routine
GOSUB Cell.Action ' compute nearby cells
' player moves to an occupied cell
SELECT CASE GetDungeonRecordLevelData(xco, yco)
    CASE 33 ' teleporter (!)
        GOSUB Teleporter
    CASE 34 ' ring (")
        GOSUB Ring
    CASE 36 ' pile of coins ($)
        GOSUB Find.Coin.Pile
    CASE 37 ' staff (%)
        GOSUB Staff
    CASE 38 ' coins (&)
        GOSUB Coins
    CASE 42 ' gems (*)
        GOSUB Gems
    CASE 60 ' staircase down (<)
        GOSUB Stairs.Down
    CASE 62 ' staircase up (>)
        GOSUB Stairs.Up
    CASE 63 ' temple (?)
        GOSUB Temple
    CASE 64 ' potion (@)
        GOSUB Potion
    CASE 94 ' trap (^)
        GOSUB Trap
    CASE 111 ' pit (o)
        GOSUB Pit
    CASE 124 ' wand (|)
        GOSUB Wand
    CASE 126 ' scroll (~)
        GOSUB Scroll
    CASE 254 ' chest symbol
        GOSUB Find.Chest
END SELECT
' check if all treasure have been found
GOSUB Check.Clear.Dungeon
' calculate updated player combat hits
GOSUB Update.Hits
RETURN ' exit routine

' check if all treasure have been found
Check.Clear.Dungeon:
IF GetDungeonRecordStats(5) = 1 THEN ' all monsters in dungeon level conquered
    IF GetDungeonRecordStats(4) = 0 THEN ' all treasure found flag not set
        FOR x = 8 TO 32 STEP 2 ' loop through dungeon treasure flags
            IF GetDungeonRecordStats(x) = 0 THEN ' check any treasure not found
                RETURN ' exit routine
            END IF
        NEXT
        CALL PutDungeonRecordStats(4, 1) ' set all dungeon level treasure found flag
        d$ = "treasure"
        GOSUB Treasure.Bonus ' bonus for all treasure
        COLOR 13 ' change color
        a$ = "The dungeon becomes replenished!" ' make display message
        GOSUB Display.Message ' output message
        GOSUB Zero.Dungeon ' first remove all dungeon treasure,
        GOSUB Replenish.Dungeon ' restock the dungeon level
    END IF
END IF
RETURN ' exit routine

' calculate award amount of gold for a bonus
Treasure.Bonus:
' calculate gold award
ex# = (RND * dlvl# * 2 + 1) * GetUserRecordStats1(2) ^ 2 / 2
ex# = ex# + dlvl# * 2 + RND * GetUserRecordStats1(2) * 50
ex# = INT(ex#)
z1# = GetUserRecordStats1(5)
z1# = z1# + ex# ' increment players gold
CALL PutUserRecordStats1(5, z1#) ' increment players gold
COLOR 14 ' change color
a$ = "You found all the " + d$ + " on this level! You gain" + STR$(ex#) + " gold!"
GOSUB Display.Message ' output message
RETURN ' exit routine

' calculate when to update players combat hits
Update.Hits:
cnt = cnt + 1 ' increment counter each input loop pass
' return if counter below 10
IF cnt < 10 THEN
    RETURN ' exit routine
END IF
cnt = 0 ' reset counter
GOSUB Increment.Hits ' update players combat hits
RETURN ' exit routine

' calculate players incremented combat hits
Increment.Hits:
x = GetUserRecordStats1(6) ' maximum hit points
y = GetUserRecordStats1(4) ' user's hit points
z = GetUserRecordStats2(4) ' constitution
IF x > y THEN ' increment plus constitution
    y = y + z
END IF
IF y > x THEN ' reset to maximum points allowed if over maximum
    y = x
END IF
CALL PutUserRecordStats1(4, y)
RETURN ' exit routine

' move player up stairs
Stairs.Up:
COLOR 14 ' change color
a$ = "You find a staircase!(5 to climb up)"
GOSUB Display.Message ' output message
GOSUB Get.Keystroke ' read a key press
' presses enter/5 to climb up
IF x = 5 THEN ' check numeric keystroke
    GOSUB Dungeon.Level.Up ' move player up one dungeon level
ELSE ' check keystroke
    Buf$ = X$ ' store typeahead buffer
END IF ' end check keystroke
RETURN ' exit routine

' move player down stairs
Stairs.Down:
COLOR 14 ' change color
a$ = "You find a staircase!(5 to climb down)"
GOSUB Display.Message ' output message
GOSUB Get.Keystroke ' read a key press
' presses enter/5 to climb down
IF x = 5 THEN ' check numeric keystroke
    GOSUB Dungeon.Level.Down ' move player down one dungeon level
ELSE ' check keystroke
    Buf$ = X$ ' store typeahead buffer
END IF ' end check keystroke
RETURN ' exit routine

' redisplay staircase up symbol,
' then enter routine to next dungeon level up
Dungeon.Level.Up:
COLOR 15 ' change color
LOCATE xco, yco, 0 ' position cursor
PRINT ">"; ' make stair symbol
 
' move player up one dungeon level
Dungeon.Up:
GOSUB Clear.Status.Line2 ' clear status line area
GOSUB Clear.Monsters ' clear monster array
GOSUB Store.Dungeon ' record the dungeon statistics and screen
GOSUB Decrement.Dungeon.Level ' subtract one from dungeon level variable
IF dlvl# > 0# THEN ' still in dungeon
    COLOR 10 ' change color
    a$ = "You return to Dungeon level" + STR$(dlvl#) + "!" ' prompt
    GOSUB Display.Message ' display next dungeon level reached message
    SLEEP 3 ' wait
    GOSUB Next.Dungeon.Level ' load dungeon level statistics
    GOSUB Cell.Action ' compute nearby cells
END IF
RETURN ' exit routine

Decrement.Dungeon.Level:
GOSUB Clear.Monsters ' clear monster array
dlvl# = dlvl# - 1# ' decrement players dungeon level counter
IF dlvl# <= 0# THEN ' if the player reaches dungeon level zero, exit game
    login = 0 ' player leaves dungeon
    dlvl# = 0# ' reset dungeon level counter to zero
    COLOR 14 ' change color
    a$ = "You escaped the Dungeon!"
    GOSUB Display.Message ' output message
    SLEEP 3 ' wait
    GOSUB Put.User.File ' store players dungeon level statistics
END IF
RETURN ' exit routine

' redisplay staircase down symbol,
' then enter routine to next dungeon level down
Dungeon.Level.Down:
COLOR 15 ' change color
LOCATE xco, yco, 0 ' position cursor
PRINT "<"; ' make stair symbol
 
' move player down one dungeon level
Dungeon.Down:
GOSUB Clear.Status.Line2 ' clear status line area
GOSUB Clear.Monsters ' clear monster array
GOSUB Store.Dungeon ' record the dungeon statistics and screen
GOSUB Increment.Dungeon.Level
RETURN ' exit routine

Increment.Dungeon.Level:
GOSUB Clear.Monsters ' clear monster array
dlvl# = dlvl# + 1# ' increment players dungeon level counter
' check if player has entered a dungeon level lower than his last recorded
IF dlvl# > GetUserRecordStats1(1) THEN ' if lower, then
    CALL PutUserRecordStats1(1, dlvl#) ' set player's last dungeon level to current
END IF
COLOR 11 ' change color
a$ = "You made it to Dungeon level" + STR$(dlvl#) + "!" ' message
GOSUB Display.Message ' display next dungeon level reached message
SLEEP 3 ' wait
GOSUB Next.Dungeon.Level ' load dungeon level statistics
GOSUB Cell.Action ' compute nearby cells
RETURN ' exit routine

' loads dungeon level statistics
Next.Dungeon.Level:
GOSUB Clear.Monsters ' clear monster array
xdlvl# = dlvl# ' store dungeon level
WHILE xdlvl# > LOF(2) / LEN(DungeonRecord) ' new dungeon record,
    dlvl# = LOF(2) / LEN(DungeonRecord) + 1# ' reset dungeon level
    GOSUB Make.Dungeon ' create the dungeon level
    GOSUB Store.Dungeon ' store new dungeon level
WEND
GOSUB Load.Dungeon.Level ' get the dungeon level from data file on disk
RETURN ' exit routine

' clears status line before storing dungeon level
Store.Dungeon.Level:
GOSUB Clear.Status.Line ' clear status line

' stores current dungeon level screen
Store.Dungeon:
GOSUB Restore.Symbols ' clear hilighted symbols
' store the dungeon screen
FOR i = 1 TO 24 ' rows by
    FOR j = 1 TO 80 ' columns
        ' move character at screen location into data file array
        CALL PutDungeonRecordScreenData(i, j, SCREEN(i, j))
    NEXT
NEXT
CALL PutDungeonRecordStats(2, xco) ' player x-coordinate
CALL PutDungeonRecordStats(3, yco) ' player y-coordinate
' calculate the number of levels stored, the length of the data file divided
' by the length of the structure, check if recording the dungeon level would
' access any record number out of range,
IF dlvl# <= LOF(2) / LEN(DungeonRecord) + 1 THEN ' check valid level
    PUT 2, dlvl#, DungeonRecord ' store the dungeon level structure into record
END IF
RETURN ' exit routine

' selects specific dungeon level previously recorded
Change.Level:
IF LOF(2) / LEN(DungeonRecord) = 0 THEN ' dungeon records in the data file
    dlvl# = 1# ' reset the dungeon level counter to one
    GOSUB Clear.Monsters ' clear monster array
    GOSUB Make.Dungeon ' create the first dungeon level
    GOSUB Store.Dungeon ' store new dungeon level
    GOSUB Cell.Action ' compute nearby cells
    RETURN ' exit routine
END IF
IF LOF(2) / LEN(DungeonRecord) = 1 THEN ' dungeon records in the data file
    COLOR 14, 1 ' blue background
    CLS ' clear screen
    LOCATE 12, 24, 0 ' position cursor
    COLOR 13 ' change color
    PRINT "You will start at dungeon level 1"; ' display dungeon level
    COLOR 7 ' change color
    SLEEP 3 ' wait
    ll = 1 ' reset cell displays cell display amount
    dlvl# = 1# ' set dungeon level variable
    GOSUB Load.Dungeon.Level ' load the dungeon statistics and screen
    GOSUB Cell.Action ' compute nearby cells
    RETURN ' exit routine
END IF
DO ' loop to get dungeon level input into l$
    ' calculate the number of levels stored, the length of the data file
    ' divided by the length of the structure, if there are no
    COLOR 14, 1 ' blue background
    CLS ' clear screen
    i$ = "You made it as far as level" + STR$(LOF(2) / LEN(DungeonRecord)) + "!"
    i = 40 - INT(LEN(i$) / 2) ' calculate prompt center
    LOCATE 10, i, 0 ' position cursor
    COLOR 14 ' change color
    PRINT "You made it as far as level"; STR$(LOF(2) / LEN(DungeonRecord)); "!";
    LOCATE 11, 23, 0 ' position cursor
    COLOR 10 ' change color
    PRINT "Which level will you start at now?" ' display input prompt
    COLOR 15 ' change color
    l$ = "" ' clear input
    LOCATE 13, 30, 1 ' position cursor
    ' loop to retrieve l$ input character by character.
    '   backspace, enter, and characters 0 to 9 are accepted,
    '   or nul for enter with no characters input. Maximum 8 characters.
    DO
        X$ = "" ' get keystroke
        WHILE X$ = "" ' wait for input
            _LIMIT 100
            X$ = INKEY$ ' get keystroke
        WEND ' end wait
        SELECT CASE X$ ' select user input
            CASE CHR$(13) ' enter key
                IF LEN(l$) THEN
                    EXIT DO ' exit loop
                END IF
            CASE CHR$(8) ' backspace
                IF LEN(l$) > 0 THEN
                    LOCATE CSRLIN, POS(0) - 1, 1 ' position cursor
                    PRINT " ";
                    LOCATE CSRLIN, POS(0) - 1, 1 ' position cursor
                    l$ = LEFT$(l$, LEN(l$) - 1) ' truncate input one character
                END IF
            CASE "0" TO "9" ' check valid character
                IF LEN(l$) < 8 THEN ' verify length of string
                    l$ = l$ + X$ ' append to l$
                    PRINT X$;
                END IF
        END SELECT
    LOOP
    dlvl# = INT(VAL(l$)) ' convert l$ to dungeon level variable
    ' if the dungeon level is within range of the records in the data file,
    IF dlvl# > 0# AND dlvl# <= LOF(2) / LEN(DungeonRecord) THEN
        ll = 1 ' reset cell displays cell display amount
        GOSUB Load.Dungeon.Level ' load the dungeon statistics and screen
        GOSUB Cell.Action ' compute nearby cells
        EXIT DO ' exit the loop
    END IF
    ' otherwise, display message
    i$ = "You have only" + STR$(LOF(2) / LEN(DungeonRecord)) + " to choose from.."
    i = 40 - INT(LEN(i$) / 2 + 1) ' calculate prompt center
    LOCATE 15, i, 0 ' position cursor
    COLOR 13 ' change color
    PRINT "You have only"; STR$(LOF(2) / LEN(DungeonRecord));
    PRINT " levels to choose from!"
    COLOR 7 ' change color
    LOCATE 13, 30, 0 ' position cursor
    PRINT SPACE$(8);
    GOSUB Key.Press ' get a keystroke
LOOP
RETURN ' exit routine

' redisplay dungeon dot, then clear status line area
Clear.Status.Line:
COLOR 7 ' change color
LOCATE xco, yco, 0 ' position cursor
PRINT "."; ' dungeon dot
 
' clear status line area
Clear.Status.Line2:
COLOR 7 ' change color
LOCATE 24, 1, 0 ' position cursor
PRINT SPACE$(79);
LOCATE 25, 1, 0 ' position cursor
PRINT SPACE$(79);
RETURN ' exit routine

' load dungeon level from data file on disk
Load.Dungeon.Level:
' if there are less dungeon record structures recorded in the data file
' than the dungeon level requested to read from,
IF LOF(2) / LEN(DungeonRecord) < dlvl# THEN
    GOSUB Make.Dungeon ' then the dungeon level is created,
    GOSUB Store.Dungeon ' store new dungeon level
    GOSUB Cell.Action ' compute nearby cells
    RETURN ' exit routine
END IF
' otherwise, read the dungeon record structure

' restores dungeon screen
Restore.Screen:
' read the dungeon record structure
GET 2, dlvl#, DungeonRecord
COLOR 14, 1 ' blue background
CLS ' clear the screen
FOR i = 1 TO 24 ' rows by
    FOR j = 1 TO 79 ' columns
        LOCATE i, j, 0 ' locate at dungeon xy
        ' display character in dungeon screen array on screen
        x = GetDungeonRecordScreenData(i, j) ' get character
        IF x = 32 THEN ' blank space
            LOCATE CSRLIN, POS(0) + 1, 0 ' increment cursor y position
        ELSE
            IF x = 46 OR x = 176 THEN ' color white for
                k = GetDungeonRecordLevelData(i, j) ' ascii of dungeon coordinate
                IF k > 21 AND k < 26 THEN ' compare dot
                    COLOR 15 ' dungeon room border
                ELSE ' color dot
                    COLOR 7 ' dungeon dot/wall
                END IF
            ELSE ' color highlight white for
                COLOR 15 ' other symbols
            END IF
            PRINT CHR$(x); ' print ascii character
        END IF
    NEXT
    COLOR 7 ' display rightmost column
    k = GetDungeonRecordScreenData(i, j) ' dungeon wall
    IF k = 176 THEN ' dungeon wall
        PRINT CHR$(176); ' print ascii character
    END IF
NEXT
xco = GetDungeonRecordStats(2) ' player x-coordinate
yco = GetDungeonRecordStats(3) ' player y-coordinate
COLOR 15 ' display player's character
LOCATE xco, yco, 0 ' in highlight white
PRINT "X";
RETURN ' exit from subroutine

' display all dungeon symbols
Display.Symbols:
COLOR 14, 1 ' yellow on blue background
FOR i = 1 TO 24 ' rows by
    FOR j = 1 TO 79 ' columns
        ' display character in dungeon screen array on screen
        x = GetDungeonRecordLevelData(i, j) ' get character
        SELECT CASE x
            CASE 32, 46, 176, 22 TO 25
                ' nul
            CASE ELSE
                t = 0 ' reset display symbol flag
                SELECT CASE s ' determine type of symbol to display
                    CASE -1 ' display all symbols
                        t = -1 ' set display flag
                    CASE 1
                        IF x = 60 OR x = 62 THEN ' check symbol type
                            t = -1 ' set display flag
                        END IF
                    CASE 2
                        IF x = 124 THEN ' check symbol type
                            t = -1 ' set display flag
                        END IF
                    CASE 3
                        IF x = 64 THEN ' check symbol type
                            t = -1 ' set display flag
                        END IF
                    CASE 4
                        IF x = 37 THEN ' check symbol type
                            t = -1 ' set display flag
                        END IF
                    CASE 5
                        IF x = 34 THEN ' check symbol type
                            t = -1 ' set display flag
                        END IF
                    CASE 6
                        IF x = 254 THEN ' check symbol type
                            t = -1 ' set display flag
                        END IF
                END SELECT
                IF t THEN ' check flag to display symbol
                    f = 0 ' reset cell flag
                    IF i = xco AND j = yco THEN ' check player symbol
                        f = -1 ' set cell flag
                    END IF
                    IF f = 0 THEN ' check cell flag
                        k = SCREEN(i, j) ' get screen character
                        IF k = 32 THEN ' check empty screen character
                            LOCATE i, j, 0 ' locate at dungeon xy
                            PRINT CHR$(x); ' print ascii character
                        END IF
                    END IF
                END IF
        END SELECT
    NEXT
NEXT
RETURN

' clear all yellow dungeon symbols
Restore.Symbols:
COLOR 14, 1 ' yellow on blue background
FOR i = 1 TO 24 ' rows by
    FOR j = 1 TO 79 ' columns
        ' remove character in dungeon screen array on screen
        x = GetDungeonRecordLevelData(i, j) ' get character
        SELECT CASE x
            CASE 32, 46, 176, 22 TO 25
                ' nul
            CASE ELSE
                f = 0 ' reset cell flag
                IF i = xco AND j = yco THEN ' check player symbol
                    f = -1 ' set cell flag
                END IF
                IF f = 0 THEN ' check cell flag
                    k = SCREEN(i, j, 1) ' get screen character color
                    IF k = 30 THEN ' check color of symbol
                        LOCATE i, j, 0 ' locate at dungeon xy
                        PRINT CHR$(32); ' print ascii character
                    END IF
                END IF
        END SELECT
    NEXT
NEXT
RETURN

' displays and stores entire dungeon screen
Display.Dungeon.Symbols:
COLOR 14, 1 ' blue background
FOR i = 1 TO 24 ' rows by
    FOR j = 1 TO 79 ' columns
        LOCATE i, j, 0 ' locate at dungeon xy
        ' display character in dungeon screen array on screen
        x = GetDungeonRecordLevelData(i, j) ' get character
        IF x = 32 THEN ' blank space
            LOCATE CSRLIN, POS(0) + 1, 0 ' increment cursor y position
        ELSE
            IF x = 46 OR x = 176 THEN ' color white for dot or wall
                COLOR 7 ' color white
            ELSE ' color for any other symbol
                COLOR 15 ' color highlight white
                IF x > 21 AND x < 26 THEN ' compare room border
                    x = 176 ' reset character to wall
                END IF
            END IF
            PRINT CHR$(x); ' print ascii character
        END IF
    NEXT
    COLOR 7 ' display rightmost column
    k = GetDungeonRecordScreenData(i, j) ' dungeon wall
    IF k = 176 THEN ' dungeon wall
        PRINT CHR$(176); ' print ascii character
    END IF
NEXT
xco = GetDungeonRecordStats(2) ' player x-coordinate
yco = GetDungeonRecordStats(3) ' player y-coordinate
COLOR 15 ' display player's character
LOCATE xco, yco, 0 ' in highlight white
PRINT "X";
RETURN ' exit from subroutine

' enter a teleporter
Teleporter:
CALL PutDungeonRecordLevelData(xco, yco, 46) ' reset cell with dungeon dot
COLOR 14 ' change color
a$ = "You find a teleporter!(5 to enter)"
GOSUB Display.Message ' output message
GOSUB Get.Keystroke ' read a key press
' presses enter/5 to enter teleporter
IF x = 5 THEN ' check numeric keystroke
    COLOR 7 ' change color
    LOCATE xco, yco, 0 ' position cursor
    PRINT "."; ' replace character X with dungeon dot
    ' teleport player based on wisdom
    IF INT(RND * 8 + 8) > GetUserRecordStats2(3) THEN
        GOSUB Teleport
    ELSE
        ' otherwise, teleport one dungeon level
        IF RND > .5 THEN
            GOSUB Dungeon.Up ' up random 50/50
        ELSE
            GOSUB Dungeon.Down ' or down random 50/50
        END IF
    END IF
ELSE ' check keystroke
    Buf$ = X$ ' store typeahead buffer
END IF ' end check keystroke
CALL SetDungeonRecordStats(15) ' decrement teleporters
RETURN ' exit routine

' enter temple
Temple:
CALL PutDungeonRecordLevelData(xco, yco, 46) ' reset cell with dungeon dot
COLOR 14 ' change color
a$ = "You find a temple!(5 to pray, 7 to pillage)"
GOSUB Display.Message ' output message
GOSUB Get.Keystroke ' read a key press
' presses enter/5 to pray or evade/7 to pillage
IF x = 5 THEN
    ' reward player based on charisma
    IF INT(RND * 8 + 8) > GetUserRecordStats2(6) THEN ' low charisma
        COLOR 10 ' change color
        a$ = "Nothing happens.." ' make display message
        GOSUB Display.Message ' output message
    ELSE ' high charisma
        COLOR 10 ' change color
        a$ = "You are rewarded with great treasure!" ' make display message
        GOSUB Display.Message ' output message
        GOSUB Coin.Pile
    END IF
ELSE
    ' pillage/7 is pressed
    IF x = 7 THEN
        ' reward player based on charisma
        IF INT(RND * 8 + 8) > GetUserRecordStats2(6) THEN ' low charisma
            COLOR 10 ' change color
            a$ = "The Ghods are angered!" ' make display message
            mhp# = 0 ' set hits to zero for
            GOSUB Hit.Player ' routine to calculate hits on player
        ELSE ' high charisma
            COLOR 10 ' change color
            a$ = "You are rewarded with great treasure!" ' make display message
            GOSUB Display.Message ' output message
            GOSUB Chest ' opens a chest
        END IF
    ELSE
        Buf$ = X$ ' store typeahead buffer
    END IF
END IF
CALL SetDungeonRecordStats(17) ' decrement temples
RETURN ' exit routine

' find gems
Gems:
CALL PutDungeonRecordLevelData(xco, yco, 46) ' reset cell with dungeon dot
' select gem type
ran! = RND ' based on percentage
IF ran! > .85 THEN
    bonus = 8 ' multiplier
    b$ = "diamonds" ' gem name
ELSE
    IF ran! > .75 THEN
        bonus = 5 ' multiplier
        b$ = "emeralds" ' gem name
    ELSE
        IF ran! > .50 THEN
            bonus = 3 ' multiplier
            b$ = "saphires" ' gem name
        ELSE
            bonus = 2 ' multiplier
            b$ = "rubies" ' gem name
        END IF
    END IF
END IF
COLOR 14 ' change color
a$ = "You find some " + b$ + "!" ' make display message
GOSUB Display.Message ' output message
' calculate increment to players gold by bonus multiplier
temp# = RND * GetUserRecordStats1(2)
temp# = temp# + RND * GetUserRecordStats1(2) + 10
temp# = INT(temp# * bonus)
z1# = GetUserRecordStats1(5)
z1# = z1# + temp#
CALL PutUserRecordStats1(5, z1#)
z = GetDungeonRecordStats(19)
z = z - 1
CALL PutDungeonRecordStats(19, z) ' decrement gems
IF z = 0 THEN ' player has taken last gem
    IF GetDungeonRecordStats(20) = 0 THEN
        CALL PutDungeonRecordStats(20, 1) ' set all dungeon level gem flag
        d$ = "gems"
        GOSUB Treasure.Bonus ' bonus for last gem
    END IF
END IF
RETURN ' exit routine

' find coins
Coins:
CALL PutDungeonRecordLevelData(xco, yco, 46) ' reset cell with dungeon dot
bonus = INT(RND * dlvl# + 5) ' number of coins
COLOR 14 ' change color
a$ = "You find" + STR$(bonus) + " coins!" ' make display message
GOSUB Display.Message ' output message
' calculate increment to players gold by bonus multiplier
z1# = GetUserRecordStats1(5) + bonus * 100
CALL PutUserRecordStats1(5, z1#)
z = GetDungeonRecordStats(21)
z = z - 1
CALL PutDungeonRecordStats(21, z) ' decrement coins
IF z = 0 THEN ' player has taken last coin
    IF GetDungeonRecordStats(22) = 0 THEN
        CALL PutDungeonRecordStats(22, 1) ' set all dungeon level coins flag
        d$ = "coins"
        GOSUB Treasure.Bonus ' bonus for last coin
    END IF
END IF
RETURN ' exit routine

' find coin pile
Find.Coin.Pile:
CALL PutDungeonRecordLevelData(xco, yco, 46) ' reset cell to dungeon dot
GOSUB Coin.Pile ' get coins to add to gold
z = GetDungeonRecordStats(11)
z = z - 1
CALL PutDungeonRecordStats(11, z) ' decrement gold
IF z = 0 THEN ' player has the last gold
    IF GetDungeonRecordStats(12) = 0 THEN
        CALL PutDungeonRecordStats(12, 1) ' set all dungeon level gold flag
        d$ = "coin piles"
        GOSUB Treasure.Bonus ' bonus for all gold
    END IF
END IF
RETURN ' exit routine

' calculate coin pile
Coin.Pile:
' select coin type
ran! = RND ' based on percentage
IF ran! > .85 THEN
    bonus = 8 ' multiplier
    b$ = "platinum" ' coin name
ELSE
    IF ran! > .75 THEN
        bonus = 5 ' multiplier
        b$ = "gold" ' coin name
    ELSE
        IF ran! > .50 THEN
            bonus = 3 ' multiplier
            b$ = "silver" ' coin name
        ELSE
            bonus = 2 ' multiplier
            b$ = "copper" ' coin name
        END IF
    END IF
END IF
COLOR 14 ' change color
a$ = "You find some " + b$ + " coins!" ' make display message
GOSUB Display.Message ' output message
' calculate increment to players gold by bonus multiplier
temp# = RND * dlvl#
temp# = temp# + RND * GetUserRecordStats1(2) + 10
temp# = INT(temp# * bonus * 10)
z1# = GetUserRecordStats1(5)
z1# = z1# + temp#
CALL PutUserRecordStats1(5, z1#)
RETURN ' exit routine

' find scroll
Scroll:
CALL PutDungeonRecordLevelData(xco, yco, 46) ' reset cell to dungeon dot
COLOR 14 ' change color
a$ = "You find a scroll!"
GOSUB Display.Message ' output message
rand = INT(RND * 40 + 1) ' select a random spell from 40
RESTORE SpellData ' restore data pointer to spell data
CALL PutUserRecordSpell(rand, 1) ' set spell flag
FOR i = 1 TO rand ' read spell
    READ b$, c$ ' spell chant, spell name
NEXT
z = GetDungeonRecordStats(7)
z = z - 1
CALL PutDungeonRecordStats(7, z) ' decrement scroll
IF z = 0 THEN ' player has the last scroll
    IF GetDungeonRecordStats(8) = 0 THEN
        CALL PutDungeonRecordStats(8, 1) ' set all dungeon level scroll flag
        d$ = "scrolls"
        GOSUB Treasure.Bonus ' bonus for all scrolls
    END IF
END IF
RETURN ' exit routine

' enter chant of spell to cast
Cast.Chant:
COLOR 12 ' change color
a$ = "Type in spell chant:" ' make display message prompt
GOSUB Display.Message ' output message
COLOR 15 ' change color
' loop to retrieve spell$ character by character.
'   backspace, enter, and characters A to Z are accepted,
'   or nul for enter with no characters input. Maximum 55 characters.
spell$ = "" ' clear spell$
LOCATE 25, 22, 1 ' position cursor
DO
    X$ = "" ' get keystroke
    WHILE X$ = "" ' wait for input
        _LIMIT 100
        X$ = INKEY$ ' get keystroke
    WEND ' end wait
    SELECT CASE X$
        CASE CHR$(13) ' enter pressed
            EXIT DO
        CASE CHR$(8) ' backspace
            IF LEN(spell$) < 0 THEN
                LOCATE CSRLIN, POS(0) - 1, 1 ' position cursor
                PRINT " ";
                LOCATE CSRLIN, POS(0) - 1, 1 ' position cursor
                spell$ = LEFT$(spell$, LEN(spell$) - 1) ' truncate input one character
            END IF
        CASE "a" TO "z", "A" TO "Z", " " 'check valid character
            IF LEN(spell$) < 55 THEN ' verify length of string
                spell$ = spell$ + X$ ' append to spell$
                PRINT X$;
            END IF
    END SELECT
LOOP
spell$ = LCASE$(spell$)
COLOR 7 ' change color
RESTORE SpellData ' restore data pointer to spell data
FOR i = 1 TO 40 ' read 40 spell data items
    READ a$, c$ ' read chant, spell name
    IF a$ = spell$ THEN ' compare input to spell chant
        spn = INT((i - 1) / 10) + 1 ' calculate spell type 1 to 4
        sp = i - (spn - 1) * 10 ' calculate spell number 1 to 10
        SELECT CASE spn ' select spell type
            CASE 1 ' spell type 1
                GOSUB Cast.Spell.Type1a
            CASE 2 ' spell type 2
                GOSUB Cast.Spell.Type2a
            CASE 3 ' spell type 3
                GOSUB Cast.Spell.Type3a
            CASE 4 ' spell type 4
                GOSUB Cast.Spell.Type4a
        END SELECT
        RETURN ' exit routine
    END IF
NEXT
COLOR 13 ' change color
a$ = "That spell chant does nothing!" ' make display message
GOSUB Display.Message ' output message
RETURN ' exit routine

' use an item of magic from inventory
Use.Item:
COLOR 14 ' change color
a$ = "Enter item(1=wand, 3=potion, 5=staff, 7=ring, or 9 to quit)"
GOSUB Display.Message ' output message
DO ' get a keystroke
    GOSUB Get.Keystroke
LOOP UNTIL x = 1 OR x = 3 OR x = 5 OR x = 7 OR x = 9
' presses 1, 3, 5, 7, to use inventory item
' use a wand from inventory
IF x = 1 THEN
    COLOR 12 ' change color
    a$ = "Type in wand letter(A-J):" ' make display message prompt
    GOSUB Display.Message ' output message
    LOCATE 25, 28, 1 ' position cursor
    DO ' loop until alphaletter A to J is pressed
        X$ = "" ' get keystroke
        WHILE X$ = "" ' wait for input
            _LIMIT 100
            X$ = INKEY$ ' get keystroke
        WEND ' end wait
        X$ = UCASE$(X$)
        IF X$ >= "A" AND X$ <= "J" THEN
            EXIT DO
        END IF
    LOOP
    COLOR 7 ' change color
    i = ASC(X$) - 64 ' calculate wand number 1 to 10
    GOSUB Use.Wand ' use selected wand
    RETURN ' exit routine
END IF
' use a potion from inventory
IF x = 3 THEN
    COLOR 12 ' change color
    a$ = "Type in potion letter(K-T):" ' make display message prompt
    GOSUB Display.Message ' output message
    LOCATE 25, 30, 1 ' position cursor
    DO ' loop until alphaletter K to T is pressed
        X$ = "" ' get keystroke
        WHILE X$ = "" ' wait for input
            _LIMIT 100
            X$ = INKEY$ ' get keystroke
        WEND ' end wait
        X$ = UCASE$(X$)
        IF X$ >= "K" AND X$ <= "T" THEN
            EXIT DO
        END IF
    LOOP
    COLOR 7 ' change color
    i = ASC(X$) - 74 ' calculate potion number 1 to 10
    GOSUB Use.Potion ' use selected potion
    RETURN ' exit routine
END IF
' use a staff from inventory
IF x = 5 THEN
    COLOR 12 ' change color
    a$ = "Type in staff letter(Q-Z):" ' make display message prompt
    GOSUB Display.Message ' output message
    LOCATE 25, 28, 0 ' position cursor
    DO ' loop until alphaletter Q to Z is pressed
        X$ = "" ' get keystroke
        WHILE X$ = "" ' wait for input
            _LIMIT 100
            X$ = INKEY$ ' get keystroke
        WEND ' end wait
        X$ = UCASE$(X$)
        IF X$ >= "Q" AND X$ <= "Z" THEN
            EXIT DO
        END IF
    LOOP
    COLOR 7 ' change color
    i = ASC(X$) - 80 ' calculate staff number 1 to 10
    GOSUB Use.Staff ' use selected staff
    RETURN ' exit routine
END IF
' use a ring from inventory
IF x = 7 THEN
    COLOR 12 ' change color
    a$ = "Type in ring letter(Q-Z):" ' make display message prompt
    GOSUB Display.Message ' output message
    LOCATE 25, 28, 1 ' position cursor
    DO ' loop until alphaletter Q to Z is pressed
        X$ = "" ' get keystroke
        WHILE X$ = "" ' wait for input
            _LIMIT 100
            X$ = INKEY$ ' get keystroke
        WEND ' end wait
        X$ = UCASE$(X$)
        IF X$ >= "Q" AND X$ <= "Z" THEN
            EXIT DO
        END IF
    LOOP
    COLOR 7 ' change color
    i = ASC(X$) - 80 ' calculate ring number 1 to 10
    GOSUB Use.Ring ' use selected ring
    RETURN ' exit routine
END IF
RETURN ' exit routine

' use selected wand
Use.Wand:
RESTORE WandData ' restore data pointer to wand data
FOR j = 1 TO i ' read up to selected wand in variable i
    READ b$, x ' wand name, spell number
NEXT
' check for wand charges
IF GetUserRecordWand(i) = 0 THEN ' no charges left
    COLOR 13 ' change color
    a$ = "You do not have a " + b$ + " wand!" ' make display message
    GOSUB Display.Message ' output message
    RETURN ' exit routine
END IF
COLOR 13 ' change color
a$ = "You use a " + b$ + " wand!" ' for wand selected
z = GetUserRecordWand(i)
z = z - 1 ' decrement wand charges
CALL PutUserRecordWand(i, z) ' decrement wand charges
IF GetUserRecordWand(i) = 0 THEN ' verify wand charges
    a$ = a$ + " It vanished!" ' make display message, empty wand
END IF
GOSUB Display.Message ' output message
spn = INT((x - 1) / 10) + 1 ' calculate spell type from data in variable x
sp = x - (spn - 1) * 10 ' calculate spell number from data in variable x
SELECT CASE spn ' select spell type
    CASE 1 ' spell type 1
        GOSUB Cast.Spell.Type1a
    CASE 2 ' spell type 2
        GOSUB Cast.Spell.Type2a
    CASE 3 ' spell type 3
        GOSUB Cast.Spell.Type3a
    CASE 4 ' spell type 4
        GOSUB Cast.Spell.Type4a
END SELECT
RETURN ' exit routine

' use a selected potion
Use.Potion:
RESTORE PotionData ' restore data pointer to potion data
FOR j = 1 TO i ' read up to selected potion in variable i
    READ b$, x ' potion name, spell number
NEXT
' check for potion charges
IF GetUserRecordPotion(i) = 0 THEN ' no charges left
    COLOR 13 ' change color
    a$ = "You do not have a " + b$ + " potion!" ' make display message
    GOSUB Display.Message ' output message
    RETURN ' exit routine
END IF
COLOR 13 ' change color
a$ = "You drink a " + b$ + " potion!" ' for potion selected
z = GetUserRecordPotion(i)
z = z - 1 ' decrement potion charges
CALL PutUserRecordPotion(i, z) ' decrement potion charges
IF GetUserRecordPotion(i) = 0 THEN ' verify potion charges
    a$ = a$ + " It vanished!" ' make display message, for empty potion
END IF
GOSUB Display.Message ' output message
spn = INT((x - 1) / 10) + 1 ' calculate spell type from variable x
sp = x - (spn - 1) * 10 ' calculate spell number from variable x
SELECT CASE spn ' select spell type
    CASE 1 ' spell type 1
        GOSUB Cast.Spell.Type1a
    CASE 2 ' spell type 2
        GOSUB Cast.Spell.Type2a
    CASE 3 ' spell type 3
        GOSUB Cast.Spell.Type3a
    CASE 4 ' spell type 4
        GOSUB Cast.Spell.Type4a
END SELECT
RETURN ' exit routine

' use selected staff
Use.Staff:
RESTORE StaffData ' restore data pointer to staff data
FOR j = 1 TO i ' read up to selected staff in variable i
    READ b$, x ' staff name, spell number
NEXT
' check for staff charges
IF GetUserRecordStaff(i) = 0 THEN ' no charges left
    COLOR 13 ' change color
    a$ = "You do not have a " + b$ + " staff!" ' make display message
    GOSUB Display.Message ' output message
    RETURN ' exit routine
END IF
COLOR 13 ' change color
a$ = "You use a " + b$ + " staff!" ' for selected staff
z = GetUserRecordStaff(i)
z = z - 1 ' decrement staff charges
IF z = 0 THEN ' verify staff charges
    a$ = a$ + " It vanished!" ' make display message, for empty staff
END IF
GOSUB Display.Message ' output message
spn = INT((x - 1) / 10) + 1 ' calculate spell type from variable x
sp = x - (spn - 1) * 10 ' calculate spell number from variable x
SELECT CASE spn ' select spell type
    CASE 1 ' spell type 1
        GOSUB Cast.Spell.Type1a
    CASE 2 ' spell type 2
        GOSUB Cast.Spell.Type2a
    CASE 3 ' spell type 3
        GOSUB Cast.Spell.Type3a
    CASE 4 ' spell type 4
        GOSUB Cast.Spell.Type4a
END SELECT
RETURN ' exit routine

' use selected ring
Use.Ring:
RESTORE RingData ' restore data pointer to ring data
FOR j = 1 TO i ' read up to selected ring in variable i
    READ b$, x ' staff name, spell number
NEXT
' check for ring charges
IF GetUserRecordRing(i) = 0 THEN ' no charges left
    COLOR 13 ' change color
    a$ = "You do not have a " + b$ + " ring!" ' make display message
    GOSUB Display.Message ' output message
    RETURN ' exit routine
END IF
COLOR 13 ' change color
a$ = "You use a " + b$ + " ring!" ' for selected ring
z = GetUserRecordRing(i)
z = z - 1 ' decrement ring charges
CALL PutUserRecordRing(i, z) ' decrement ring charges
IF GetUserRecordRing(i) = 0 THEN ' verify ring charges
    a$ = a$ + " It vanished!" ' make display message, for empty ring
END IF
GOSUB Display.Message ' output message
spn = INT((x - 1) / 10) + 1 ' calculate spell type from variable x
sp = x - (spn - 1) * 10 ' calculate spell number from variable x
SELECT CASE spn ' select spell type
    CASE 1 ' spell type 1
        GOSUB Cast.Spell.Type1a
    CASE 2 ' spell type 2
        GOSUB Cast.Spell.Type2a
    CASE 3 ' spell type 3
        GOSUB Cast.Spell.Type3a
    CASE 4 ' spell type 4
        GOSUB Cast.Spell.Type4a
END SELECT
RETURN ' exit routine

' find a wand
Wand:
CALL PutDungeonRecordLevelData(xco, yco, 46) ' reset dungeon cell to dungeon dot
COLOR 14 ' change color
a$ = "You find a wand!"
GOSUB Display.Message ' output message
rand = INT(RND * 10 + 1) ' select a random wand from 10
a = GetUserRecordWand(rand) ' get wand array number
IF a = 0 THEN ' if wand exists in inventory
    CALL PutUserRecordWand(rand, INT(RND * 3 + 1)) ' set wand to random charges
END IF
' determine existing wands in dungeon level
z = GetDungeonRecordStats(25)
z = z - 1
CALL PutDungeonRecordStats(25, z) ' decrement wands
IF z = 0 THEN ' player has taken last wand
    IF GetDungeonRecordStats(26) = 0 THEN
        CALL PutDungeonRecordStats(26, 1) ' set all dungeon level wand flag
        d$ = "wands"
        GOSUB Treasure.Bonus ' bonus for last wand
    END IF
END IF
RETURN ' exit routine

' find a potion
Potion:
CALL PutDungeonRecordLevelData(xco, yco, 46) ' reset dungeon cell to dungeno dot
COLOR 14 ' change color
a$ = "You find a potion!"
GOSUB Display.Message ' output message
rand = INT(RND * 10 + 1) ' select a random potion from 10
a = GetUserRecordPotion(rand) ' get potion array number
IF a = 0 THEN ' if potion exists in inventory
    CALL PutUserRecordPotion(rand, INT(RND * 3 + 1)) ' set potion to random charges
END IF
' determine existing potions in dungeon level
z = GetDungeonRecordStats(27)
z = z - 1
CALL PutDungeonRecordStats(27, z) ' decrement potions
IF GetDungeonRecordStats(27) = 0 THEN ' player has take last potion
    IF GetDungeonRecordStats(28) = 0 THEN
        CALL PutDungeonRecordStats(28, 1) ' set all dungeon level potion flag
        d$ = "potions"
        GOSUB Treasure.Bonus ' bonus for last potion
    END IF
END IF
RETURN ' exit routine

' find a staff
Staff:
CALL PutDungeonRecordLevelData(xco, yco, 46) ' reset dungeon cell to dungeon dot
COLOR 14 ' change color
a$ = "You find a staff!"
GOSUB Display.Message ' output message
rand = INT(RND * 10 + 1) ' select a random staff from 10
a = GetUserRecordStaff(rand) ' get staff array number
IF a = 0 THEN ' if staff exists in inventory
    CALL PutUserRecordStaff(rand, INT(RND * 3 + 2)) ' set staff to random charges
END IF
' determine existing staffs in dungeon level
z = GetDungeonRecordStats(29)
z = z - 1
CALL PutDungeonRecordStats(29, z) ' decrement staffs
IF GetDungeonRecordStats(29) = 0 THEN ' player has taken last staff
    IF GetDungeonRecordStats(30) = 0 THEN
        CALL PutDungeonRecordStats(30, 1) ' set all dungeon level staff flag
        d$ = "staffs"
        GOSUB Treasure.Bonus ' bonus for last staff
    END IF
END IF
RETURN ' exit routine

' find a ring
Ring:
CALL PutDungeonRecordLevelData(xco, yco, 46) ' reset dungeon cell to dungeon dot
COLOR 14 ' change color
a$ = "You find a ring!"
GOSUB Display.Message ' output message
rand = INT(RND * 10 + 1) ' select a random ring from 10
a = GetUserRecordRing(rand) ' get ring array number
IF a = 0 THEN ' if ring exists in inventory
    CALL PutUserRecordRing(rand, INT(RND * 3 + 2)) ' set ring to random charges
END IF
' determine existing rings in dungeon level
z = GetDungeonRecordStats(31)
z = z - 1
CALL PutDungeonRecordStats(31, z) ' decrement rings
IF GetDungeonRecordStats(31) = 0 THEN ' player has taken last ring
    IF GetDungeonRecordStats(32) = 0 THEN
        CALL PutDungeonRecordStats(32, 1) ' set all dungeon level ring flag
        d$ = "rings"
        GOSUB Treasure.Bonus ' bonus for last ring
    END IF
END IF
RETURN ' exit routine

' calculate path for player to evade monster in nearby cell
Evade.Monster:
DO ' loop until empty cell to move player
    FOR k = 1 TO INT(RND * 2 + 2) ' move player random times
        c = 0 ' loop counter
        DO ' find empty cell
            c = c + 1 ' increment counter
            IF c > 10 THEN ' avoid infinite loop
                EXIT FOR ' exit routine
            END IF
            cxco = xco ' store temporary x-coordinate
            cyco = yco ' store temporary y-coordinate
            GOSUB Get.Random.Direction ' get valid random direction
            IF Coor.Direction THEN ' returned valid random direction
                IF GetDungeonRecordLevelData(xco + xo, yco + yo) = 46 THEN ' empty cell
                    EXIT DO ' found empty cell at random direction
                END IF
            END IF
        LOOP
        ' move to selected cell
        xco = xco + xo ' increment players x coordinate
        yco = yco + yo ' increment players y coordinate
        GOSUB Update.Player ' update nearby cell actions
    NEXT ' get next evade move
    IF GetDungeonRecordLevelData(xco, yco) = 46 THEN ' move to an dungeon dot
        EXIT DO ' end move player
    END IF
LOOP
RETURN ' exit routine

' player attacks all nearby monsters at all coordinates
Attack.Nearby.Monsters:
FOR mxco = xco - 1 TO xco + 1
    FOR myco = yco - 1 TO yco + 1
        IF mxco > 1 AND mxco < 23 THEN
            IF myco > 1 AND myco < 79 THEN
                GOSUB Attack.Monster
            END IF
        END IF
    NEXT
NEXT
RETURN ' exit from subroutine

' player attacks a nearby monster at coordinates mxco, myco
Attack.Monster:
f = 0 ' attack monster flag
FOR mn = 1 TO 24 ' loop through each of eight monsters in surrounding cells
    IF Monsters(mn, 1) = mxco AND Monsters(mn, 2) = myco THEN ' select monster coor.
        f = 1
        EXIT FOR
    END IF
NEXT
IF f = 0 THEN ' no monster found
    RETURN ' exit routine
END IF
x = GetUserRecordStats2(5) + GetUserRecordStats2(12) ' dexterity plus boots
y = INT(RND * (20 + Monsters(mn, 3))) ' miss monster equation
IF y > x THEN ' misses monster
    COLOR 12 ' change color
    a$ = "You missed!" ' make display message
    GOSUB Display.Message ' output message
    RETURN ' exit routine
END IF
ran! = RND ' calculate bonus to hit monster by random percentage
IF ran! > .96 THEN ' random hit one,
    bonus = 4 ' high hit bonus
ELSE
    IF ran! > .91 THEN ' random hit two,
        bonus = 3 ' normal hit bonus
    ELSE
        IF ran! > .86 THEN ' random hit three
            bonus = 2 ' low hit bonus
        ELSE
            bonus = 1 ' no hit bonus
        END IF
    END IF
END IF
' calculate hits on monster
hits# = RND * (GetUserRecordStats2(1) + stp) + GetUserRecordStats1(2) / 2
hits# = hits# * bonus + GetUserRecordStats2(7) + dlvl#
hits# = INT(hits#)
IF hits# <= 0 THEN ' zero hits misses
    COLOR 12 ' change color
    a$ = "You missed!" ' make display message
    GOSUB Display.Message ' output message
    RETURN ' exit routine
END IF
COLOR 14 ' change color
' make display message with monster name and hits on monster
a$ = "You hit the " + MonsterNames(Monsters(mn, 6)) + " for" + STR$(hits#) + " hits!"
IF slp THEN ' sleep flag
    a$ = a$ + " It's helpless!"
END IF
GOSUB Display.Message ' output message
Monsters(mn, 4) = Monsters(mn, 4) - hits# ' decrement monsters hits
IF Monsters(mn, 4) > 0 THEN ' monsters hits below zero, monster died,
    RETURN ' exit routine
END IF
ex# = Monsters(mn, 5) ' get experience
GOSUB Update.Stats ' add to players statistics
CALL PutDungeonRecordLevelData(mxco, myco, 46) ' reset dungeon cell to dungeon dot
Monsters(mn, 1) = 0 ' clear monster x-coordinate
Monsters(mn, 2) = 0 ' clear monster y=coordinate
COLOR 15 ' change color
LOCATE mxco, myco, 0 ' position cursor
PRINT "$"; ' make $ at dead monster cell
' select coin type
ran! = RND ' based on percentage
IF ran! > .85 THEN
    bonus = 8 ' multiplier
    b$ = "platinum" ' coin name
ELSE
    IF ran! > .75 THEN
        bonus = 5 ' multiplier
        b$ = "gold" ' coin name
    ELSE
        IF ran! > .50 THEN
            bonus = 3 ' multiplier
            b$ = "silver" ' coin name
        ELSE
            bonus = 2 ' multiplier
            b$ = "copper" ' coin name
        END IF
    END IF
END IF
' make display message with monster killed and coins found
COLOR 14 ' change color
a$ = "You killed the " + MonsterNames(Monsters(mn, 6)) + "!"
a$ = a$ + " On it is some " + b$ + "!"
GOSUB Display.Message ' output message
temp# = RND * Monsters(mn, 4)
temp# = temp# + RND * Monsters(mn, 5) + 10
temp# = INT(temp# * bonus * 10)
z1# = GetUserRecordStats1(5)
z1# = z1# + temp#
CALL PutUserRecordStats1(5, z1#)
IF RND > .5 THEN ' 50/50 find some gems
    COLOR 15 ' change color
    LOCATE mxco, myco, 0 ' position cursor
    PRINT "*"; ' select gem type
    ran! = RND ' based on percentage
    IF ran! > .85 THEN
        bonus = 8 ' multiplier
        b$ = "diamonds" ' gem name
    ELSE
        IF ran! > .75 THEN
            bonus = 5 ' multiplier
            b$ = "emeralds" ' gem name
        ELSE
            IF ran! > .50 THEN
                bonus = 3 ' multiplier
                b$ = "saphires" ' gem name
            ELSE
                bonus = 2 ' multiplier
                b$ = "rubies" ' gem name
            END IF
        END IF
    END IF
    COLOR 14 ' change color
    ' make display message with gem type found
    a$ = "You find some " + b$ + "!"
    GOSUB Display.Message ' output message
    temp# = RND * Monsters(mn, 4)
    temp# = temp# + RND * Monsters(mn, 5) + 10
    temp# = INT(temp# * bonus)
    z1# = GetUserRecordStats1(5)
    z1# = z1# + temp#
    CALL PutUserRecordStats1(5, z1#)
END IF
IF RND > .75 THEN ' 3/4 percent chance for chest
    COLOR 15 ' change color
    LOCATE mxco, myco, 0 ' position cursor
    PRINT CHR$(254); ' chest symbol
    GOSUB Chest ' open the chest
END IF
COLOR 7 ' change color
LOCATE mxco, myco, 0 ' position cursor
PRINT "."; ' display dungeon dot
z = GetDungeonRecordStats(6)
z = z - 1
CALL PutDungeonRecordStats(6, z) ' decrement dungeon monsters
IF GetDungeonRecordStats(6) = 0 THEN ' last monster
    IF GetDungeonRecordStats(5) = 0 THEN ' all dungeon level monsters flags
        CALL PutDungeonRecordStats(5, 1) ' set all dungeon level monsters flag
        ' calculate experience gained for last monster killed
        ex# = (RND * dlvl# * 2 + 1) * GetUserRecordStats1(2) ^ 2 / 2
        ex# = ex# + dlvl# * 2 + RND * GetUserRecordStats1(2) * 50
        ex# = INT(ex#)
        COLOR 14 ' change color
        a$ = "You killed all the monsters on this level! You gain"
        a$ = a$ + STR$(ex#) + " experience!"
        GOSUB Display.Message ' output message
        GOSUB Update.Stats ' add to players statistics
    END IF
END IF
RETURN ' exit routine

' clear monster array
Clear.Monsters:
FOR i = 1 TO 24 ' clear monster array
    FOR j = 1 TO 6 ' of all statistics
        Monsters(i, j) = 0 ' set to zero
    NEXT ' end clear loop
NEXT ' end clear loop
RETURN ' exit routine

' reset/increment any player counters
Update.Player.Counters:
llc = llc - 1 ' decrement extended cell display counter
IF llc < 0 THEN ' check extended cell display counter
    ll = 1 ' reset cell displays cell display amount
    llc = 0 ' reset extended cell display counter
END IF
gdc = gdc - 1 ' decrement extended monster cell death counter
IF gdc < 0 THEN ' check extended monster cell death counter
    gd = 0 ' reset cell monster death flag
    gdc = 0 ' reset extended monster cell death counter
END IF
' update any remaining counters
GOSUB Update.Other.Counters
RETURN ' exit routine

' reset/increment any player counters
Update.Monster.Counters:
shpc = shpc - 1 ' decrement shield counter
IF shpc < 0 THEN ' check shield counter
    shp = 0 ' reset shield amount
    shpc = 0 ' reset shield counter
END IF
mhpmc = mhpmc - 1 'decrement monster hit point minus counter
IF mhpmc < 0 THEN ' check monster hit point minus counter
    mhpm = 0 ' reset monster hit point minus amount
    mhpmc = 0 ' reset monster hit point minus counter
END IF
slpc = slpc - 1 ' decrement sleep counter
IF slpc < 0 THEN ' check sleep counter
    slp = 0 ' reset sleep flag
    slpc = 0 ' reset sleep counter
END IF
stpc = stpc - 1 ' decrement strength counter
IF stpc < 0 THEN ' check strength counter
    stp = 0 ' reset strength amount
    stpc = 0 ' reset strength counter
END IF
' update any remaining counters
GOSUB Update.Other.Counters
RETURN ' exit routine

' reset/increment any other counters
Update.Other.Counters:
pwc = pwc - 1 ' decrement pass monsters counter
IF pwc < 0 THEN ' check pass monsters counter
    pw = 0 ' reset pass monsters flag
    pwc = 0 ' reset pass monsters counter
END IF
IF pw = 1 THEN ' check pass monsters flag
    slp = 1 ' set sleep flag
    slpc = 1 ' set sleep counter
END IF
RETURN ' exit routine

' find a trap
Trap:
CALL PutDungeonRecordLevelData(xco, yco, 46) ' reset cell to dungeon dot
COLOR 15 ' change color
LOCATE xco, yco, 0 ' position cursor
PRINT "^"; ' display a blinking trap where the character X is
COLOR 11 ' change color
a$ = "You triggered a trap! " ' message to display in hit.player routine
mhp# = 0 ' reset monster hit points to zero, then
GOSUB Hit.Player ' hit player will calculate hits from trap
CALL SetDungeonRecordStats(9) ' decrement traps
RETURN ' exit routine

' find a pit
Pit:
CALL PutDungeonRecordLevelData(xco, yco, 46) ' reset cell to dungeon dot
COLOR 15 ' change color
LOCATE xco, yco, 0 ' position cursor
PRINT "o"; ' display a blinking pit where the character X is
COLOR 11 ' change color
a$ = "You fell in a pit! " ' message to display in hit.player routine
mhp# = 0 ' reset monster hit points to zero, then
GOSUB Hit.Player ' hit player will calculate hits from pit
CALL SetDungeonRecordStats(13) ' decrement pits
RETURN ' exit routine

' open located chest
Find.Chest:
CALL PutDungeonRecordLevelData(xco, yco, 46) ' reset cell with dungeon dot
GOSUB Chest
z = GetDungeonRecordStats(23)
z = z - 1
CALL PutDungeonRecordStats(23, z) ' decrement chests
IF GetDungeonRecordStats(23) = 0 THEN ' player has opened last chest
    IF GetDungeonRecordStats(24) = 0 THEN
        CALL PutDungeonRecordStats(24, 1) ' set all dungeon level chest flag
        d$ = "chests"
        GOSUB Treasure.Bonus ' bonus for last chest
    END IF
END IF
RETURN ' exit routine

' open any chest
Chest:
COLOR 14 ' change color
a$ = "You find a chest!(5 to open)"
GOSUB Display.Message ' output message
GOSUB Get.Keystroke ' read a key press
' presses enter/5 to open chest
IF x = 5 THEN
    choice = INT(RND * 6 + 1) ' select random treasure in chest
    SELECT CASE choice ' get treasure name
        CASE 1 ' type 1
            b$ = "a weapon"
        CASE 2 ' type 2
            b$ = "a shield"
        CASE 3 ' type 3
            b$ = "some armor"
        CASE 4 ' type 4
            b$ = "a cloak"
        CASE 5 ' type 5
            b$ = "a helmet"
        CASE 6 ' type 6
            b$ = "some boots"
    END SELECT
    COLOR 13 ' change color
    a$ = "In it you find " + b$ + "!"
    GOSUB Display.Message ' output message
    ' calculate bonus of item selected
    bonus = INT(RND * dlvl# * 2 + dlvl#)
    IF bonus > 99 THEN ' restrict bonus plus of item to 99
        bonus = 99 ' reset to 99 if greater
    END IF
    ' set the inventory to the bonus
    IF bonus > GetUserRecordStats2(choice + 6) THEN ' is chest bonus greater?
        CALL PutUserRecordStats2(choice + 6, bonus) ' increase weapon plus;
    END IF
    IF RND > .99 THEN ' 1 percent chance of chest containing a globe of power
        GOSUB Found.Globe
    END IF
ELSE
    Buf$ = X$ ' store typeahead buffer
END IF
RETURN ' exit from subroutine

' gets a globe of power
Found.Globe:
IF GetDungeonRecordStats(1) = 0 THEN ' if dungeon level has not had one yet
    IF GetUserRecordStats2(17) = 0 THEN ' and if player does not have a globe
        a$ = "In it you find The Globe of Power!" ' make display message
        GOTO Get.Globe ' player gets a globe
    END IF ' check player has globe
END IF ' check level has a globe
RETURN ' exit routine

' adds a globe of power
Make.Globe:
a$ = "You find the Globe of Power!" ' make display message

' adds a globe of power
Get.Globe:
CALL PutDungeonRecordStats(1, 1) ' set dungeon level globe flag
CALL PutUserRecordStats2(17, 1) ' set player globe flag
FOR i = 1 TO 10 ' loop through globe array
    CALL PutUserRecordGlobe(i, 0) ' set usage flag for each
NEXT ' globe power to zero
COLOR 14 ' change color
GOSUB Display.Message ' output message
COLOR 7 ' change color
RETURN ' exit routine

' use a power of the globe
Use.Globe:
IF GetUserRecordStats2(17) = 0 THEN ' check if player has the globe
    COLOR 13 ' change color
    a$ = "You do not have The Globe of Power!" ' make display message
    GOSUB Display.Message ' output message
    RETURN ' exit routine
END IF
IF GetUserRecordGlobe(i) = 1 THEN ' if globe power in variable i has been used,
    COLOR 13 ' change color
    a$ = "This Globe Power is depleted!" ' make display message
    GOSUB Display.Message ' output message
    RETURN ' exit routine
END IF
CALL PutUserRecordGlobe(i, 1) ' set globe power variable i flag to used
SELECT CASE i ' select globe power variable i 1 to 10
    CASE 1 ' replenish level
        COLOR 13 ' change color
        a$ = "The dungeon becomes replenished!" ' make display message
        GOSUB Display.Message ' output message
        GOSUB Zero.Dungeon ' first remove all dungeon treasure,
        GOSUB Replenish.Dungeon ' then replenish dungeon level treasure
    CASE 2 ' down 10 levels
        GOSUB Store.Dungeon.Level ' record the dungeon statistics and screen
        dlvl# = dlvl# + 10# ' increment players dungeon level counter
        ' check if player has entered a dungeon level lower than his last recorded
        IF dlvl# > GetUserRecordStats1(1) THEN ' if lower,
            CALL PutUserRecordStats1(1, dlvl#) ' set players maximum dungeon to current level
        END IF
        GOSUB Next.Dungeon.Level ' load dungeon level statistics
        GOSUB Cell.Action ' compute nearby cells
    CASE 3 ' up 10 levels
        GOSUB Store.Dungeon.Level ' record the dungeon statistics and screen
        FOR dt = 1 TO 10 ' count up 10 levels
            GOSUB Decrement.Dungeon.Level ' subtract one from dungeon level variable
            IF dlvl# = 0# THEN ' exited the dungeon
                RETURN ' exit routine
            END IF
        NEXT
        GOSUB Next.Dungeon.Level ' load dungeon level statistics
        GOSUB Cell.Action ' compute nearby cells
    CASE 4 ' collect all
        COLOR 13 ' change color
        a$ = "You now have all treasure!" ' make display message
        GOSUB Display.Message ' output message
        FOR j = 1 TO 10 ' set all inventory
            CALL PutUserRecordWand(j, INT(RND * 3 + 2)) ' charges randomly
            CALL PutUserRecordPotion(j, INT(RND * 3 + 2))
            CALL PutUserRecordStaff(j, INT(RND * 3 + 2))
            CALL PutUserRecordRing(j, INT(RND * 3 + 2))
        NEXT
    CASE 5 ' use all wands
        gdc = 0 ' reset extended monster cell death counter
        FOR j = 1 TO 10 ' add up all wand charges
            gdc = gdc + GetUserRecordWand(j) ' set extended monster cell death counter
            CALL PutUserRecordWand(j, 0) ' reset wands charges to zero
        NEXT
        IF gdc > 0 THEN ' if any charges added,
            gd = 1 ' set extended monster cell death flag
            COLOR 13 ' change color
            a$ = "You use all wands! They vanished!" ' make display message
            GOSUB Display.Message ' output message
            GOSUB Cell.Monster.Attack ' all nearby monsters die
        ELSE
            gd = 0 ' reset extended monster cell death flag
            COLOR 13 ' change color
            a$ = "You have no wands!" ' make display message
            GOSUB Display.Message ' output message
        END IF
    CASE 6 ' use all potions
        gdc = 0 ' reset extended monster cell death counter
        FOR j = 1 TO 10 ' add up all potion charges
            gdc = gdc + GetUserRecordPotion(j) ' set extended monster cell death counter
            CALL PutUserRecordPotion(j, 0) ' reset potion charges to zero
        NEXT
        IF gdc > 0 THEN ' if any charges added,
            gd = 1 ' set extended monster cell death flag
            COLOR 13 ' change color
            a$ = "You use all potions! They vanished!" ' make display message
            GOSUB Display.Message ' output message
            GOSUB Cell.Monster.Attack ' all nearby monsters die
        ELSE
            gd = 0 ' set extended monster cell death flag
            COLOR 13 ' change color
            a$ = "You have no potions!" ' make display message
            GOSUB Display.Message ' output message
        END IF
    CASE 7 ' use all staffs
        gdc = 0 ' reset extended monster cell death counter
        FOR j = 1 TO 10 ' add up all staff charges
            gdc = gdc + GetUserRecordStaff(j) ' set extended monster cell death counter
            CALL PutUserRecordStaff(j, 0) ' reset staff charges to zero
        NEXT
        IF gdc > 0 THEN ' if any charges added,
            gd = 1 ' set extended monster cell death flag
            COLOR 13 ' change color
            a$ = "You use all staffs! They vanished!" ' make display message
            GOSUB Display.Message ' output message
            GOSUB Cell.Monster.Attack ' all nearby monsters die
        ELSE
            gd = 0 ' reset extended monster cell death flag
            COLOR 13 ' change color
            a$ = "You have no staffs!" ' make display message
            GOSUB Display.Message ' output message
        END IF
    CASE 8 ' use all rings
        gdc = 0 ' reset extended monster cell death counter
        FOR j = 1 TO 10 ' add up all ring charges
            gdc = gdc + GetUserRecordRing(j) ' set extended monster cell death counter
            CALL PutUserRecordRing(j, 0) ' reset ring charges to zero
        NEXT
        IF gdc > 0 THEN ' if any charges added,
            gd = 1 ' set extended monster cell death flag
            COLOR 13 ' change color
            a$ = "You use all rings! They vanished!" ' make display message
            GOSUB Display.Message ' output message
            GOSUB Cell.Monster.Attack ' all nearby monsters die
        ELSE
            gd = 0 ' reset extended monster cell death flag
            COLOR 13 ' change color
            a$ = "You have no rings!" ' make display message
            GOSUB Display.Message ' output message
        END IF
    CASE 9 ' trade all
        k = 0 ' set counter to add all inventry charges
        FOR j = 1 TO 10 ' add all
            k = k + GetUserRecordWand(j) ' wand charges,
            CALL PutUserRecordWand(j, 0) ' set wands to zero
            k = k + GetUserRecordPotion(j) ' potion charges,
            CALL PutUserRecordPotion(j, 0) ' set potions to zero
            k = k + GetUserRecordStaff(j) ' staff charges,
            CALL PutUserRecordStaff(j, 0) ' set staffs to zero
            k = k + GetUserRecordRing(j) ' ring charges,
            CALL PutUserRecordRing(j, 0) ' set to zero
        NEXT
        k = INT(k / 6) ' average for increase to each statistic
        FOR z1 = 1 TO 12
            z2 = GetUserRecordStats2(z1)
            z2 = z2 + k ' add average to stat
            CALL PutUserRecordStats2(z1, z2) ' add average to strength
            IF GetUserRecordStats2(z1) > 99 THEN ' check limit,
                CALL PutUserRecordStats2(z1, 99) ' set to maximum
            END IF
        NEXT
        COLOR 13 ' change color
        a$ = "You trade all treasure! They vanished!" ' make display message
        GOSUB Display.Message ' output message
        GOSUB Extended.Status.Line
    CASE 10 ' clear level
        CALL PutDungeonRecordStats(4, 1) ' set cleared dungeon flag
        GOSUB Dungeon.Worth ' total all dungeon treasure
        GOSUB Zero.Dungeon ' clear all dungeon level treasure
        COLOR 13 ' change color
        a$ = "You now have" + STR$(GetUserRecordStats1(5)) + " gold!" ' display message
        GOSUB Display.Message ' output message
END SELECT
FOR j = 1 TO 10 ' loop through globe power usage flag array
    IF GetUserRecordGlobe(j) = 0 THEN ' remaining globe power
        RETURN ' exit routine
    END IF
NEXT
CALL PutUserRecordStats2(17, 0) ' reset player globe flag
COLOR 13 ' change color
a$ = "The Globe of Power is depleted!" ' make display message
GOSUB Display.Message ' output message
RETURN ' exit routine

' increase experience and reset players statistics
Update.Stats:
z1# = GetUserRecordStats1(3)
z1# = z1# + ex# ' increase experience
CALL PutUserRecordStats1(3, z1#) ' increase experience
IF GetUserRecordStats1(2) = 99 THEN ' check valid player level
    RETURN ' exit routine
END IF
x# = GetUserRecordStats1(2) + 6
y# = 2 ^ x#
IF GetUserRecordStats1(3) < y# THEN ' check sufficient experience
    RETURN ' exit routine
END IF
x# = GetUserRecordStats1(2)
y# = 2 ^ x#
IF GetUserRecordStats1(5) < y# THEN ' and gold to reach next level
    RETURN ' exit routine
END IF
z1# = GetUserRecordStats1(5)
z1# = z1# - y# ' decrease gold for new level
CALL PutUserRecordStats1(5, z1#) ' decrease gold for new level

' increase players maximum hit points by constitution
z1# = GetUserRecordStats1(6)
z1# = z1# + GetUserRecordStats2(4)
CALL PutUserRecordStats1(6, z1#)

CALL PutUserRecordStats1(4, z1#) ' set players hit points

z1# = GetUserRecordStats1(2)
z1# = z1# + 1 ' increment players level
CALL PutUserRecordStats1(2, z1#) ' increment players level
x = GetUserRecordStats1(2) * 10 ' set players spell points
FOR z1 = 13 TO 16
    CALL PutUserRecordStats2(z1, x)
NEXT
RETURN ' exit routine

' display additional players statistics at row 25
Extended.Status.Line:
a$ = "" ' set status line
FOR s1 = 1 TO 12 ' loop through all statistics
    b$ = STR$(GetUserRecordStats2(s1)) + " " ' store statistic
    SELECT CASE s1 ' choose statistics
        CASE 1 ' strength
            IF LEN(a$) + LEN(b$) + 3 < 80 THEN ' check line length
                a$ = a$ + "STR" + b$ ' append stat
            ELSE ' line length
                EXIT FOR ' end statistic loop
            END IF ' end check line length
        CASE 2 ' intelligence
            IF LEN(a$) + LEN(b$) + 3 < 80 THEN ' check line length
                a$ = a$ + "INT" + b$ ' append stat
            ELSE ' line length
                EXIT FOR ' end statistic loop
            END IF ' end check line length
        CASE 3 ' wisdom
            IF LEN(a$) + LEN(b$) + 3 < 80 THEN ' check line length
                a$ = a$ + "WIS" + b$ ' append stat
            ELSE ' line length
                EXIT FOR ' end statistic loop
            END IF ' end check line length
        CASE 4 ' constitution
            IF LEN(a$) + LEN(b$) + 3 < 80 THEN ' check line length
                a$ = a$ + "CON" + b$ ' append stat
            ELSE ' line length
                EXIT FOR ' end statistic loop
            END IF ' end check line length
        CASE 5 ' dexterity
            IF LEN(a$) + LEN(b$) + 3 < 80 THEN ' check line length
                a$ = a$ + "DEX" + b$ ' append stat
            ELSE ' line length
                EXIT FOR ' end statistic loop
            END IF ' end check line length
        CASE 6 ' charisma
            IF LEN(a$) + LEN(b$) + 3 < 80 THEN ' check line length
                a$ = a$ + "CHA" + b$ ' append stat
            ELSE ' line length
                EXIT FOR ' end statistic loop
            END IF ' end check line length
        CASE 7 ' weapon plus
            IF LEN(a$) + LEN(b$) + 2 < 80 THEN ' check line length
                a$ = a$ + "WE" + b$ ' append stat
            ELSE ' line length
                EXIT FOR ' end statistic loop
            END IF ' end check line length
        CASE 8 ' shield plus
            IF LEN(a$) + LEN(b$) + 2 < 80 THEN ' check line length
                a$ = a$ + "SH" + b$ ' append stat
            ELSE ' line length
                EXIT FOR ' end statistic loop
            END IF ' end check line length
        CASE 9 ' armor plus
            IF LEN(a$) + LEN(b$) + 2 < 80 THEN ' check line length
                a$ = a$ + "AR" + b$ ' append stat
            ELSE ' line length
                EXIT FOR ' end statistic loop
            END IF ' end check line length
        CASE 10 ' cloak plus
            IF LEN(a$) + LEN(b$) + 2 < 80 THEN ' check line length
                a$ = a$ + "CL" + b$ ' append stat
            ELSE ' line length
                EXIT FOR ' end statistic loop
            END IF ' end check line length
        CASE 11 ' helmet plus
            IF LEN(a$) + LEN(b$) + 2 < 80 THEN ' check line length
                a$ = a$ + "HE" + b$ ' append stat
            ELSE ' lline length
                EXIT FOR ' end statistic loop
            END IF ' end check line length
        CASE 12 ' boot plus
            IF LEN(a$) + LEN(b$) + 2 < 80 THEN ' check line length
                a$ = a$ + "BO" + b$ ' append stat
            ELSE ' line length
                EXIT FOR ' end statistic loop
            END IF ' end check line length
    END SELECT
NEXT ' end loop through all statistics
' display extended status line
a$ = LEFT$(a$, 79) ' truncate to status length
a$ = a$ + SPACE$(79 - LEN(a$)) ' add blanks
LOCATE 25, 1, 0 ' position cursor
COLOR 12 ' change color
c = 0 ' status line counter
d = 0 ' color toggle
FOR s1 = 1 TO LEN(a$) ' loop through status line
    IF MID$(a$, s1, 1) = " " THEN ' find space
        c = c + 1 ' word counter
        IF c <= 10 THEN ' first half of status
            d = NOT d ' toggle color
            IF d THEN ' select color
                COLOR 15 ' change color
            ELSE ' other color
                COLOR 12 ' change color
            END IF ' end select color
        ELSE ' second half of status
            d = NOT d ' toggle color
            IF d THEN ' select color
                COLOR 15 ' change color
            ELSE ' other color
                COLOR 14 ' change color
            END IF ' end select color
        END IF ' end check first half
    END IF ' end check space
    PRINT MID$(a$, s1, 1); ' display status character
NEXT ' end loop through status line
a$ = "" ' clear message
RETURN ' exit routine

' display message at row 25
Display.Message:
a$ = LEFT$(a$, 79) ' truncate to status length
a$ = a$ + SPACE$(79 - LEN(a$)) ' add blanks
LOCATE 25, 1, 0 ' position cursor
PRINT a$; ' display status message
a$ = "" ' clear message
GOSUB Pause.Second ' pause before next message
RETURN ' exit routine

' make players statistics status line at row 24
Status.Line:
' make first half of status line
e$ = "DLVL" + STR$(dlvl#) ' dungeon level
e$ = e$ + " LVL" + STR$(GetUserRecordStats1(2)) ' players level
e$ = e$ + " HP" + STR$(GetUserRecordStats1(4)) ' players hit points
e$ = e$ + " EXP" + STR$(GetUserRecordStats1(3)) ' experience
e$ = e$ + " GOLD" + STR$(GetUserRecordStats1(5)) ' gold
e$ = e$ + " NM" + STR$(GetDungeonRecordStats(6)) + " " ' number of monsters remaining
x = LEN(e$) ' calculate length of first half of status line
' make second half of status line
f$ = "SP" ' spell numbers
FOR i = 13 TO 16 ' spell level points left
    f$ = f$ + STR$(GetUserRecordStats2(i)) ' append spell level points
NEXT
y = LEN(f$) ' calculate length of second half of status line
' remake first half of status line
LOCATE 24, 1, 0 ' position cursor
COLOR 12 ' change color
PRINT "DLVL"; ' dungeon level
COLOR 15 ' change color
PRINT STR$(dlvl#); " ";
COLOR 12 ' change color
PRINT "LVL"; ' players level
COLOR 15 ' change color
PRINT STR$(GetUserRecordStats1(2)); " "; ' players level
COLOR 14 ' change color
PRINT "HP"; ' players hit points
COLOR 15 ' change color
PRINT STR$(GetUserRecordStats1(4)); " "; ' players hit points
COLOR 14 ' change color
PRINT "EXP"; ' experience
COLOR 15 ' change color
PRINT STR$(GetUserRecordStats1(3)); " "; ' players experience
COLOR 10 ' change color
PRINT "GOLD"; ' gold
COLOR 15 ' change color
PRINT STR$(GetUserRecordStats1(5)); " "; ' players gold
COLOR 10 ' change color
PRINT "NM"; ' number of monsters remaining
COLOR 15 ' change color
PRINT STR$(GetDungeonRecordStats(6)); " "; ' monsters remaining
IF (x + y) < 80 THEN ' if both halfs of status line can be displayed
    COLOR 11 ' change color
    PRINT "SP"; ' spell level points remaining
    COLOR 15 ' change color
    FOR i = 13 TO 16 ' spell level points left
        PRINT STR$(GetUserRecordStats2(i)); ' display spell level points
    NEXT
    PRINT SPACE$(79 - x - y); ' append blanks
    RETURN ' exit routine
END IF
PRINT SPACE$(79 - x); ' append blanks
RETURN ' exit routine

' pause one-tenth second after displaying a message
Pause.Second:
tim! = TIMER ' store start time
DO ' loop until timer elapsed
    z$ = INKEY$ ' eat keystroke
    e! = TIMER - tim! ' calculate time elapsed
    IF e! < 0! THEN ' check past midnight during loop
        e! = e! + 86400! ' increment one day of seconds
    END IF ' end check midnight
    IF e! > .1! THEN ' verify time passed
        EXIT DO ' exit routine
    END IF ' end verify time passed
LOOP ' end loop until time elapsed
RETURN ' exit routine

' retreives a key entered from the keyboard, the key entry may be numeric,
' a extended keypad scan code, or any literal keyboard character.
Get.Keystroke:
x = 0 ' reset keystroke value
IF Buf$ <> "" THEN ' check typeahead buffer
    X$ = Buf$ ' copy buffer
    Buf$ = "" ' reset buffer
ELSE ' check buffer
    X$ = "" ' get keystroke
    WHILE X$ = "" ' wait for input
        _LIMIT 100
        X$ = INKEY$ ' get keystroke
    WEND ' end wait
END IF ' end check buffer
SELECT CASE LEN(X$)
    CASE 1
        SELECT CASE X$
            CASE "0" TO "9"
                x = VAL(X$) ' convert to numeric variable
            CASE CHR$(13) ' if key is the enter key,
                x = 5 ' set numeric variable to 5
        END SELECT
    CASE 2 ' extended scan code
        IF pwl = 1 THEN ' passwall has separate input loop
            RETURN ' exit routine
        END IF
        SELECT CASE ASC(RIGHT$(X$, 1))
            CASE 79 ' end key
                x = 1 ' set numeric variable to 1
            CASE 80 ' 2, move player down
                x = 2 ' set numeric variable to 2
            CASE 81 ' page down key
                x = 3 ' set numeric variable to 3
            CASE 75 ' 4, move player left
                x = 4 ' set numeric variable to 4
            CASE 76 ' center key (enter)
                x = 5 ' set numeric variable to 5
            CASE 77 ' 6, move player right
                x = 6 ' set numeric variable to 6
            CASE 71 ' home key
                x = 7 ' set numeric variable to 7
            CASE 72 ' 8, move player up
                x = 8 ' set numeric variable to 8
            CASE 73 ' page up key (ignore)
                x = 9 ' set numeric variable to 9
        END SELECT
END SELECT
RETURN ' exit routine

' user list routine
User.List:
GOSUB Store.Dungeon.Level ' store dungeon screen
GOSUB Put.User.File ' store user record
CLS ' clear screen
CALL User.List.Menu ' run utility program
GET 1, User.Index, UserRecord ' read the user record from data file
GOSUB Restore.Screen ' redraw dungeon screen
RETURN ' exit routine

' edit user routine
Edit.User:
GOSUB Store.Dungeon.Level ' store dungeon screen
GOSUB Put.User.File ' store user record
CLS ' clear screen
CALL Edit.User.Menu ' run edit program
GET 1, User.Index, UserRecord ' read the user record from data file
GOSUB Restore.Screen ' redraw dungeon screen
RETURN ' exit routine

' map edit routine
Map.Edit:
GOSUB Store.Dungeon.Level ' store dungeon screen
GOSUB Put.User.File ' store user record
CLS ' clear screen
CALL Map.Editor ' run utility program
GET 1, User.Index, UserRecord ' read the user record from data file
GOSUB OpenData.File ' reopen data file
GOSUB Restore.Screen ' redraw dungeon screen
RETURN ' exit routine

' shell to DOS routine ' Alt-Down
DOS.Shell:
GOSUB Store.Dungeon.Level ' store dungeon screen
GOSUB Put.User.File ' store user record
LOCATE , , 1 ' reset cursor
SHELL ' shell to DOS
LOCATE , , 0 ' reset cursor
GET 1, User.Index, UserRecord ' read the user record from data file
GOSUB Restore.Screen ' redraw dungeon screen
RETURN ' exit routine

' help function/0 key
Display.Help:
GOSUB Store.Dungeon.Level ' store dungeon screen
CLS ' clear screen
CALL Display.Help.Menu ' run help program
GOSUB Restore.Screen ' redraw dungeon screen
RETURN ' exit routine

Display.Help2: ' Alt-Right
GOSUB Store.Dungeon.Level ' store dungeon screen
GOSUB Display.Brief.Help.Screen ' display default help screen
GOSUB Restore.Screen ' redraw dungeon screen
RETURN ' exit routine

' display default help screens
Display.Brief.Help.Screen:
COLOR 14, 1 ' blue background
CLS ' clear screen
PRINT
PRINT "Cursor Keys                         Numeric Keypad";
PRINT "       Dungeon Runes"
PRINT
PRINT "End               - fight           1  - fight";
PRINT "           X  - your player"
PRINT "Down              - south           2  - south";
PRINT "           #  - monster"
PRINT "Page Down         - cast            3  - cast";
PRINT "            ~  - scroll"
PRINT "Left              - west            4  - west";
PRINT "            $  - gold"
PRINT "Return            - accept          5  - accept";
PRINT "          "; CHR$(254); "  - chest"
PRINT "Right             - east            6  - east";
PRINT "            ^  - trap"
PRINT "Home              - evade           7  - evade";
PRINT "           o  - pit"
PRINT "Up                - north           8  - north";
PRINT "           *  - gems"
PRINT "Page Up           - ignore          9  - ignore";
PRINT "          !  - teleporter"
PRINT "Insert            - help            0  - help";
PRINT "            ?  - temple"
PRINT "Delete            - status line     .  - status line";
PRINT "     >  - stairs up"
PRINT "Control-Right     - information     +  - information";
PRINT "     <  - stairs down"
PRINT "Control-PageDown  - display chants  -  - display chants";
PRINT "  &  - coins"
PRINT "Control-Home      - trade           \  - trade";
PRINT "           |  - wand"
PRINT "Control-End       - change level    =  - change level";
PRINT "    @  - potion"
PRINT "Control-PageUp    - use item        '  - use item";
PRINT "        %  - staff"
PRINT "Control-Left      - inventory       /  - inventory";
PRINT "       "; CHR$(34); "  - ring"
PRINT "Shift-Tab         - buy inventory   ;  - buy inventory";
PRINT "   "; CHR$(176); "  - dungeon wall";
PRINT "Alt-=             - user list       `  - user list (left apostrophe)"
PRINT "Alt--             - user edit       _  - user edit (underscore)"
PRINT "Ctrl-Delete       - DOS Shell       Alt-Up - Map Editor"
PRINT "Alt-Left          - Power Globe     F11/F12 - Display/Clear Symbols";
GOSUB Key.Press
COLOR 14, 1 ' blue background
CLS ' clear screen
PRINT SPACE$(10);
PRINT "Spell Level 1  -  Normal          Spell Level 3  -  Shift-"
PRINT SPACE$(10);
PRINT "F1  - heal                        F1  - regenerate"
PRINT SPACE$(10);
PRINT "F2  - protection from evil        F2  - forget"
PRINT SPACE$(10);
PRINT "F3  - magic missile               F3  - lightning bolt"
PRINT SPACE$(10);
PRINT "F4  - shield                      F4  - phantasmal force"
PRINT SPACE$(10);
PRINT "F5  - strength                    F5  - wall of force"
PRINT SPACE$(10);
PRINT "F6  - sleep                       F6  - charm monster"
PRINT SPACE$(10);
PRINT "F7  - locate                      F7  - continual light"
PRINT SPACE$(10);
PRINT "F8  - pyrotechnics                F8  - passwall"
PRINT SPACE$(10);
PRINT "F9  - firestorm                   F9  - demon summon"
PRINT SPACE$(10);
PRINT "F10 - transmute                   F10 - doomkill"
PRINT
PRINT SPACE$(10);
PRINT "Spell Level 2  -  Control-        Spell Level 4  -  Alt-"
PRINT SPACE$(10);
PRINT "F1  - restoration                 F1  - resurrection"
PRINT SPACE$(10);
PRINT "F2  - scare                       F2  - fear"
PRINT SPACE$(10);
PRINT "F3  - fireball                    F3  - disintegrate"
PRINT SPACE$(10);
PRINT "F4  - obscurement                 F4  - blade barrier"
PRINT SPACE$(10);
PRINT "F5  - power shield                F5  - enchanted weapon"
PRINT SPACE$(10);
PRINT "F6  - wall of will                F6  - hold monster"
PRINT SPACE$(10);
PRINT "F7  - light                       F7  - teleport"
PRINT SPACE$(10);
PRINT "F8  - levitate                    F8  - polymorph"
PRINT SPACE$(10);
PRINT "F9  - monster summon              F9  - time stop"
PRINT SPACE$(10);
PRINT "F10 - finger of death             F10 - genocide";
GOSUB Key.Press
COLOR 14, 1 ' blue background
CLS ' clear screen
PRINT SPACE$(10);
PRINT "Wands                             Staffs"
PRINT SPACE$(10);
PRINT "Alt-A  - magic missile            Control-A  - locate"
PRINT SPACE$(10);
PRINT "Alt-B  - pyrotechnics             Control-B  - transmute"
PRINT SPACE$(10);
PRINT "Alt-C  - firestorm                Control-C  - light"
PRINT SPACE$(10);
PRINT "Alt-D  - fireball                 Control-D  - levitate"
PRINT SPACE$(10);
PRINT "Alt-E  - finger of death          Control-E  - continual light"
PRINT SPACE$(10);
PRINT "Alt-F  - lightning bolt           Control-F  - passwall"
PRINT SPACE$(10);
PRINT "Alt-G  - doomkill                 Control-G  - fear"
PRINT SPACE$(10);
PRINT "Alt-H  - disintegrate             Control-H  - enchanted weapon"
PRINT SPACE$(10);
PRINT "Alt-I  - blade barrier            Control-I  - teleport"
PRINT SPACE$(10);
PRINT "Alt-J  - genocide                 Control-J  - time stop"
PRINT
PRINT SPACE$(10);
PRINT "Potions                           Rings"
PRINT SPACE$(10);
PRINT "Alt-Q  - heal                     Control-Q  - sleep"
PRINT SPACE$(10);
PRINT "Alt-R  - protection               Control-R  - scare"
PRINT SPACE$(10);
PRINT "Alt-S  - shield                   Control-S  - obscurement"
PRINT SPACE$(10);
PRINT "Alt-T  - strength                 Control-T  - monster summon"
PRINT SPACE$(10);
PRINT "Alt-U  - restoration              Control-U  - forget"
PRINT SPACE$(10);
PRINT "Alt-V  - power shield             Control-V  - wall of force"
PRINT SPACE$(10);
PRINT "Alt-W  - wall of will             Control-W  - charm monster"
PRINT SPACE$(10);
PRINT "Alt-X  - regeneration             Control-X  - demon summon"
PRINT SPACE$(10);
PRINT "Alt-Y  - phantasmal force         Control-Y  - hold monster"
PRINT SPACE$(10);
PRINT "Alt-Z  - resurrection             Control-Z  - polymorph";
GOSUB Key.Press
COLOR 14, 1 ' blue background
CLS ' clear screen
PRINT
PRINT
PRINT
PRINT
PRINT
PRINT SPACE$(15);
PRINT "The Alt-0 to Alt-9 are the Power Globe keys"
PRINT
PRINT
PRINT
PRINT SPACE$(15);
PRINT "Alt-0 = clear level       Alt-5 = use all wands"
PRINT SPACE$(15);
PRINT "Alt-1 = replenish level   Alt-6 = use all potions"
PRINT SPACE$(15);
PRINT "Alt-2 = down 10 levels    Alt-7 = use all staffs"
PRINT SPACE$(15);
PRINT "Alt-3 = up 10 levels      Alt-8 = use all rings"
PRINT SPACE$(15);
PRINT "Alt-4 = collect all       Alt-9 = trade all"
PRINT
PRINT SPACE$(10);
PRINT "There is 1% chance of finding the Power Globe in a chest.";
GOSUB Key.Press
COLOR 7 ' change color
RETURN ' exit routine

' displays screen of various monster, dungeon level, and player statistics
Display.Info:
GOSUB Store.Dungeon.Level ' store dungeon screen
COLOR 14, 1 ' blue background
CLS ' clear screen
GOSUB Info.Screen ' display screen
' display players combined armors plus
LOCATE 13, 25, 0 ' position cursor
x = 0
FOR i = 8 TO 11
    x = x + GetUserRecordStats2(i)
NEXT
PRINT STR$(x);
LOCATE 14, 26, 0 ' position cursor
PRINT STR$(GetUserRecordStats2(7)); ' display players weapon plus
' variables store dungeon level minimums, maximums, and averages
min.mlvl# = 0 ' minimum monster level
max.mlvl# = 0 ' monster maximum level
tot.mlvl# = 0 ' monster average level
min.hp# = 0 ' monster minimum hit points
max.hp# = 0 ' monster maximum hit points
tot.hp# = 0 ' monsters total hit points
min.ex# = 0 ' monsters minimum experience
max.ex# = 0 ' monsters maximum experience
tot.ex# = 0 ' monsters total experience
min.hits# = 0 ' monsters minimum hits
max.hits# = 0 ' monsters maximum hits
tot.hits# = 0 ' monsters total hits
FOR i = 1 TO 10 ' total 10 loops of statistics, calculating minimum and maximum
    ' random dungeon level monsters level
    mmlvl# = INT(RND * dlvl# + dlvl# / 2 + 1)
    IF min.mlvl# = 0 THEN ' set minimum monsters level
        min.mlvl# = mmlvl#
    END IF
    IF mmlvl# < min.mlvl# THEN ' reset minimum monsters level
        min.mlvl# = mmlvl#
    END IF
    IF mmlvl# > max.mlvl# THEN ' reset maximum monsters level
        max.mlvl# = mmlvl#
    END IF
    tot.mlvl# = tot.mlvl# + mmlvl# ' increase total monster level
    ' random dungeon level monsters hit points
    mhp# = INT((RND * dlvl# * 2 + 1) * mmlvl# ^ 2 / 2 + dlvl# * 2 + RND * mmlvl# * 50)
    IF min.hp# = 0 THEN ' set minimum monsters hit points
        min.hp# = mhp#
    END IF
    IF mhp# < min.hp# THEN ' reset minimum monsters hit points
        min.hp# = mhp#
    END IF
    IF mhp# > max.hp# THEN ' reset maximum monsters hit points
        max.hp# = mhp#
    END IF
    tot.hp# = tot.hp# + mhp# ' increase total monster hit points
    ' random dungeon level monsters experience
    mex# = INT(mhp# / mmlvl#) * INT(dlvl# / 2 + 1) + INT(RND * mhp#)
    IF min.ex# = 0 THEN ' set minimum monsters experience
        min.ex# = mex#
    END IF
    IF mex# < min.ex# THEN ' reset minimum monsters experience
        min.ex# = mex#
    END IF
    IF mex# > max.ex# THEN ' reset maximum monsters experience
        max.ex# = mex#
    END IF
    tot.ex# = tot.ex# + mex# ' increase total monster experience
    ' random dungeon level monsters hits
    mhs# = 0
    FOR j = 8 TO 11
        mhs# = mhs# + GetUserRecordStats2(j)
    NEXT
    mhits# = INT(RND * mhp# / (mhs# + 1) + mmlvl#)
    IF min.hits# = 0 THEN ' set minimum monsters hits
        min.hits# = mhits#
    END IF
    IF mhits# < min.hits# THEN ' reset minimum monsters hits
        min.hits# = mhits#
    END IF
    IF mhits# > max.hits# THEN ' reset maximum monsters hits
        max.hits# = mhits#
    END IF
    tot.hits# = tot.hits# + mhits# ' increase total monster hits
NEXT
COLOR 12 ' change color
LOCATE 20, 10, 0 ' position cursor
PRINT STR$(min.mlvl#); ' display monster minimum level
PRINT STRING$(15 - LEN(STR$(min.mlvl#)), 32); ' pad with spaces
PRINT STR$(min.hp#); ' display monster minimum hit points
PRINT STRING$(15 - LEN(STR$(min.hp#)), 32); ' pad with spaces
PRINT STR$(min.ex#); ' display monster minimum experience
PRINT STRING$(15 - LEN(STR$(min.ex#)), 32); ' pad with spaces
PRINT STR$(min.hits#); ' display monster minimum hits
LOCATE 21, 10, 0 ' position cursor
PRINT STR$(max.mlvl#); ' display monster maximum level
PRINT STRING$(15 - LEN(STR$(max.mlvl#)), 32); ' pad with spaces
PRINT STR$(max.hp#); ' display monster maximum hit points
PRINT STRING$(15 - LEN(STR$(max.hp#)), 32); ' pad with spaces
PRINT STR$(max.ex#); ' display monster maximum experience
PRINT STRING$(15 - LEN(STR$(max.ex#)), 32); ' pad with spaces
PRINT STR$(max.hits#); ' display monster maximum hits
i# = INT(tot.mlvl# / 10 + .5) ' average monster total level
LOCATE 22, 10, 0 ' position cursor
PRINT STR$(i#); ' display average total
PRINT STRING$(15 - LEN(STR$(i#)), 32); ' pad with spaces
i# = INT(tot.hp# / 10 + .5) ' average monster total hit points
PRINT STR$(i#); ' display average total
PRINT STRING$(15 - LEN(STR$(i#)), 32); ' pad with spaces
i# = INT(tot.ex# / 10 + .5) ' average monster total experience
PRINT STR$(i#); ' display average total
PRINT STRING$(15 - LEN(STR$(i#)), 32); ' pad with spaces
PRINT STR$(INT(tot.hits# / 10 + .5)); ' average monster total hits
COLOR 14 ' change color
x# = GetUserRecordStats1(2)
y# = 2 ^ x#
LOCATE 15, 37, 0 ' position cursor
PRINT STR$(y#);
x# = x# + 6
y# = 2 ^ x#
LOCATE 16, 43, 0 ' position cursor
PRINT STR$(y#);
GOSUB Key.Press ' pause for keystroke
GOSUB Restore.Screen ' redraw dungeon screen
RETURN ' exit routine

Info.Screen:
COLOR 14 ' change color
PRINT "Mlvl=Int(Rnd*Dlvl+Dlvl/2+1)"
PRINT "Mhp=Int((Rnd*Dlvl*2+1)*Mlvl^2/2+Dlvl*2+Rnd*Mlvl+50)"
PRINT "Exp=Int(Hp/Mlvl)*Int(Dlvl/2+1)+Int(Rnd*Hp)"
PRINT "Mhits=Int(Rnd*(Mhp/Armor)+Mlvl)"
PRINT "Hits=Int(Rnd*Strength+Level/2)*Bonus+Weapon+Dlvl"
PRINT
COLOR 11 ' change color
PRINT "Where Mlvl is the monster level, Dlvl is the dungeon level, Mhp is the monster"
PRINT "hit points, Exp is the monsters experience, Armor is your combined armor plus,"
PRINT "Weapon is your weapon plus, Strength is your strength, Level is your expertise"
PRINT "level, Hits is the number of points damage your can do to a monster, and Mhits"
PRINT "is the number of points damage a monster can do to you."
COLOR 15 ' change color
LOCATE 13, 1, 0 ' position cursor
PRINT "You now an armor plus of";
LOCATE 14, 1, 0 ' position cursor
PRINT "You have a weapon plus of";
LOCATE 15, 1, 0 ' position cursor
PRINT "Gold needed to reach the next level:";
LOCATE 16, 1, 0 ' position cursor
PRINT "Experience needed to reach the next level:";
LOCATE 17, 1, 0 ' position cursor
PRINT "On this dungeon level, monsters have the following statistics:";
LOCATE 19, 10, 0 ' position cursor
COLOR 14 ' change color
PRINT "Level         Hit points    Experience     Combat hits";
LOCATE 20, 1, 0 ' position cursor
PRINT "min.";
LOCATE 21, 1, 0 ' position cursor
PRINT "max.";
LOCATE 22, 1, 0 ' position cursor
PRINT "avg.";
RETURN ' exit routine

' randomize coordinate increments
Get.Random.Direction:
SELECT CASE RND ' random value for selecting nearby x-coordinate cell
    CASE IS <= .3
        xo = -1 ' cell to the north
    CASE IS >= .7
        xo = 1 ' cell to the south
    CASE ELSE
        xo = 0 ' neither
END SELECT
SELECT CASE RND ' random value for selecting nearby y-coordinate cell
    CASE IS <= .3
        yo = -1 ' cell to the west
    CASE IS >= .7
        yo = 1 ' cell to the east
    CASE ELSE
        yo = 0 ' neither
END SELECT
Coor.Direction = 0
IF cxco + xo > 1 THEN ' x-coordinates boundary
    IF cxco + xo < 22 THEN ' x-coordinate boundary
        IF cyco + yo > 1 THEN ' y-coordinate boundary
            IF cyco + yo < 76 THEN ' y-coordinate boundary
                IF NOT (xo = 0 AND yo = 0) THEN ' not original xy-coordinate
                    Coor.Direction = -1
                END IF
            END IF
        END IF
    END IF
END IF
RETURN ' exit routine

' cast spell type 1, decrement spell points
Cast.Spell.Type1:
z1 = GetUserRecordStats2(13) ' decrement spell points
z1 = z1 - 1
CALL PutUserRecordStats2(13, z1) ' decrement spell points
IF GetUserRecordStats2(13) < 0 THEN ' not enough remaining to cast spell,
    CALL PutUserRecordStats2(13, 0) ' reset spell point counter to zero
    RETURN ' exit routine
END IF
' cast spell type 1
Cast.Spell.Type1a:
GOSUB Cast.Spell.Message ' display spell name being cast
SELECT CASE sp ' select spell number 1 to 10
    CASE 1 ' heal
        GOSUB Increment.Hits ' update players combat hits
    CASE 2 ' protection
        mhpm = GetUserRecordStats2(5) ' set monster hit point minus amount to dexterity
        mhpmc = 1 ' monster hit point minus counter
    CASE 3 ' magic missile
        bonus = 2 ' multiplier
        GOSUB Attack.Nearby.Monsters ' attack nearby monsters
        GOSUB Monster.Attack
        GOSUB Encounter.Cells
    CASE 4 ' shield
        shp = GetUserRecordStats2(9) ' shield amount equal to players armor
        shpc = 1 ' shield counter
    CASE 5 ' strength
        stp = GetUserRecordStats2(1) ' strength amount equal to players strength
        stpc = 1 ' strength counter
    CASE 6 ' sleep
        slp = 1 ' sleep flag
        slpc = 1 ' sleep counter
    CASE 7 ' locate
        ' find traps/pits
        i = 0 ' traps flag
        j = 0 ' pits flag
        ' check surrounding cells
        IF GetDungeonRecordLevelData(xco - 1, yco) = 94 THEN ' trap to the north
            COLOR 15 ' change color
            LOCATE xco - 1, yco, 0 ' position cursor
            PRINT "^"; ' display nearby trap symbol
            i = -1 ' set trap flag
        END IF
        IF GetDungeonRecordLevelData(xco + 1, yco) = 94 THEN ' trap to the south
            COLOR 15 ' change color
            LOCATE xco + 1, yco, 0 ' position cursor
            PRINT "^"; ' display nearby trap
            i = -1 ' set trap flag
        END IF
        IF GetDungeonRecordLevelData(xco, yco - 1) = 94 THEN ' trap to the west
            COLOR 15 ' change color
            LOCATE xco, yco - 1, 0 ' position cursor
            PRINT "^"; ' display trap
            i = -1 ' set trap flag
        END IF
        IF GetDungeonRecordLevelData(xco, yco + 1) = 94 THEN ' trap to the east
            COLOR 15 ' change color
            LOCATE xco, yco + 1, 0 ' position cursor
            PRINT "^"; ' display trap
            i = -1 ' set trap flag
        END IF
        IF GetDungeonRecordLevelData(xco - 1, yco) = 111 THEN ' pit to the north
            COLOR 15 ' change color
            LOCATE xco - 1, yco, 0 ' position cursor
            PRINT "o"; ' display pit
            j = -1 ' set pit flag
        END IF
        IF GetDungeonRecordLevelData(xco + 1, yco) = 111 THEN ' pit to the south
            COLOR 15 ' change color
            LOCATE xco + 1, yco, 0 ' position cursor
            PRINT "o"; ' display pit
            j = -1 ' set pit flag
        END IF
        IF GetDungeonRecordLevelData(xco, yco - 1) = 111 THEN ' pit to the west
            COLOR 15 ' change color
            LOCATE xco, yco - 1, 0 ' position cursor
            PRINT "o"; ' display pit
            j = -1 ' set pit flag
        END IF
        IF GetDungeonRecordLevelData(xco, yco + 1) = 111 THEN ' pit to the east
            COLOR 15 ' change color
            LOCATE xco, yco + 1, 0 ' position cursor
            PRINT "o"; ' display pit
            j = -1 ' set pit flag
        END IF
        IF i = 0 AND j = 0 THEN ' no traps or pits
            COLOR 14 ' change color
            a$ = "There are no traps or pits here.."
        ELSE ' traps and/or pits
            COLOR 11 ' change color
            a$ = "You discovered some "
            IF i AND j THEN ' both traps and pits
                a$ = a$ + "traps and pits.."
            ELSE ' traps or pits
                IF i THEN ' only traps
                    a$ = a$ + "traps.."
                ELSE ' only pits
                    a$ = a$ + "pits.."
                END IF
            END IF
        END IF
        GOSUB Display.Message ' output message
    CASE 8 ' pyrotechnics
        bonus = 2 ' multiplier
        GOSUB Attack.Nearby.Monsters ' attack nearby monsters
        GOSUB Monster.Attack
        GOSUB Encounter.Cells
    CASE 9 ' firestorm
        bonus = 2 ' multiplier
        GOSUB Attack.Nearby.Monsters ' attack nearby monsters
        GOSUB Monster.Attack
        GOSUB Encounter.Cells
    CASE 10 ' transmute
        ' convert surrounding cells to gold
        IF GetDungeonRecordLevelData(xco - 1, yco) = 46 AND xco > 1 THEN
            COLOR 15 ' change color
            LOCATE xco - 1, yco, 0 ' position cursor
            PRINT "$"; ' display gold
            CALL PutDungeonRecordLevelData(xco - 1, yco, 36) ' set dungeon cell to gold
            IF GetDungeonRecordStats(12) = 0 THEN ' all dungeon level coins not found yet
                ' increment number of coins in dungeon level
                z = GetDungeonRecordStats(11)
                z = z + 1
                CALL PutDungeonRecordStats(11, z)
            END IF
        END IF
        IF GetDungeonRecordLevelData(xco + 1, yco) = 46 AND xco < 23 THEN
            COLOR 15 ' change color
            LOCATE xco + 1, yco, 0 ' position cursor
            PRINT "$"; ' display gold
            CALL PutDungeonRecordLevelData(xco + 1, yco, 36) ' set dungeon cell to gold
            IF GetDungeonRecordStats(12) = 0 THEN ' all dungeon level coins not found yet
                ' increment number of coins in dungeon level
                z = GetDungeonRecordStats(11)
                z = z + 1
                CALL PutDungeonRecordStats(11, z)
            END IF
        END IF
        IF GetDungeonRecordLevelData(xco, yco - 1) = 46 AND yco > 1 THEN
            COLOR 15 ' change color
            LOCATE xco, yco - 1, 0 ' position cursor
            PRINT "$"; ' display gold
            CALL PutDungeonRecordLevelData(xco, yco - 1, 36) ' set dungeon cell to gold
            IF GetDungeonRecordStats(12) = 0 THEN ' all dungeon level coins not found yet
                ' increment number of coins in dungeon level
                z = GetDungeonRecordStats(11)
                z = z + 1
                CALL PutDungeonRecordStats(11, z)
            END IF
        END IF
        IF GetDungeonRecordLevelData(xco, yco + 1) = 46 AND yco < 76 THEN
            COLOR 15 ' change color
            LOCATE xco, yco + 1, 0 ' position cursor
            PRINT "$"; ' display gold
            CALL PutDungeonRecordLevelData(xco, yco + 1, 36) ' set dungeon cell to gold
            IF GetDungeonRecordStats(12) = 0 THEN ' all dungeon level coins not found yet
                ' increment number of coins in dungeon level
                z = GetDungeonRecordStats(11)
                z = z + 1
                CALL PutDungeonRecordStats(11, z)
            END IF
        END IF
END SELECT
RETURN ' exit routine

' cast spell type 2, decrement spell points
Cast.Spell.Type2:
z1 = GetUserRecordStats2(14)
z1 = z1 - 2 ' decrement spell points
CALL PutUserRecordStats2(14, z1) ' decrement spell points
IF GetUserRecordStats2(14) < 0 THEN ' not enough remaining to cast spell,
    CALL PutUserRecordStats2(14, 0) ' reset spell point counter to zero
    RETURN ' exit routine
END IF
' cast spell type2
Cast.Spell.Type2a:
GOSUB Cast.Spell.Message ' display spell name being cast
SELECT CASE sp ' select spell 1 to 10
    CASE 1 ' restoration
        ' increment players hit points by constitution times two
        z1# = GetUserRecordStats1(4)
        z1# = z1# + GetUserRecordStats2(4)
        CALL PutUserRecordStats1(4, z1#)
        GOSUB Increment.Hits ' update players combat hits
    CASE 2 ' scare
        mhpm = GetUserRecordStats2(5) ' set monster hit point minus amount to dexterity
        mhpmc = 2 ' monster hit point minus counter
    CASE 3 ' fireball
        bonus = 3 ' multiplier
        GOSUB Attack.Nearby.Monsters ' attack nearby monsters
        GOSUB Monster.Attack
        GOSUB Encounter.Cells
    CASE 4 ' obscurement
        shp = GetUserRecordStats2(9) ' shield amount equal to players armor
        shpc = 2 ' shield counter
    CASE 5 ' power shield
        stp = GetUserRecordStats2(1) ' strength amount equal to players strength
        stpc = 2 ' strength counter
    CASE 6 ' wall of will
        slp = 1 ' sleep flag
        slpc = 2 ' sleep counter
    CASE 7 ' light
        ll = 3 ' set extended cell display amount
        llc = 1 ' set extended cell display counter
        GOSUB Cell.Action ' compute nearby cells
    CASE 8 ' levitate
        DO ' loop until empty cell to move player
            FOR k = 1 TO INT(RND * 4 + 2) ' move player random times
                c = 0 ' loop counter
                DO ' find empty cell
                    c = c + 1 ' increment counter
                    IF c > 10 THEN ' avoid infinite loop
                        EXIT FOR ' exit routine
                    END IF
                    cxco = xco ' store x-coordinate
                    cyco = yco ' store y-coordinate
                    GOSUB Get.Random.Direction ' get valid random direction
                    IF Coor.Direction THEN ' returned valid random direction
                        EXIT DO ' found valid random direction
                    END IF
                LOOP
                ' move to selected cell
                xco = xco + xo ' increment players x coordinate
                yco = yco + yo ' increment players y coordinate
                GOSUB Update.Player ' update nearby cell actions
            NEXT ' get next levitate move
            IF GetDungeonRecordLevelData(xco, yco) = 46 THEN ' move to an dungeon dot
                EXIT DO ' end move player
            END IF
        LOOP
    CASE 9 ' monster summon
        FOR k = 1 TO INT(RND * 4 + 2) ' encounter random number of monsters
            c = 0 ' loop counter
            DO ' loop until empty nearby cell for monster
                c = c + 1 ' increment loop counter
                IF c > 10 THEN ' avoid infinite loop
                    EXIT FOR ' exit more than 10 loops
                END IF
                cxco = xco ' store x-coordinate
                cyco = yco ' store y-coordinate
                GOSUB Get.Random.Direction ' get valid random direction
                IF Coor.Direction THEN ' returned valid random direction
                    IF GetDungeonRecordLevelData(xco + xo, yco + yo) = 46 THEN ' empty cell
                        EXIT DO ' fond empty cell at random direction
                    END IF
                END IF
            LOOP
            mxco = xco + xo ' set monster xy
            myco = yco + yo ' set monster xy
            ' increment number of monsters on level
            z = GetDungeonRecordStats(6)
            z = z + 1
            CALL PutDungeonRecordStats(6, z)
            ' set dungeon array monster xy to monster symbol
            CALL PutDungeonRecordLevelData(mxco, myco, 35)
            COLOR 15 ' change color
            LOCATE mxco, myco, 0 ' position cursor
            PRINT "#"; ' display monster symbol
            mn = INT(RND * 100 + 1) ' select random monster
            GOSUB Encounter.Monster ' get the monster statistics
        NEXT ' get next monster
    CASE 10 ' finger of death
        GOSUB Cell.Monster.Attack ' all nearby monsters die
END SELECT
RETURN ' exit routine

' cast spell type 3, decrement spell points
Cast.Spell.Type3:
z1 = GetUserRecordStats2(15)
z1 = z1 - 3 ' decrement spell points
CALL PutUserRecordStats2(15, z1) ' decrement spell points
IF GetUserRecordStats2(15) < 0 THEN ' not enough remaining to cast spell,
    CALL PutUserRecordStats2(15, 0) ' reset spell point counter to zero
    RETURN ' exit routine
END IF
' cast spell type3
Cast.Spell.Type3a:
GOSUB Cast.Spell.Message ' display spell name being cast
SELECT CASE sp ' select spell 1 to 10
    CASE 1 ' regeneration
        ' increase players hit points by constitution times three
        z1# = GetUserRecordStats2(4) * 2
        CALL PutUserRecordStats1(4, z1#)
        GOSUB Increment.Hits ' reset any counters
    CASE 2 ' forget
        ' monster hit point minus amount equal to players dexterity
        mhpm = GetUserRecordStats2(5)
        mhpmc = 3 ' monster hit point minus counter
    CASE 3 ' lightning bolt
        bonus = 4 ' multiplier
        GOSUB Attack.Nearby.Monsters ' attack nearby monsters
        GOSUB Monster.Attack
        GOSUB Encounter.Cells
    CASE 4 ' phantasmal force
        shp = GetUserRecordStats2(9) ' shield amount equal to players armor
        shpc = 3 ' shield counter
    CASE 5 ' wall of force
        stp = GetUserRecordStats2(1) ' strength amount equal to players strength
        stpc = 3 ' strength counter
    CASE 6 ' charm monster
        slp = 1 ' sleep flag
        slpc = 3 ' sleep counter
    CASE 7 ' continual light
        ll = 2 ' set extended cell display amount
        llc = 4 ' set extended cell display counter
        GOSUB Cell.Action ' compute nearby cells
    CASE 8 ' passwall
        IF pwl = 1 THEN ' already in passwall mode
            RETURN ' exit routine
        END IF
        pwl = 1 ' set passwall flag
        FOR q = 1 TO INT(RND * 4 + 2) ' random amount of passwall moves
            GOSUB Move.Passwall ' separate input loop for passwall
        NEXT ' to move through dungeon walls
        ' continue passwall if loop ended in dungeon wall
        WHILE GetDungeonRecordLevelData(xco, yco) = 176
            GOSUB Move.Passwall ' move until out of dungeon wall
        WEND ' end passwall
        pwl = 0 ' reset passwall flag
        RETURN ' exit routine
    CASE 9 ' demon summon
        FOR k = 1 TO INT(RND * 4 + 2) ' encounter random number of demons
            c = 0 ' loop counter
            DO ' loop until empty nearby cell for demon
                c = c + 1 ' increment loop counter
                IF c > 10 THEN ' avoid infinite loop
                    EXIT FOR ' exit more than ten loops
                END IF
                cxco = xco ' store x-coordinate
                cyco = yco ' store y-coordinate
                GOSUB Get.Random.Direction ' get valid random direction
                IF Coor.Direction THEN ' returned valid random direction
                    IF GetDungeonRecordLevelData(xco + xo, yco + yo) = 46 THEN ' empty cell
                        EXIT DO ' found empty cell at random direction
                    END IF
                END IF
            LOOP
            ' increment number of monsters on level
            z = GetDungeonRecordStats(6)
            z = z + 1
            CALL PutDungeonRecordStats(6, z)
            mxco = xco + xo ' set monster xy
            myco = yco + yo ' set monster xy
            ' set dungeon array monster xy to monster symbol
            CALL PutDungeonRecordLevelData(mxco, myco, 35)
            COLOR 15 ' change color
            LOCATE mxco, myco, 0 ' position cursor
            PRINT "#"; ' display monster symbol
            mn = 101 ' monster number 54 is demon in data
            GOSUB Encounter.Monster ' get the monster statistics
        NEXT ' get next demon
    CASE 10 ' doomkill
        GOSUB Cell.Monster.Attack ' all nearby monsters die
END SELECT
RETURN ' exit routine

' cast spell type 4, decrement spell points
Cast.Spell.Type4:
z1 = GetUserRecordStats2(16)
z1 = z1 - 4 ' decrement spell points
CALL PutUserRecordStats2(16, z1) ' decrement spell points
IF GetUserRecordStats2(16) < 0 THEN ' not enough remaining to cast spell,
    CALL PutUserRecordStats2(16, 0) ' reset spell point counter to zero
    RETURN ' exit routine
END IF
' cast spell type4
Cast.Spell.Type4a:
GOSUB Cast.Spell.Message ' display spell name being cast
SELECT CASE sp ' select spell 1 to 10
    CASE 1 ' resurrection
        ' increase players hit points by constitution times four
        z1# = GetUserRecordStats2(4) * 3
        CALL PutUserRecordStats1(4, z1#)
        GOSUB Increment.Hits
    CASE 2 ' fear
        ' monster hit point minus amount equal to players dexterity
        mhpm = GetUserRecordStats2(5)
        mhpmc = 4 ' monster hit point minus counter
    CASE 3 ' disintegrate
        bonus = 5 ' multiplier
        GOSUB Attack.Nearby.Monsters ' attack nearby monsters
        GOSUB Monster.Attack
        GOSUB Encounter.Cells
    CASE 4 ' blade barrier
        shp = GetUserRecordStats2(9) ' shield amount equal to armor
        shpc = 4 ' shield counter
    CASE 5 ' enchanted weapon
        stp = GetUserRecordStats2(1) ' strength amount equal to players strength
        stpc = 4 ' strength counter
    CASE 6 ' hold monster
        slp = 1 ' sleep flag
        slpc = 4 ' sleep counter
    CASE 7 ' teleport
        GOSUB Teleport ' teleport player
    CASE 8 ' polymorph
        GOSUB Cell.Monster.Attack ' all nearby monsters die
    CASE 9 ' time stop
        pw = 1 ' pass monsters flag
        pwc = 5 ' pass monsters counter
    CASE 10 ' genocide
        gd = 1 ' set extended monster cell death flag
        gdc = 4 ' set extended monster cell death counter
        GOSUB Cell.Monster.Attack ' all nearby monsters die
END SELECT
RETURN ' exit routine

' passwall input routine
Move.Passwall:
DO ' get a keystroke
    GOSUB Get.Keystroke ' only keystrokes allowed 0 to 9, and
LOOP UNTIL (x >= 0 AND x <= 9) OR LEN(X$) = 2 ' extended ascii
xo = xco ' store players x-coordinate
yo = yco ' store players y-coordinate
SELECT CASE LEN(X$) ' process input
    CASE 1 ' input length
        SELECT CASE X$ ' select input key
            CASE "1" ' attack nearby monsters
                GOSUB Attack.Nearby.Monsters
                GOSUB Monster.Attack
                GOSUB Encounter.Cells
            CASE "2" ' passwall move up
                xo = xo + 1 ' increment temporary variable
                IF xo < 23 THEN ' check array bounds
                    xco = xo ' update players x-coordinate
                    yco = yo ' update players y-coordinate
                    GOSUB Update.Player ' update nearby cell actions
                END IF
            CASE "3" ' cast spell
                GOSUB Cast.Chant
            CASE "4" ' passwall move left
                yo = yo - 1 ' decrement temporary variable
                IF yo > 1 THEN ' check array bounds
                    xco = xo ' update players x-coordinate
                    yco = yo ' update players y-coordinate
                    GOSUB Update.Player ' update nearby cell actions
                END IF
            CASE "5", CHR$(13) ' enter/accept key/null action
                GOSUB Update.Hits
            CASE "6" ' passwall move right
                yo = yo + 1 'increment temporary variable
                IF yo < 76 THEN ' check array bouns
                    xco = xo ' update players x-coordinate
                    yco = yo ' update players y-coordinate
                    GOSUB Update.Player ' update nearby cell actions
                END IF
            CASE "7" ' evade monster
                GOSUB Evade.Monster
            CASE "8" ' passwall move down
                xo = xo - 1 ' decrement temporary variable
                IF xo > 1 THEN ' check array bounds
                    xco = xo ' update players x-coordinate
                    yco = yo ' update players y-coordinate
                    GOSUB Update.Player ' update nearby cell actions
                END IF
            CASE "9" ' ignore key/null action/update hits
                GOSUB Update.Hits
        END SELECT
    CASE 2 ' input length
        sp = ASC(MID$(X$, 2)) ' convert extended ascii input
        SELECT CASE sp ' select second input key,
            CASE 79 ' 1, attack nearby monsters
                GOSUB Attack.Nearby.Monsters
                GOSUB Monster.Attack
                GOSUB Encounter.Cells
            CASE 80 ' 2, passwall move player down
                xo = xo + 1 ' increment temporary variable
                IF xo < 23 THEN ' check array bounds
                    xco = xo ' update players x-coordinate
                    yco = yo ' update players y-coordinate
                    GOSUB Update.Player ' update nearby cell actions
                END IF
            CASE 81 ' 3, cast spell
                GOSUB Cast.Chant
            CASE 75 ' 4, passwall move player left
                yo = yo - 1 ' decrement temporary variable
                IF yo > 1 THEN ' check array bounds
                    xco = xo ' update players x-coordinate
                    yco = yo ' update players y-coordinate
                    GOSUB Update.Player ' update nearby cell actions
                END IF
            CASE 76 ' 5, enter/accept key/null action
                GOSUB Update.Hits
            CASE 77 ' 6, passwall move player right
                yo = yo + 1 ' increment temporary variable
                IF yo < 76 THEN ' check array bounds
                    xco = xo ' update players x-coordinate
                    yco = yo ' update players y-coordinate
                    GOSUB Update.Player ' update nearby cell actions
                END IF
            CASE 71 ' 7, evade monster
                GOSUB Evade.Monster
            CASE 72 ' 8, passwall move player up
                xo = xo - 1 ' decrement temporary variable
                IF xo > 1 THEN ' check array bounds
                    xco = xo ' update players x-coordinate
                    yco = yo ' update players y-coordinate
                    GOSUB Update.Player ' update nearby cell actions
                END IF
            CASE 73 ' 9, ignore key/null action/update hits
                GOSUB Update.Hits
                ' F1 to F10, cast spell type 1
            CASE 59 TO 68
                spn = 1 ' set spell type
                sp = sp - 58 ' calculate spell number
                GOSUB Cast.Spell.Type1
                ' Control-F1 to F10, cast spell type 2
            CASE 94 TO 103
                spn = 2 ' set spell type
                sp = sp - 93 ' calculate spell number
                GOSUB Cast.Spell.Type2
                ' Shift-F1 to F10, cast spell type 3
            CASE 84 TO 93
                spn = 3 ' set spell type
                sp = sp - 83 ' calculate spell number
                GOSUB Cast.Spell.Type3
                ' Alt-F1 to F10, cast spell type 4
            CASE 104 TO 113
                spn = 4 ' set spell type
                sp = sp - 103 ' calculate spell number
                GOSUB Cast.Spell.Type4
        END SELECT
END SELECT
RETURN ' exit routine

' kill all nearby monsters
Cell.Monster.Attack:
FOR mn = 1 TO 24 ' loop through all eight surrounding cells
    IF Monsters(mn, 1) OR Monsters(mn, 2) THEN ' monster xy exists
        mxco = Monsters(mn, 1) ' get monster x-coordinate
        myco = Monsters(mn, 2) ' get monster y-coordinate
        ex# = Monsters(mn, 5) ' get monster experience
        Monsters(mn, 1) = 0 ' reset monster x-coordinate
        Monsters(mn, 2) = 0 ' reset monster y-coordinate
        GOSUB Update.Stats ' update players statistics with experience
        ' reset dungeon array monster to gold symbol
        CALL PutDungeonRecordLevelData(mxco, myco, 36)
        IF GetDungeonRecordStats(12) = 0 THEN ' all dungeon level coins not found yet
            z = GetDungeonRecordStats(11)
            z = z + 1
            CALL PutDungeonRecordStats(11, z) ' increment coins
        END IF
        COLOR 15 ' change color
        LOCATE mxco, myco, 0 ' position cursor
        PRINT "$"; ' display gold
        z = GetDungeonRecordStats(6)
        z = z - 1
        CALL PutDungeonRecordStats(6, z) ' decrement monsters
        IF GetDungeonRecordStats(6) = 0 THEN ' last monster counter
            IF GetDungeonRecordStats(5) = 0 THEN ' last monster in dungeon level flag
                CALL PutDungeonRecordStats(5, 1) ' set last monster in dungeon level flag
                ' calculate bonus experience for last monster
                ex# = (RND * dlvl# * 2 + 1) * GetUserRecordStats1(2) ^ 2 / 2
                ex# = ex# + dlvl# * 2 + RND * GetUserRecordStats1(2) * 50
                ex# = INT(ex#)
                COLOR 14 ' change color
                a$ = "You killed all the monsters on this level! You gain"
                a$ = a$ + STR$(ex#) + " experience!"
                GOSUB Display.Message ' output message
                GOSUB Update.Stats ' update players statistics wth experience
            END IF
        END IF
    END IF
NEXT ' get next monster
RETURN ' exit routine

' teleport player randomly in dungeon level to another cell
Teleport:
COLOR 7 ' display cell player will be teleported from
LOCATE xco, yco, 0 ' position cursor
PRINT CHR$(GetDungeonRecordLevelData(xco, yco)); ' display dungeon cell symbol
DO ' loop until player teleports to an empty cell
    txo = xco ' store players x-coordinate
    tyo = yco ' store players y-coordinate
    FOR k = 1 TO INT(RND * 2 + 2) ' random number of times to teleport
        c = 0 ' loop counter
        DO ' loop until empty nearby cell for demon
            c = c + 1 ' increment loop counter
            IF c > 10 THEN ' avoid infinite loop
                EXIT FOR ' exit more than ten loops
            END IF
            IF RND > .5 THEN ' select teleport up 50/50
                xo = -1 ' set x-coordinate increment
            ELSE ' teleport down 50/50
                xo = 1 ' set x-coordinate increment
            END IF
            IF RND > .5 THEN ' select teleport left 50/50
                yo = -1 ' set y-coordinate increment
            ELSE ' teleport right 50/50
                yo = 1 ' set y-coordinate increment
            END IF
            ' until players xy-coor plus selected xy is within bounds
            IF txo + xo > 1 THEN ' x-coordinate boundary
                IF txo + xo < 22 THEN ' x-coordinate boundary
                    IF tyo + yo > 1 THEN ' y-coordinate boundary
                        IF tyo + yo < 76 THEN ' y-coordinate boundary
                            f = 0 ' reset cell flag
                            IF (txo + xo) = xco THEN ' not players xy-coordinate
                                IF (tyo + yo) = yco THEN ' not players xy-coordinate
                                    f = -1 ' reset cell flag
                                END IF
                            END IF
                            IF f = 0 THEN ' check cell flag
                                EXIT DO
                            END IF
                        END IF
                    END IF
                END IF
            END IF
        LOOP
        txo = txo + xo ' update stored players x-coordinate
        tyo = tyo + yo ' update stored players y-coordinate
    NEXT ' get next cell
    ' exit loop if destination teleport cell is dungeon dot
    IF GetDungeonRecordLevelData(txo, tyo) = 46 THEN ' move to an dungeon dot
        EXIT DO ' end move player
    END IF
LOOP
xco = txo ' set players x-coordinate to destination x-coordinate
yco = tyo ' set players y-coordinate to destination y-coordinate
GOSUB Update.Player ' update nearby cell actions
RETURN ' exit routine

' display prompt, get keypress
Key.Press:
COLOR 13 ' change color
LOCATE 25, 35, 0 ' position cursor
PRINT "Press a key:"; ' display message
GOSUB Get.Key.Press ' get a single keystroke
RETURN ' exit routine

' get a single keystroke
Get.Key.Press:
Buf$ = "" ' reset typeahead buffer
WHILE INKEY$ = "" ' wait for any input
    _LIMIT 100
WEND ' wait
RETURN ' exit routine

' routine to buy player statistics with gold
Trade.Stats:
GOSUB Store.Dungeon.Level ' store dungeon screen
COLOR 14, 1 ' blue background
CLS ' clear screen
' make statistic trading screen
COLOR 14 ' change color
LOCATE 1, 25, 0 ' position cursor
PRINT "Trade Gold For Player Statistics";
COLOR 15 ' change color
LOCATE 10, 28, 0 ' position cursor
PRINT "(1) (2) (3) (4) (5) (6)";
COLOR 12 ' change color
LOCATE 11, 28, 0 ' position cursor
PRINT "Str Int Wis Con Dex Cha";
COLOR 13 ' change color
LOCATE 13, 15, 0 ' position cursor
PRINT "Purchasing Character Stats(1, 2, 3, 4, 5, or 6 to buy)";
COLOR 15 ' change color
LOCATE 25, 22, 0 ' position cursor
PRINT "Press <enter> to return to the Dungeon..";
' calculate cost of statistic
buy# = INT(RND * GetUserRecordStats1(2) + 5) * 50 + INT(RND * 100)
' display message of cost
i$ = "The average stat point is" + STR$(buy#) + " Gold."
i = 40 - INT(LEN(i$) / 2) ' center
COLOR 14 ' change color
LOCATE 14, i, 0 ' position cursor
PRINT i$;
COLOR 14 ' change color
LOCATE 15, 33, 0 ' position cursor
PRINT "Gold remaining:";
DO ' loop to select statistics
    COLOR 10 ' change color
    LOCATE 12, 29, 0 ' position cursor
    ' display players statistics
    FOR x = 1 TO 6
        PRINT RIGHT$(STR$(GetUserRecordStats2(x)), 2) + "  ";
    NEXT
    LOCATE 16, 1, 0 ' position cursor
    PRINT SPACE$(79);
    ' display players gold amount remaining
    i = 40 - LEN(STR$(GetUserRecordStats1(5))) / 2 ' center
    COLOR 15 ' change color
    LOCATE 16, i, 0 ' position cursor
    PRINT STR$(GetUserRecordStats1(5)); ' output gold amount
    IF GetUserRecordStats1(5) - buy# <= 0 THEN ' calculate player gold to stat cost
        GOSUB Clear.Status.Line2 ' clear status line area
        COLOR 14 ' change color
        LOCATE 24, 21, 0 ' position cursor
        PRINT "You do not have enough Gold remaining!";
        EXIT DO ' exit loop
    END IF
    DO ' get a keystroke
        GOSUB Get.Keystroke
    LOOP UNTIL (x >= 1 AND x <= 6) OR X$ = CHR$(13)
    ' presses 1 to 6 to buy statistic space to exit
    IF X$ = CHR$(13) THEN ' enter pressed
        GOSUB Clear.Status.Line2 ' clear status line area
        EXIT DO ' exit loop
    END IF
    z1 = GetUserRecordStats2(x)
    z1 = z1 + 1 ' increment statistic
    CALL PutUserRecordStats2(x, z1) ' increment statistic
    IF GetUserRecordStats2(x) < 100 THEN ' check valid statistic
        z1# = GetUserRecordStats1(5)
        z1# = z1# - buy# ' subtract cost from gold
        CALL PutUserRecordStats1(5, z1#) ' subtract cost from gold
    ELSE
        CALL PutUserRecordStats2(x, 99) ' reset statistic
    END IF
LOOP ' loop to get stat to buy
GOSUB Key.Press ' wait
GOSUB Restore.Screen ' redraw dungeon screen
RETURN ' exit routine

' routine to buy inventory items with gold
Buy.Inventory:
GOSUB Store.Dungeon.Level ' store dungeon screen
COLOR 14, 1 ' blue background
CLS ' clear screen
' make inventory buying screen
COLOR 14 ' change color
LOCATE 1, 25, 0 ' position cursor
PRINT "Buy Inventory Items With Gold";
COLOR 15 ' change color
LOCATE 10, 28, 0 ' position cursor
PRINT "(1) (2) (3) (4) (5) (6)";
COLOR 12 ' change color
LOCATE 11, 29, 0 ' position cursor
PRINT "WE  SH  AR  CL  HE  BO";
COLOR 13 ' change color
LOCATE 13, 15, 0 ' position cursor
PRINT "Purchasing Inventory Items(1, 2, 3, 4, 5, or 6 to buy)";
COLOR 15 ' change color
LOCATE 25, 22, 0 ' position cursor
PRINT "Press <enter> to return to the Dungeon..";
' calculate cost of inventory item
buy# = INT(RND * GetUserRecordStats1(2) + 5) * 50 + INT(RND * 100)
' display message of cost
i$ = "The average item point is" + STR$(buy#) + " Gold."
i = 40 - INT(LEN(i$) / 2) ' center
COLOR 14 ' change color
LOCATE 14, i, 0 ' position cursor
PRINT i$;
COLOR 14 ' change color
LOCATE 15, 33, 0 ' position cursor
PRINT "Gold remaining:";
DO ' loop to select item
    COLOR 10 ' change color
    LOCATE 12, 29, 0 ' position cursor
    ' display players statistics
    FOR x = 7 TO 12
        PRINT RIGHT$(STR$(GetUserRecordStats2(x)), 2) + "  ";
    NEXT
    LOCATE 16, 1, 0 ' position cursor
    PRINT SPACE$(79);
    ' display players gold amount remaining
    i = 40 - LEN(STR$(GetUserRecordStats1(5))) / 2 ' center
    COLOR 15 ' change color
    LOCATE 16, i, 0 ' position cursor
    PRINT STR$(GetUserRecordStats1(5)); ' output gold amount
    IF GetUserRecordStats1(5) - buy# <= 0 THEN ' calculate player gold to stat cost
        GOSUB Clear.Status.Line2 ' clear status line area
        COLOR 14 ' change color
        LOCATE 24, 21, 0 ' position cursor
        PRINT "You do not have enough Gold remaining!";
        EXIT DO ' exit loop
    END IF
    DO ' get a keystroke
        GOSUB Get.Keystroke
    LOOP UNTIL (x >= 1 AND x <= 6) OR X$ = CHR$(13)
    ' presses 1 to 6 to buy statistic space to exit
    IF X$ = CHR$(13) THEN ' enter pressed
        GOSUB Clear.Status.Line2 ' clear status line area
        EXIT DO ' exit loop
    END IF
    z1 = GetUserRecordStats2(x + 6)
    z1 = z1 + 1 ' increment inventory stat
    CALL PutUserRecordStats2(x + 6, z1) ' increment inventory stat
    IF GetUserRecordStats2(x + 6) < 100 THEN ' check valid statistic
        z1# = GetUserRecordStats1(5)
        z1# = z1# - buy# ' subtract cost from gold
        CALL PutUserRecordStats1(5, z1#) ' subtract cost from gold
    ELSE
        CALL PutUserRecordStats2(x + 6, 99) ' reset statistic
    END IF
LOOP ' loop to get stat to buy
GOSUB Key.Press ' wait
GOSUB Restore.Screen ' redraw dungeon screen
RETURN ' exit routine

' display screen of players learned spells
Learned.Spells:
GOSUB Store.Dungeon.Level ' store dungeon screen
COLOR 14, 1 ' blue background
CLS ' clear screen
' make learned spells screen
LOCATE 1, 15, 0 ' position cursor
COLOR 15 ' change color
PRINT "Your player has memorized the following spell chants:";
' restore data pointer to spell data
COLOR 14 ' change color
RESTORE SpellData
FOR i = 1 TO 40 ' loop through 40 spells
    READ b$, c$ ' read spell chant, spell name
    IF GetUserRecordSpell(i) THEN ' spells array element flag
        IF i > 20 THEN ' place first 20 on left side of screen
            LOCATE i - 17, 5, 0 ' position cursor
            PRINT b$; ' append spell name
        ELSE ' place second 20 on right side of screen
            LOCATE i + 3, 45, 0 ' position cursor
            PRINT b$; ' append spell name
        END IF
    END IF
NEXT ' get next spell
GOSUB Key.Press ' wait
GOSUB Restore.Screen ' redraw dungeon screen
RETURN ' exit routine

' display screen of players inventory
Inventory:
GOSUB Store.Dungeon.Level ' store dungeon screen
COLOR 14, 1 ' blue background
CLS ' clear screen
' make inventory display screen
COLOR 15 ' change color
LOCATE 1, 15, 0 ' position cursor
PRINT "Your player has collected the following items:";
' display wands and potions on left side of screen
COLOR 10 ' change color
LOCATE 2, 5, 0 ' position cursor
PRINT "Key: Alt-"; ' display key command for wands
LOCATE 2, 45, 0 ' position cursor
PRINT "Key: Alt-"; ' display key command for potions
COLOR 14 ' change color
RESTORE WandData ' restore data pointer to wand data
FOR i = 1 TO 10 ' loop through 10 wands
    READ b$, x ' read wand name, wand spell number
    IF GetUserRecordWand(i) THEN ' check wands array element flag
        LOCATE i + 2, 5, 0 ' position cursor
        PRINT "(" + CHR$(i + 64) + ")wand of " + b$; ' append wand name
    END IF
NEXT
RESTORE PotionData ' restore data pointer to potion data
FOR i = 1 TO 10 ' loop through 10 potions
    READ b$, x ' read potion name, spell number
    IF GetUserRecordPotion(i) THEN ' check potions array element flag
        LOCATE i + 2, 45, 0 ' position cursor
        PRINT "(" + CHR$(i + 80) + ")potion of " + b$; ' append potion name
    END IF
NEXT
COLOR 10 ' change color
LOCATE 14, 5, 0 ' position cursor
PRINT "Key: Control-"; ' display key command for staffs
LOCATE 14, 45, 0 ' position cursor
PRINT "Key: Control-"; ' display key command for rings
COLOR 14 ' change color
RESTORE StaffData ' restore data pointer to staff data
FOR i = 1 TO 10 ' loop through 10 staffs
    READ b$, x ' read staff name, staff spell number
    IF GetUserRecordStaff(i) THEN ' check staff array element flag
        LOCATE i + 14, 5 ' position cursor
        PRINT "(" + CHR$(i + 64) + ")staff of " + b$; ' append staff name
    END IF
NEXT
RESTORE RingData ' restore data pointer to ring data
FOR i = 1 TO 10 ' loop through 10 rings
    READ b$, x ' read ring name, ring spell number
    IF GetUserRecordRing(i) THEN ' check ring array element flag
        LOCATE i + 14, 45, 0 ' position cursor
        PRINT "(" + CHR$(i + 80) + ")ring of " + b$; ' append ring name
    END IF
NEXT
GOSUB Key.Press ' wait
GOSUB Restore.Screen ' redraw dungeon screen
RETURN ' exit routine

' open the user file, length of user record
Open.User.File:
CLOSE 1 ' close file 1
' open user file random
OPEN "players.dat" FOR RANDOM SHARED AS #1 LEN = LEN(UserRecord)
RETURN ' exit routine

' open the data file, length of dungeon record
OpenData.File:
CLOSE 2 ' close file 2
' name the data file to store the dungeon statistics,
' the extension being the users index padded in zeroes
Filename$ = "datafile." + RIGHT$(STR$(User.Index + 1000), 3)
' open the file, randomly, length of the record equal to
' the record structure containing the dungeon level data
OPEN Filename$ FOR RANDOM SHARED AS #2 LEN = LEN(DungeonRecord)
RETURN ' exit routine

' store user record and data to user file, at user record number
Put.User.File:
UserRecord.CodeName = fnencrypt$(CodeName$) ' store encrypted codename
UserRecord.PassWord = fnencrypt$(PassWord$) ' store encrypted password
PUT 1, User.Index, UserRecord ' put user record into user index number
RETURN ' exit routine

' create new dungeon level
Make.Dungeon:
COLOR 14, 1 ' blue background
CLS ' clear screen
COLOR 13 ' change color
LOCATE 12, 27, 0 ' position cursor
PRINT "Creating Dungeon Level"; ' display dungeon level being created
PRINT STR$(dlvl#); ' dungeon level
COLOR 7 ' change color
SLEEP 3 ' wait
ll = 1 ' reset extended cell display amount
llc = 0 ' reset extended cell display counter
FOR i = 0 TO 26 ' clear temporary dungeon arrays
    FOR j = 0 TO 11
        Maze1(i, j) = 0
        Maze2(i, j) = 0
    NEXT
NEXT
' reset temporary dungeon creation counters
q = 0
z = 1
x = INT(RND * 26 + 1)
c = 1
Maze2(x, 1) = c
c = c + 1
r = x
s = 1
' following source code makes a maze in Maze1, and Maze2..
GOTO 2540
2440 IF r <> 26 THEN 2520
IF s <> 11 THEN 2490
r = 1
s = 1
GOTO 2530
2490 r = 1
s = s + 1
GOTO 2530
2520 r = r + 1
2530 IF Maze2(r, s) = 0 THEN 2440
2540 IF r - 1 = 0 THEN 2830
IF Maze2(r - 1, s) THEN 2830
IF s - 1 = 0 THEN 2680
IF Maze2(r, s - 1) THEN 2680
IF r = 26 THEN 2610
IF Maze2(r + 1, s) THEN 2610
x = INT(RND * 3 + 1): ON x GOTO 3140, 3210, 3280
2610 IF s <> 11 THEN 2650
IF z = 1 THEN 2670
q = 1
GOTO 2660
2650 IF Maze2(r, s + 1) THEN 2670
2660 x = INT(RND * 3 + 1): ON x GOTO 3140, 3210, 3370
2670 x = INT(RND * 2 + 1): ON x GOTO 3140, 3210
2680 IF r = 26 THEN 2770
IF Maze2(r + 1, s) THEN 2770
IF s <> 11 THEN 2740
IF z = 1 THEN 2760
q = 1
GOTO 2750
2740 IF Maze2(r, s + 1) THEN 2760
2750 x = INT(RND * 3 + 1): ON x GOTO 3140, 3280, 3370
2760 x = INT(RND * 2 + 1): ON x GOTO 3140, 3280
2770 IF s <> 11 THEN 2810
IF z = 1 THEN 3140
q = 1
GOTO 2820
2810 IF Maze2(r, s + 1) THEN 3140
2820 x = INT(RND * 2 + 1): ON x GOTO 3140, 3370
2830 IF s - 1 = 0 THEN 3000
IF Maze2(r, s - 1) THEN 3000
IF r = 26 THEN 2940
IF Maze2(r + 1, s) THEN 2940
IF s <> 11 THEN 2910
IF z = 1 THEN 2930
q = 1
GOTO 2920
2910 IF Maze2(r, s + 1) THEN 2930
2920 x = INT(RND * 3 + 1): ON x GOTO 3210, 3280, 3370
2930 x = INT(RND * 2 + 1): ON x GOTO 3210, 3280
2940 IF s <> 11 THEN 2980
IF z = 1 THEN 3210
q = 1
GOTO 2990
2980 IF Maze2(r, s + 1) THEN 3210
2990 x = INT(RND * 2 + 1): ON x GOTO 3210, 3370
3000 IF r = 26 THEN 3080
IF Maze2(r + 1, s) THEN 3080
IF s <> 11 THEN 3060
IF z = 1 THEN 3280
q = 1
GOTO 3220
3060 IF Maze2(r, s + 1) THEN 3280
x = INT(RND * 2 + 1): ON x GOTO 3280, 3370
3080 IF s <> 11 THEN 3120
IF z = 1 THEN 2440
q = 1
GOTO 3370
3120 IF Maze2(r, s + 1) THEN 2440
GOTO 3370
3140 Maze2(r - 1, s) = c
c = c + 1
Maze1(r - 1, s) = 2
r = r - 1
IF c > 286 THEN 3570
q = 0
GOTO 2540
3210 Maze2(r, s - 1) = c
3220 c = c + 1
Maze1(r, s - 1) = 1
s = s - 1
IF c > 286 THEN 3570
q = 0
GOTO 2540
3280 Maze2(r + 1, s) = c
c = c + 1
IF Maze1(r, s) = 0 THEN 3330
Maze1(r, s) = 3
GOTO 3340
3330 Maze1(r, s) = 2
3340 r = r + 1
IF c > 286 THEN 3570
GOTO 2830
3370 IF q = 1 THEN 3470
Maze2(r, s + 1) = c
c = c + 1
IF Maze1(r, s) = 0 THEN 3430
Maze1(r, s) = 3
GOTO 3440
3430 Maze1(r, s) = 1
3440 s = s + 1
IF c > 286 THEN 3570
GOTO 2540
3470 z = 1
IF Maze1(r, s) = 0 THEN 3520
Maze1(r, s) = 3
q = 0
GOTO 2440
3520 Maze1(r, s) = 1
q = 0
r = 1
s = 1
GOTO 2530
3570
' maze maker ends here..
FOR i = 1 TO 24 ' clear maze array
    FOR j = 1 TO 80
        CALL PutDungeonRecordLevelData(i, j, 46) ' with dungeon dots
    NEXT
NEXT
FOR i = 1 TO 26 ' convert array to maze
    FOR j = 1 TO 11
        SELECT CASE Maze1(i, j)
            CASE 0
                CALL PutDungeonRecordLevelData(j * 2, i * 3 + 1, 176)
                CALL PutDungeonRecordLevelData(j * 2 + 1, i * 3 - 2, 176)
                CALL PutDungeonRecordLevelData(j * 2 + 1, i * 3 - 1, 176)
                CALL PutDungeonRecordLevelData(j * 2 + 1, i * 3, 176)
            CASE 1
                CALL PutDungeonRecordLevelData(j * 2, i * 3 + 1, 176)
                CALL PutDungeonRecordLevelData(j * 2 + 1, i * 3 - 2, 176)
            CASE 2
                CALL PutDungeonRecordLevelData(j * 2 + 1, i * 3 - 2, 176)
                CALL PutDungeonRecordLevelData(j * 2 + 1, i * 3 - 1, 176)
                CALL PutDungeonRecordLevelData(j * 2 + 1, i * 3, 176)
            CASE 3
                CALL PutDungeonRecordLevelData(j * 2 + 1, i * 3 - 2, 176)
        END SELECT
    NEXT
NEXT
FOR i = 1 TO 80 ' border
    CALL PutDungeonRecordLevelData(1, i, 176)
    CALL PutDungeonRecordLevelData(23, i, 176)
NEXT
FOR i = 1 TO 24 ' border
    CALL PutDungeonRecordLevelData(i, 1, 176)
    CALL PutDungeonRecordLevelData(i, 79, 176)
NEXT
' LevelData now contains borderlike dungeon/maze
u1 = 15 ' set room total gold items counter
u2 = 30 ' set room total coins items counter
u3 = 15 ' set room total gems items counter
FOR p = 1 TO 4 ' make four rooms
    SELECT CASE p
        CASE 1 ' first room
            m = INT(RND * 4 + 2) ' room x-coordinate
            n = INT(RND * 30 + 2) ' room y-coordinate in upper-left half of dungeon
        CASE 2 ' second room
            m = INT(RND * 4 + 2) ' room x-coordinate
            n = INT(RND * 31 + 40) ' room y-coordinate in upper-right half of dungeon
        CASE 3 ' third room
            m = INT(RND * 4 + 12) ' room x-coordinate
            n = INT(RND * 30 + 2) ' room y-coordinate in lower-left half of dungeon
        CASE 4 ' fourth room
            m = INT(RND * 4 + 12) ' room x-coordinate
            n = INT(RND * 31 + 40) ' room y-coordinate in lower-right half of dungeon
    END SELECT
    x1 = 4 ' size of room side
    y1 = 6 ' size of room side
    FOR q1 = 0 TO x1 ' make room border
        CALL PutDungeonRecordLevelData(m + q1, n, 21 + p) ' specific border character
        CALL PutDungeonRecordLevelData(m + q1, n + y1, 21 + p) ' specific border character
    NEXT
    FOR q1 = 0 TO y1 ' make room border
        CALL PutDungeonRecordLevelData(m, n + q1, 21 + p) ' specific border character
        CALL PutDungeonRecordLevelData(m + x1, n + q1, 21 + p) ' specific border character
    NEXT
    SELECT CASE p ' select item to fill room
        CASE 1 ' gold
            t = 36 ' gold symbol
        CASE 2 ' coins
            t = 38 ' coins symbol
        CASE 3 ' gems
            t = 42 ' gems symbol
        CASE 4 ' coins
            t = 38 ' coins symbol
    END SELECT ' end select item
    FOR q1 = m + 1 TO m + x1 - 1 ' loop through inside of room
        FOR q2 = n + 1 TO n + y1 - 1 ' loop through inside of room
            CALL PutDungeonRecordLevelData(q1, q2, t)
        NEXT ' end room contents loop
    NEXT ' end room contents loop
NEXT ' end make rooms
GOSUB Find.Dot ' find m, n empty dungeon dot coordinate
CALL PutDungeonRecordLevelData(m, n, 62) ' staircase up ('>')
GOSUB Find.Dot ' find m, n empty dungeon dot coordinate
CALL PutDungeonRecordLevelData(m, n, 60) ' staircase down ('<')
COLOR 14, 1 ' blue background
CLS ' clear screen
COLOR 13 ' change color
LOCATE 12, 25, 0 ' position cursor
PRINT "Replenishing Dungeon Level"; ' make message
PRINT STR$(dlvl#); ' dungeon level being replenished
COLOR 7 ' change color
SLEEP 3 ' wait
GOSUB Replenish.Dungeon ' add dungeon symbols/characters
COLOR 14, 1 ' blue background
CLS ' clear screen
t = 100 ' reset loop counter
DO ' loop until dungeon dot found
    t = t - 1 ' decrement loop counter
    IF t <= 0 THEN ' check counter value
        EXIT DO ' exit loop
    END IF ' end check loop counter
    xco = INT(RND * 20 + 2) ' select player x-coordinate
    yco = INT(RND * 76 + 2) ' select player y-coordinate
LOOP UNTIL GetDungeonRecordLevelData(xco, yco) = 46 ' loop until dungeon dot found
u1 = 0 ' reset room total gold in rooms
u2 = 0 ' reset room total coins in rooms
u3 = 0 ' reset room total gems in rooms
RETURN ' exit routine

' locate empty dungeon dot, return coordinate in (m,n).
Find.Dot:
t = 100 ' set maximum search counter
DO ' loop until dungeon dot found
    IF t <= 0 THEN ' check counter
        RETURN ' exit routine
    END IF ' end check counter
    t = t - 1 ' decrement counter
    m = INT(RND * 20 + 2) ' random dungeon coordinate
    n = INT(RND * 76 + 2) ' random dungeon coordinate
LOOP UNTIL GetDungeonRecordLevelData(m, n) = 46 ' loop until dot found
RETURN ' exit routine

' reset any dungeon symbols/characters to dungeon dot,
' except for stairs up/down
Zero.Dungeon:
FOR i = 1 TO 24 ' loop through maze
    FOR j = 1 TO 80 ' loop through maze
        SELECT CASE GetDungeonRecordLevelData(i, j) ' find symbol
            CASE 35, 126, 94, 36, 111, 33, 63, 42, 38, 254, 124, 64, 37, 34
                CALL PutDungeonRecordLevelData(i, j, 46) ' reset to dungeon dot
        END SELECT ' end find symbol
    NEXT ' end loop
NEXT ' end loop
u1 = 0 ' reset room total gold in rooms
u2 = 0 ' reset room total coins in rooms
u3 = 0 ' reset room total gems in rooms
RETURN ' exit routine

' calculate price of entire dungeon:
' add number of dungeon symbols/characters,
' except for stairs up/down..
Dungeon.Worth:
k = 0 ' counter
FOR i = 1 TO 24 ' loop through maze
    FOR j = 1 TO 80 ' loop through maze
        SELECT CASE GetDungeonRecordLevelData(i, j) ' select symbol
            CASE 35, 126, 94, 36, 111, 33, 63, 42, 38, 254, 124, 64, 37, 34
                k = k + 1 ' increment counter
        END SELECT
    NEXT
NEXT
' calculate price according to random values and counter
temp# = (RND * dlvl# * 2 + 1) * GetUserRecordStats1(2) ^ 2 / 2
temp# = temp# + dlvl# * 2 + RND * GetUserRecordStats1(2) * 50
temp# = temp# * k
z1# = GetUserRecordStats1(5)
z1# = z1# + INT(temp#)
CALL PutUserRecordStats1(5, z1#)
RETURN ' exit routine

' restore dungeon with new symbols/characters
Replenish.Dungeon:
FOR p = 1 TO 4 ' search for four rooms
    FOR m = 1 TO 24 ' loop through maze
        FOR n = 1 TO 80 ' loop through maze
            IF GetDungeonRecordLevelData(m, n) = 21 + p THEN ' find room corner
                x1 = 4 ' size of room side
                y1 = 6 ' size of room side
                FOR q1 = m + 1 TO m + x1 - 1 ' loop through inside of room
                    FOR q2 = n + 1 TO n + y1 - 1 ' loop through inside of room
                        IF GetDungeonRecordLevelData(q1, q2) = 46 THEN ' find empty dot
                            SELECT CASE p ' check room type
                                CASE 1 ' gold
                                    t = 36 ' gold symbol
                                    u1 = u1 + 1 ' number of gold
                                CASE 2 ' coins
                                    t = 38 ' coins symbol
                                    u2 = u2 + 1 ' number of coins
                                CASE 3 ' gems
                                    t = 42 ' gems symbol
                                    u3 = u3 + 1 ' number gems
                                CASE 4 ' coins
                                    t = 38 ' coins symbol
                                    u2 = u2 + 1 ' number of coins
                            END SELECT
                            CALL PutDungeonRecordLevelData(q1, q2, t)
                        END IF
                    NEXT ' end room contents loop
                NEXT ' end room contents loop
                n = 81 ' increment to end of loop
                m = 25 ' increment to end of loop
            END IF ' end find room corner
        NEXT ' end room contents loop
    NEXT ' end room contents loop
NEXT ' end replenish rooms
CALL PutDungeonRecordStats(11, u1) ' number of gold
CALL PutDungeonRecordStats(21, u2) ' number of coins
CALL PutDungeonRecordStats(19, u3) ' number gems
CALL PutDungeonRecordStats(4, 0) ' all level counters
CALL PutDungeonRecordStats(6, 0) ' number of monster
CALL PutDungeonRecordStats(5, 0) ' all monsters dead flag
FOR i = 1 TO INT(RND * 10 + 10) ' random loop number
    GOSUB Find.Dot ' find m, n empty dungeon dot coordinate
    CALL PutDungeonRecordLevelData(m, n, 35) ' make cell monster symbol
    z = GetDungeonRecordStats(6)
    z = z + 1
    CALL PutDungeonRecordStats(6, z) ' increment monsters
NEXT ' loop through
CALL PutDungeonRecordStats(7, 0) ' number of scrolls
CALL PutDungeonRecordStats(8, 0) ' all scrolls found flag
FOR i = 1 TO INT(RND * 10 + 5) ' random loop number
    GOSUB Find.Dot ' find m, n empty dungeon dot coordinate
    CALL PutDungeonRecordLevelData(m, n, 126) ' make cell scroll symbol
    z = GetDungeonRecordStats(7)
    z = z + 1
    CALL PutDungeonRecordStats(7, z) ' increment scrolls
NEXT ' loop through
CALL PutDungeonRecordStats(9, 0) ' number of traps
CALL PutDungeonRecordStats(10, 0) ' all traps found flag
FOR i = 1 TO INT(RND * 10 + 5) ' random loop number
    GOSUB Find.Dot ' find m, n empty dungeon dot coordinate
    CALL PutDungeonRecordLevelData(m, n, 94) ' make cell trap symbol
    z = GetDungeonRecordStats(9)
    z = z + 1
    CALL PutDungeonRecordStats(9, z) ' increment traps
NEXT ' loop through
CALL PutDungeonRecordStats(12, 0) ' all gold found flag
FOR i = 1 TO INT(RND * 10 + 10) ' random loop number
    GOSUB Find.Dot ' find m, n empty dungeon dot coordinate
    CALL PutDungeonRecordLevelData(m, n, 36) ' make cell gold symbol
    z = GetDungeonRecordStats(11)
    z = z + 1
    CALL PutDungeonRecordStats(11, z) ' increment gold
NEXT ' loop through
CALL PutDungeonRecordStats(13, 0) ' number of pits
CALL PutDungeonRecordStats(14, 0) ' all pits found flag
FOR i = 1 TO INT(RND * 10 + 5) ' random loop number
    GOSUB Find.Dot ' find m, n empty dungeon dot coordinate
    CALL PutDungeonRecordLevelData(m, n, 111) ' make cell pit symbol
    z = GetDungeonRecordStats(13)
    z = z + 1
    CALL PutDungeonRecordStats(13, z) ' increment pits
NEXT ' loop through
CALL PutDungeonRecordStats(15, 0) ' number of teleporters
CALL PutDungeonRecordStats(16, 0) ' all teleporters found flag
FOR i = 1 TO INT(RND * 5 + 5) ' random loop number
    GOSUB Find.Dot ' find m, n empty dungeon dot coordinate
    CALL PutDungeonRecordLevelData(m, n, 33) ' make cell teleporter symbol
    z = GetDungeonRecordStats(15)
    z = z + 1
    CALL PutDungeonRecordStats(15, z) ' increment teleporters
NEXT ' loop through
CALL PutDungeonRecordStats(17, 0) ' number of temples
CALL PutDungeonRecordStats(18, 0) ' all temples found flag
FOR i = 1 TO INT(RND * 5 + 5) ' random loop number
    GOSUB Find.Dot ' find m, n empty dungeon dot coordinate
    CALL PutDungeonRecordLevelData(m, n, 63) ' make cell temple symbol
    z = GetDungeonRecordStats(17)
    z = z + 1
    CALL PutDungeonRecordStats(17, z) ' increment temples
NEXT ' loop through
CALL PutDungeonRecordStats(20, 0) ' all gems found flag
FOR i = 1 TO INT(RND * 10 + 5) ' random loop number
    GOSUB Find.Dot ' find m, n empty dungeon dot coordinate
    CALL PutDungeonRecordLevelData(m, n, 42) ' make cell gem symbol
    z = GetDungeonRecordStats(19)
    z = z + 1
    CALL PutDungeonRecordStats(19, z) ' increment gems
NEXT ' loop through
CALL PutDungeonRecordStats(22, 0) ' all coins found flag
FOR i = 1 TO INT(RND * 10 + 5) ' random loop number
    GOSUB Find.Dot ' find m, n empty dungeon dot coordinate
    CALL PutDungeonRecordLevelData(m, n, 38) ' make cell coins symbol
    z = GetDungeonRecordStats(21)
    z = z + 1
    CALL PutDungeonRecordStats(21, z) ' increment coins
NEXT ' loop through
CALL PutDungeonRecordStats(23, 0) ' number of chests
CALL PutDungeonRecordStats(24, 0) ' all chests found flag
FOR i = 1 TO INT(RND * 10 + 5) ' random loop number
    GOSUB Find.Dot ' find m, n empty dungeon dot coordinate
    CALL PutDungeonRecordLevelData(m, n, 254) ' make cell chest symbol
    z = GetDungeonRecordStats(23)
    z = z + 1
    CALL PutDungeonRecordStats(23, z) ' increment chests
NEXT ' loop through
CALL PutDungeonRecordStats(25, 0) ' number of wands
CALL PutDungeonRecordStats(26, 0) ' all wands found flag
FOR i = 1 TO INT(RND * 5 + 2) ' random loop number
    GOSUB Find.Dot ' find m, n empty dungeon dot coordinate
    CALL PutDungeonRecordLevelData(m, n, 124) ' make cell wand symbol
    z = GetDungeonRecordStats(25)
    z = z + 1
    CALL PutDungeonRecordStats(25, z) ' increment wands
NEXT ' loop through
CALL PutDungeonRecordStats(27, 0) ' number of potions
CALL PutDungeonRecordStats(28, 0) ' all potions found flag
FOR i = 1 TO INT(RND * 5 + 2) ' random loop number
    GOSUB Find.Dot ' find m, n empty dungeon dot coordinate
    CALL PutDungeonRecordLevelData(m, n, 64) ' make cell potion symbol
    z = GetDungeonRecordStats(27)
    z = z + 1
    CALL PutDungeonRecordStats(27, z) ' increment potions
NEXT ' loop through
CALL PutDungeonRecordStats(29, 0) ' number of staffs
CALL PutDungeonRecordStats(30, 0) ' all staffs found flag
FOR i = 1 TO INT(RND * 5 + 2) ' random loop number
    GOSUB Find.Dot ' find m, n empty dungeon dot coordinate
    CALL PutDungeonRecordLevelData(m, n, 37) ' make cell staff symbol
    z = GetDungeonRecordStats(29)
    z = z + 1
    CALL PutDungeonRecordStats(29, z) ' increment staffs
NEXT ' loop through
CALL PutDungeonRecordStats(31, 0) ' number of rings
CALL PutDungeonRecordStats(32, 0) ' all rings found flag
FOR i = 1 TO INT(RND * 5 + 2) ' random loop number
    GOSUB Find.Dot ' find m, n empty dungeon dot coordinate
    CALL PutDungeonRecordLevelData(m, n, 34) ' make cell ring symbol
    z = GetDungeonRecordStats(31)
    z = z + 1
    CALL PutDungeonRecordStats(31, z) ' increment rings
NEXT ' loop through
RETURN ' exit routine

' compute nearby cell activity
Cell.Action:
GOSUB Display.Cells ' display cells around player
GOSUB Monster.Attack ' monsters attack player
GOSUB Remove.Monsters ' remove monsters not nearby player
GOSUB Move.Monsters ' monsters move nearer to player
GOSUB Encounter.Cells ' nearby cells repopulated with monster symbols
RETURN ' exit routine

' display dungeon cells surrounding the players X characters,
' ll being the length of area displayed around player
Display.Cells:
COLOR 7 ' calculate boundaries
x1 = xco - ll ' calculate area of surrounding cells to display
WHILE x1 < 1 ' keep loop counter in bounds
    x1 = x1 + 1 ' increment x1 loop counter
WEND ' end loop bounds counter
y1 = yco - ll ' calculate area of surrounding cells to display
WHILE y1 < 1 ' keep loop counter in bounds
    y1 = y1 + 1 ' increment y1 loop counter
WEND ' end loop bounds counter
x2 = xco + ll ' calculate area of syrrounding cells to display
WHILE x2 > 23 ' keep loop counter in bounds
    x2 = x2 - 1 ' decrement x2 loop counter
WEND ' end loop bounds counter
y2 = yco + ll ' calculate area of surrounding cells to display
WHILE y2 > 79 ' keep loop counter in bounds
    y2 = y2 - 1 ' decrment y2 loop counter
WEND ' end loop bounds counter
uf = 0 ' xy coordinate room display flag
FOR i = x1 TO x2 ' loop surrounding players xy-coordinate
    FOR j = y1 TO y2 ' surrounding y-coordinate
        LOCATE i, j, 0 ' locate at xy cell display
        t = GetDungeonRecordLevelData(i, j) ' get symbol at xy in maze
        IF t = 94 OR t = 111 THEN ' traps/pits
            t = 46 ' display dungeon dot
        END IF
        IF i = xco AND j = yco THEN ' xy is the players X character
            IF t <> 176 THEN ' and is not a dungeon wall
                t = 88 ' display an X
            END IF
        END IF
        IF t = 46 OR t = 176 THEN ' xy is dungeon dot/wall
            COLOR 7 ' color white
        ELSE ' otherwise
            COLOR 15 ' color highlight white
        END IF
        IF t > 21 AND t < 26 THEN ' xy is room border
            IF uf = 0 THEN ' room has been displayed
                uf = -1 ' set room display flag
                GOSUB Display.Dungeon.Room ' display entire room
            END IF
        ELSE
            PRINT CHR$(t); ' display dungeon square symbol
        END IF
    NEXT ' loop all nearby cells
NEXT
COLOR 7 ' change color
RETURN ' exit routine

' display entire room next to room border character,
' room border is 22 to 25.
Display.Dungeon.Room:
rx = i ' room border counter
ry = j ' room border counter
FOR q1 = rx - 4 TO rx + 4 ' loop any xy-coordinates around room
    FOR q2 = ry - 6 TO ry + 6 ' loop any xy-coordinates around room
        IF q1 > 1 AND q1 < 23 THEN ' check array bounds
            IF q2 > 1 AND q2 < 79 THEN ' check array bounds
                IF GetDungeonRecordLevelData(q1, q2) = t THEN ' xy-coor. is room border
                    LOCATE q1, q2, 0 ' locate at room border
                    PRINT CHR$(176); ' display room border
                END IF
            END IF
        END IF
    NEXT
NEXT
RETURN ' exit routine

' monsters attack the player
Monster.Attack:
' reset any monster increment counters
GOSUB Update.Monster.Counters
IF slp = 1 THEN ' sleep flag
    RETURN ' exit routine
END IF
FOR mn = 1 TO 24 ' loop through monster array
    mxco = Monsters(mn, 1) ' get monster x-coordinate
    myco = Monsters(mn, 2) ' get monster y-coordinate
    IF mxco OR myco THEN
        x = ABS(xco - mxco) ' calculate distance from players xy
        y = ABS(yco - myco)
        IF (x + y) = 1 OR (x = 1 AND y = 1) THEN ' distance one square from player
            ' calculate monsters hits on player
            x = 0 ' calculate shield
            FOR i = 8 TO 11
                x = x + GetUserRecordStats2(i)
            NEXT
            x = x + shp + 1
            mhp# = INT(RND * (Monsters(mn, 4) / x) + Monsters(mn, 3) - mhpm)
            ' display hits on player
            COLOR 14 ' change color
            a$ = "The " + MonsterNames(Monsters(mn, 6)) + " attacks you! "
            IF mhp# <= 0 THEN ' monster missed
                a$ = a$ + "It missed!" ' make display message
                GOSUB Display.Message ' output message
            ELSE
                GOSUB Hit.Player ' hit player for monsters hit attack points
            END IF
        END IF
    END IF
NEXT
RETURN ' exit routine

' remove monsters not nearby player cell
Remove.Monsters:
FOR mn = 1 TO 24 ' loop through monsters
    mxco = Monsters(mn, 1) ' get monster x-coordinate
    myco = Monsters(mn, 2) ' get monster y-coordinate
    IF mxco OR myco THEN ' active monster xy-coordinates
        ' get distance to monster from players xy,
        ' standard hypotenuse formula for right angle.
        di = INT(SQR((mxco - xco) ^ 2 + (myco - yco) ^ 2))
        ' determine distance from player
        IF di > 2 THEN
            ' remove monster
            FOR j = 1 TO 6 ' clear statistics
                Monsters(mn, j) = 0 ' set to zero
            NEXT ' end clear loop
        END IF ' end determine distance
    END IF ' end check active monster
NEXT ' end loop through monsters
RETURN ' exit routine

' calculate monsters actions near player cell
Move.Monsters:
IF pw = 0 THEN ' pass monsters flag
    FOR mn = 1 TO 24 ' loop through monsters
        mxco = Monsters(mn, 1) ' get monster x-coordinate
        myco = Monsters(mn, 2) ' get monster y-coordinate
        IF mxco OR myco THEN ' active monster xy-coordinates
            ' get distance to monster from players xy,
            ' standard hypotenuse formula for right angle.
            di = INT(SQR((mxco - xco) ^ 2 + (myco - yco) ^ 2))
            ' determine distance from monster
            SELECT CASE di
                CASE 1 ' monster next to player
                    IF Monsters(mn, 4) <= 10 THEN ' monster is weak
                        GOSUB Monster.Evade ' move monster farther
                    END IF
                CASE 2 ' monster is near to player
                    IF Monsters(mn, 4) > 10 THEN ' monster is strong
                        GOSUB Monster.Closer ' move monster closer
                    END IF
            END SELECT ' end check distance
        END IF ' end check active monster
    NEXT ' end loop through monsters
END IF ' end check pass monsters
RETURN ' exit routine

' monster evades player
Monster.Evade:
c = 0 ' loop counter
DO ' loop until empty cell found
    c = c + 1 ' increment counter
    IF c = 10 THEN ' avoid infinite loop
        EXIT DO ' exit empty cell search loop
    END IF
    cxco = mxco ' store x-coordinate
    cyco = myco ' store y-coordinate
    GOSUB Get.Random.Direction ' get valid random direction
    IF Coor.Direction THEN ' returned valid random direction
        IF GetDungeonRecordLevelData(mxco + xo, myco + yo) = 46 THEN ' empty cell
            IF SCREEN(mxco + xo, myco + yo) <> 32 THEN ' active cell
                dx = INT(SQR((mxco - xco + xo) ^ 2 + (myco - yco + yo) ^ 2)) ' compute distance
                IF dx = 2 THEN ' monster moves away from player
                    COLOR 7 ' change color
                    LOCATE mxco, myco, 0 ' position cursor
                    PRINT "."; ' clear monster with dungeon dot
                    ' reset old dungeon array cell to dungeon dot
                    CALL PutDungeonRecordLevelData(mxco, myco, 46)
                    mxco = mxco + xo ' new monster xy-coordinate
                    myco = myco + yo ' new monster xy-coordinate
                    ' available empty cell evade,
                    COLOR 15 ' change color
                    LOCATE mxco, myco, 0 ' position cursor
                    PRINT "#"; ' and display monster symbol farther
                    ' set dungeon array monster xy to monster symbol
                    Monsters(mn, 1) = mxco ' reset monster array x-coordinate
                    Monsters(mn, 2) = myco ' reset monster array y-coordinate
                    ' reset new dungeon array cell to monster symbol
                    CALL PutDungeonRecordLevelData(mxco, myco, 35)
                    EXIT DO
                END IF
            END IF
        END IF
    END IF
LOOP
RETURN ' exit routine

' move monster closer
Monster.Closer:
u = SGN(xco - mxco) ' store monsters xy-coordinates
v = SGN(yco - myco) ' distance one square from player
' recalculated monsters xy-coordinates can move to empty cell
IF GetDungeonRecordLevelData(mxco + u, myco + v) = 46 THEN
    COLOR 7 ' change color
    LOCATE mxco, myco, 0 ' position cursor
    PRINT "."; ' clear monster with dungeon dot
    ' reset old dungeon array cell to dungeon dot
    CALL PutDungeonRecordLevelData(mxco, myco, 46)
    COLOR 15 ' change color
    mxco = mxco + u ' increment monster array x-coordinate
    myco = myco + v ' increment monster array y-coordinate
    LOCATE mxco, myco, 0 ' position cursor
    PRINT "#"; ' and display monster symbol closer
    Monsters(mn, 1) = mxco ' reset monster array x-coordinate
    Monsters(mn, 2) = myco ' reset monster array y-coordinate
    ' reset new dungeon array cell to monster symbol
    CALL PutDungeonRecordLevelData(mxco, myco, 35)
END IF
RETURN ' exit from subroutine

' get any monsters nearby players dungeon cell
Encounter.Cells:
x1 = xco - 2 ' calculate area of surrounding cells
WHILE x1 < 1 ' compare bounds
    x1 = x1 + 1 ' increment coordinate
WEND
y1 = yco - 2 ' calculate area of surrounding cells
WHILE y1 < 1 ' compare bounds
    y1 = y1 + 1 ' increment coordinate
WEND
x2 = xco + 2 ' calculate area of surrounding cells
WHILE x2 > 23 ' compare bounds
    x2 = x2 - 1 ' decrement coordinate
WEND
y2 = yco + 2 ' calculate area of surrounding cells
WHILE y2 > 79 ' compare bounds
    y2 = y2 - 1 ' decrement coordinate
WEND
FOR mxco = x1 TO x2 ' distance of cell display amount
    FOR myco = y1 TO y2 ' surrounding player
        ' dungeon has monster at xy
        IF GetDungeonRecordLevelData(mxco, myco) = 35 THEN
            mn = INT(RND * 100 + 1) ' select random monster name
            GOSUB Encounter.Monster ' get monster statistics
        END IF
    NEXT
NEXT ' loop through all cells
RETURN ' exit routine

' puts monsters statistics in monster array,
' dependent on monster at xy-coordinate in mxco,myco
Encounter.Monster:
' verify monster does not already exist in monster array
FOR w = 1 TO 24 ' loop through monster xy array
    IF Monsters(w, 1) = mxco AND Monsters(w, 2) = myco THEN ' monster at xy already
        RETURN ' exit routine
    END IF
NEXT
' find empty monster array xy-coordinates
FOR w = 1 TO 24 ' loop through monster xy array
    IF Monsters(w, 1) = 0 AND Monsters(w, 2) = 0 THEN ' empty monster xy array found
        bonus = 0 ' reset bonus for monster leve;
        IF mn = 101 THEN ' monster selection is demon
            bonus = 5 ' store monster level
        END IF ' end calculate monster level
        ' calculate monster level
        mlvl# = INT(RND * (dlvl# + bonus) + (dlvl# + bonus) / 2 + 1)
        ' calculate monsters hit points by random values
        hp# = INT((RND * dlvl# * 2 + 1) * mlvl# ^ 2 / 2 + dlvl# * 2 + RND * mlvl# * 50)
        ' calculate monsters experience by random values
        ex# = INT(hp# / mlvl#) * INT(dlvl# / 2 + 1) + INT(RND * hp#)
        Monsters(w, 1) = mxco ' store monster x-coordinate
        Monsters(w, 2) = myco ' store monster y-coordinate
        Monsters(w, 3) = mlvl# ' store monster level
        Monsters(w, 4) = hp# ' store monster hit points
        Monsters(w, 5) = ex# ' store monster experience
        Monsters(w, 6) = mn ' store monster number
        EXIT FOR ' exit loop
    END IF
NEXT
RETURN ' exit routine

' hits player for points in mhp#
Hit.Player:
COLOR 14 ' change color
IF mhp# = 0 THEN ' calculate hits on player if mhp# is zero
    mhp# = INT(RND * dlvl# * 10 + 1) - GetUserRecordStats2(2) ' calculate hits
    IF mhp# <= 0 THEN ' zero or less hits
        GOSUB Display.Message ' output prepared message
        RETURN ' exit routine
    END IF
END IF
' precalculated hits on player, mhp#
a$ = "It hit you for" + STR$(mhp#) + " hits!" ' make display message
GOSUB Display.Message ' output message
' decrease players current hits remaining by mhp# points
z1# = GetUserRecordStats1(4)
z1# = z1# - mhp#
CALL PutUserRecordStats1(4, z1#)
IF GetUserRecordStats1(4) > 0 THEN ' players hits less than zero
    RETURN ' exit routine
END IF
' player loses a level
' decrement players maximum hits to lower level
z1# = GetUserRecordStats1(6)
z1# = z1# - GetUserRecordStats2(4)
CALL PutUserRecordStats1(6, z1#)
IF GetUserRecordStats1(6) < 15 THEN ' compute minimum players max hits variable
    CALL PutUserRecordStats1(6, 15) ' reset to 15, the minimum
END IF

z1# = GetUserRecordStats1(6) ' set hits to lower max hits
CALL PutUserRecordStats1(4, z1#) ' set hits to lower max hits

z1# = GetUserRecordStats1(2)
z1# = z1# - 1 ' decrement players level by one
CALL PutUserRecordStats1(2, z1#) ' decrement players level by one
IF GetUserRecordStats1(2) > 0 THEN ' players level is zero
    RETURN ' exit routine
END IF
' player died
CALL PutUserRecordStats1(2, 1) ' reset player to level one
CALL PutUserRecordStats1(4, 15) ' reset players hits
CALL PutUserRecordStats1(6, 15) ' reset players maximum hits
' make player died message prompt
COLOR 31 ' change color
a$ = "You died!" ' make death message
GOSUB Display.Message ' send message
Max.Points = GetUserRecordStats1(2) * 10 ' reset players spell points
FOR z2 = 13 TO 16
    CALL PutUserRecordStats2(z2, Max.Points) ' reset spell points
NEXT
RETURN ' exit routine

' global error routine
Error.Routine:
COLOR 7, 0 ' reset color
CLS ' clear screen
LOCATE 10, 10 ' locate message
PRINT "The Dungeon Crashed! (Error:" + STR$(ERR) + ")" ' print message
LOCATE 11, 10 ' locate message
PRINT "IDE Line: "; _ERRORLINE
END ' end program

' four levels of spell chants and names
SpellData:
DATA "i cleanse thy body","heal"
DATA "i protect thee","protection"
DATA "darts fly from my hands","magic missile"
DATA "i shield thee","shield"
DATA "i strengthen thee","strength"
DATA "your eyes shut tight","sleep"
DATA "objects become visible","locate"
DATA "flames fly from my fingertips","pyrotechnics"
DATA "i call upon winds of fire","firestorm"
DATA "i create gold from air","transmute"

DATA "i restore thee","restoration"
DATA "fright causes you to falter","scare"
DATA "balls of fire engulf you","fireball"
DATA "it becomes hard to see","obscurement"
DATA "a shield for my body","power shield"
DATA "i will a wall","wall of will"
DATA "i must see","light"
DATA "gravity leaves","levitate"
DATA "rise from the dead","monster summon"
DATA "i point and you die","finger of death"

DATA "i regenerate thee","regeneration"
DATA "you must not remember","forget"
DATA "a bolt from hell carries thee away","lightning bolt"
DATA "multicolored shield protect me","phantasmal force"
DATA "i create a wall","wall of force"
DATA "i cast sleep unto your eyes","charm monster"
DATA "i will see more","continual light"
DATA "i will travel through thee","passwall"
DATA "monsters of hell hear me","demon summon"
DATA "you will all die","doomkill"

DATA "i resurrect thee","resurrection"
DATA "i frighten thee","fear"
DATA "you turn to dust","disintegrate"
DATA "swirling swords slice you","blade barrier"
DATA "may my weapon become powerful","enchanted weapon"
DATA "you are binded tightly","hold monster"
DATA "i travel to parts unknown","teleport"
DATA "you become another","polymorph"
DATA "let me pass","time stop"
DATA "everything dies continuously","genocide"

' ten wand spells, and spell number
WandData:
DATA "magic missile",3,"pyrotechnics",8,"firestorm",9,"fireball",13
DATA "finger of death",20,"lightning bolt",23,"doomkill",30
DATA "disintegrate",33,"blade barrier",34,"genocide",40

' ten potion spells, and spell number
PotionData:
DATA "heal",1,"protection",2,"shield",4,"strength",5
DATA "restoration",11,"power shield",15,"wall of will",16
DATA "regeneration",21,"phantasmal force",24,"resurrection",31

' ten staff spells, and spell number
StaffData:
DATA "locate",7,"transmute",10,"light",17,"levitate",18
DATA "continual light",27,"passwall",28,"fear",32
DATA "enchanted weapon",35,"teleport",37,"time stop",39

' ten ring spells, and spell number
RingData:
DATA "sleep",6,"scare",12,"obscurement",14,"monster summon",19
DATA "forget",22,"wall of force",25,"charm monster",26
DATA "demon summon",29,"hold monster",36,"polymorph",38

' 100 monster names
MonsterData:
DATA algoid,ankheg,azer,baku,bandit,basilisk,bat,bear,beetle,beholder,bowler
DATA bugbear,bullywug,caterwaul,centipede,chimera,cloaker,cockatrice,corpse
DATA crawler,dao,demilich,diakk,drider,duergar,elf,firebat,frog,gargoyle
DATA ghast,ghost,giant,githyanki,gnoll,gnome,goblin,golem,grell,grimlock
DATA grue,hornet,horror,hulk,hydra,kenku,killer,kobold,lamprey,lich,lion
DATA lizard,mage,margoyle,meenlock,mephit,mimic,minotaur,mummy,norker,ogre
DATA orc,otyugh,owlbear,pech,pedipalp,phantom,phycomid,piercer,planetar
DATA quaggoth,rakshasa,rat,reptile,scorpion,shocker,skeleton,slaad,slime
DATA slug,snake,solifugid,spectator,spectre,spider,stalker,stirge,tarrasque
DATA thoquaa,toad,troglodyte,troll,ustilagor,vampire,verbeeg,vulching,weasel
DATA webbird,wight,wraith,wyvern,demon

FUNCTION GetDungeonRecordLevelData (i, j)
GetDungeonRecordLevelData = CVI(MID$(DungeonRecord.LevelData, (i - 1) * 160 + (j - 1) * 2 + 1, 2))
END FUNCTION

SUB PutDungeonRecordLevelData (i, j, k)
MID$(DungeonRecord.LevelData, (i - 1) * 160 + (j - 1) * 2 + 1, 2) = MKI$(k)
END SUB

FUNCTION GetDungeonRecordScreenData (i, j)
GetDungeonRecordScreenData = CVI(MID$(DungeonRecord.ScreenData, (i - 1) * 160 + (j - 1) * 2 + 1, 2))
END FUNCTION

SUB PutDungeonRecordScreenData (i, j, k)
MID$(DungeonRecord.ScreenData, (i - 1) * 160 + (j - 1) * 2 + 1, 2) = MKI$(k)
END SUB

FUNCTION GetDungeonRecordStats (i)
GetDungeonRecordStats = CVI(MID$(DungeonRecord.Stats, (i - 1) * 2 + 1, 2))
END FUNCTION

SUB PutDungeonRecordStats (i, j)
MID$(DungeonRecord.Stats, (i - 1) * 2 + 1, 2) = MKI$(j)
END SUB

SUB SetDungeonRecordStats (i)
k = GetDungeonRecordStats(i)
k = k - 1 ' decrement items
CALL PutDungeonRecordStats(i, k)
IF k = 0 THEN ' player has entered/used last item
    IF GetDungeonRecordStats(i + 1) = 0 THEN
        CALL PutDungeonRecordStats(i + 1, 1) ' set all dungeon level item flag
    END IF
END IF
END SUB

FUNCTION GetUserRecordSpell (i)
GetUserRecordSpell = CVI(MID$(UserRecord.Spell, (i - 1) * 2 + 1, 2))
END FUNCTION

SUB PutUserRecordSpell (i, j)
MID$(UserRecord.Spell, (i - 1) * 2 + 1, 2) = MKI$(j)
END SUB

FUNCTION GetUserRecordWand (i)
GetUserRecordWand = CVI(MID$(UserRecord.Wand, (i - 1) * 2 + 1, 2))
END FUNCTION

SUB PutUserRecordWand (i, j)
MID$(UserRecord.Wand, (i - 1) * 2 + 1, 2) = MKI$(j)
END SUB

FUNCTION GetUserRecordPotion (i)
GetUserRecordPotion = CVI(MID$(UserRecord.Potion, (i - 1) * 2 + 1, 2))
END FUNCTION

SUB PutUserRecordPotion (i, j)
MID$(UserRecord.Potion, (i - 1) * 2 + 1, 2) = MKI$(j)
END SUB

FUNCTION GetUserRecordStaff (i)
GetUserRecordStaff = CVI(MID$(UserRecord.Staff, (i - 1) * 2 + 1, 2))
END FUNCTION

SUB PutUserRecordStaff (i, j)
MID$(UserRecord.Staff, (i - 1) * 2 + 1, 2) = MKI$(j)
END SUB

FUNCTION GetUserRecordRing (i)
GetUserRecordRing = CVI(MID$(UserRecord.Ring, (i - 1) * 2 + 1, 2))
END FUNCTION

SUB PutUserRecordRing (i, j)
MID$(UserRecord.Ring, (i - 1) * 2 + 1, 2) = MKI$(j)
END SUB

FUNCTION GetUserRecordGlobe (i)
GetUserRecordGlobe = CVI(MID$(UserRecord.Globe, (i - 1) * 2 + 1, 2))
END FUNCTION

SUB PutUserRecordGlobe (i, j)
MID$(UserRecord.Globe, (i - 1) * 2 + 1, 2) = MKI$(j)
END SUB

FUNCTION GetUserRecordStats1# (i)
GetUserRecordStats1# = CVD(MID$(UserRecord.Stats1, (i - 1) * 8 + 1, 8))
END FUNCTION

SUB PutUserRecordStats1 (i, j#)
MID$(UserRecord.Stats1, (i - 1) * 8 + 1, 8) = MKD$(j#)
END SUB

FUNCTION GetUserRecordStats2 (i)
GetUserRecordStats2 = CVI(MID$(UserRecord.Stats2, (i - 1) * 2 + 1, 2))
END FUNCTION

SUB PutUserRecordStats2 (i, j)
MID$(UserRecord.Stats2, (i - 1) * 2 + 1, 2) = MKI$(j)
END SUB

' define function to convert 20 character string to encrypted string
FUNCTION fnencrypt$ (z$)
i$ = "" ' reset encrypted return string
FOR i = 1 TO 4 ' loop in 4 blocks of 5 bytes
    j# = 0 ' reset value of block
    j$ = MID$(z$, i * 5 - 4, 5) ' store next block of 5 bytes
    FOR q = 1 TO 5 ' loop in each block byte
        j# = j# * 1000 + ASC(MID$(j$, q, 1)) ' multiply block value, add block byte value
    NEXT ' next block byte
    i$ = i$ + MKD$(j#) ' add value to return string
NEXT ' next block
fnencrypt$ = i$ ' store function return string
END FUNCTION ' end encryption function

' define function to convert encrypted string to 20 character string
FUNCTION fndecrypt$ (z$)
i$ = "" ' reset decrypted return string
FOR j = 1 TO 4 ' loop in 4 blocks of 5 bytes
    j$ = "" ' reset block string
    z# = CVD(MID$(z$, j * 8 - 7, 8)) ' store block value
    FOR q = 1 TO 5 ' loop in block bytes
        dvid# = INT(z# / 1000) ' store block byte value
        num# = z# - dvid# * 1000 ' compute block byte value
        z# = dvid# ' decrement block value
        j$ = j$ + CHR$(num#) ' add byte to string
    NEXT ' next block byte
    y$ = MID$(j$, 1, 1) ' reverse stored bytes
    MID$(j$, 1, 1) = MID$(j$, 5, 1) ' reverse
    MID$(j$, 5, 1) = y$ ' reverse
    y$ = MID$(j$, 2, 1) ' reverse stored bytes
    MID$(j$, 2, 1) = MID$(j$, 4, 1) ' reverse
    MID$(j$, 4, 1) = y$ ' reverse
    i$ = i$ + j$ ' add string to return string
NEXT ' next block
fndecrypt$ = i$ ' store return function string
END FUNCTION ' end decryption function

REM Ansi.bas v1.0 screen for Dungeon v12.0 r3.0
SUB ANSI
' define color types
CONST Black = 0
CONST Blue = 1
CONST Green = 2
CONST Cyan = 3
CONST Red = 4
CONST Magenta = 5
CONST Brown = 6
CONST White = 7
CONST Gray = 8
CONST HighBlue = 9
CONST HighGreen = 10
CONST HighCyan = 11
CONST HighRed = 12
CONST HighMagenta = 13
CONST HighYellow = 14
CONST HighWhite = 15
' define block characters
DarkBlock$ = CHR$(176)
MediumBlock$ = CHR$(177)
LightBlock$ = CHR$(178)
WhiteBlock$ = CHR$(219)
Dot$ = CHR$(250)
Dot2$ = CHR$(249)
Dot3$ = "."
Square$ = CHR$(254)
UpDot$ = CHR$(220)
DownDot$ = CHR$(223)
Bar$ = CHR$(221)
DoubleBar$ = CHR$(186)
' define block ascii values
DarkBlock% = 176
MediumBlock% = 177
LightBlock% = 178
WhiteBlock% = 219
Dot% = 250
Dot2% = 249
Square% = 254
UpDot% = 220
DownDot% = 223
DoubleBar% = 186
' define author messages
Message1$ = "Welcome To"
Message2$ = "The Dungeon v12.0"
Message3$ = "r3.0"
Message4$ = "An Adventure Game"
Message5$ = "By The Author"
Message6$ = "Of Dnd Door"
' define ascii characters
UnderScore$ = "_"
Slash$ = "/"
BackSlash$ = "\"
CLS ' clear screen
COLOR HighRed
PRINT SPACE$(4);
COLOR White
PRINT Dot$;
COLOR Gray
PRINT DarkBlock$;
PRINT SPACE$(15);
COLOR White
PRINT Dot$;
PRINT SPACE$(25);
COLOR Gray
PRINT DarkBlock$;
PRINT SPACE$(2);
PRINT DarkBlock$;
PRINT SPACE$(5);
COLOR White
PRINT Dot2$;
PRINT SPACE$(6);
PRINT Dot$;
PRINT SPACE$(3);
PRINT Dot2$;
PRINT SPACE$(5);
COLOR Gray
PRINT DarkBlock$;
COLOR White
PRINT Dot2$
PRINT SPACE$(6);
COLOR Gray
PRINT DarkBlock$;
COLOR White
PRINT Dot2$;
PRINT SPACE$(10);
COLOR Gray
PRINT DarkBlock$;
PRINT SPACE$(1);
PRINT DarkBlock$;
PRINT SPACE$(8);
COLOR White
PRINT Dot2$;
PRINT SPACE$(2);
PRINT Dot$;
PRINT SPACE$(4);
COLOR HighWhite
PRINT UpDot$;
PRINT LightBlock$;
PRINT LightBlock$;
PRINT WhiteBlock$;
PRINT WhiteBlock$;
PRINT UpDot$;
PRINT SPACE$(3);
COLOR White
PRINT Dot2$;
PRINT SPACE$(1);
COLOR Gray
PRINT DarkBlock$;
PRINT DarkBlock$;
PRINT SPACE$(15);
COLOR HighWhite
PRINT Message1$;
COLOR Gray
PRINT DarkBlock$;
PRINT SPACE$(1);
COLOR White
PRINT Dot2$
PRINT SPACE$(4);
COLOR Gray
PRINT DarkBlock$;
PRINT Dot2$;
PRINT DarkBlock$;
PRINT MediumBlock$;
PRINT DarkBlock$;
PRINT SPACE$(1);
COLOR White
PRINT Dot2$;
PRINT SPACE$(9);
COLOR Gray
PRINT MediumBlock$;
PRINT Dot$;
PRINT SPACE$(1);
PRINT DarkBlock$;
PRINT SPACE$(12);
COLOR HighWhite
PRINT DarkBlock$;
PRINT WhiteBlock$;
PRINT WhiteBlock$;
PRINT WhiteBlock$;
PRINT LightBlock$;
PRINT WhiteBlock$;
PRINT LightBlock$;
PRINT DarkBlock$;
PRINT SPACE$(3);
COLOR Gray
PRINT DarkBlock$;
PRINT SPACE$(1);
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT DarkBlock$;
PRINT SPACE$(7);
COLOR HighWhite
PRINT Dot$;
PRINT SPACE$(2);
PRINT Message2$
PRINT SPACE$(5);
COLOR Gray
PRINT DarkBlock$;
PRINT DarkBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT DarkBlock$;
PRINT SPACE$(9);
COLOR White
PRINT Dot$;
PRINT SPACE$(2);
COLOR Gray
PRINT MediumBlock$;
PRINT SPACE$(2);
PRINT DarkBlock$;
COLOR White
PRINT Dot3$;
PRINT SPACE$(1);
COLOR HighWhite
PRINT Dot$;
PRINT SPACE$(1);
COLOR Gray
PRINT DarkBlock$;
PRINT SPACE$(1);
PRINT Dot$;
PRINT SPACE$(4);
COLOR HighWhite
PRINT DownDot$;
PRINT LightBlock$;
PRINT WhiteBlock$;
PRINT LightBlock$;
PRINT WhiteBlock$;
PRINT DownDot$;
PRINT SPACE$(6);
COLOR Gray
PRINT DarkBlock$;
PRINT SPACE$(1);
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT SPACE$(3);
COLOR White
PRINT Dot2$;
PRINT SPACE$(6);
COLOR HighWhite
PRINT Dot3$;
PRINT SPACE$(3);
PRINT Dot$;
PRINT Message3$;
PRINT SPACE$(1);
COLOR White
PRINT Dot2$;
PRINT SPACE$(3);
PRINT Dot$
PRINT SPACE$(2);
COLOR HighWhite
PRINT Dot3$;
PRINT SPACE$(3);
PRINT Dot$;
PRINT SPACE$(3);
COLOR White
PRINT Dot2$;
PRINT SPACE$(2);
PRINT Dot$;
PRINT SPACE$(6);
PRINT Dot2$;
PRINT SPACE$(1);
COLOR Gray
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT LightBlock$;
PRINT SPACE$(2);
COLOR White
PRINT Dot$;
PRINT SPACE$(1);
PRINT Dot3$;
PRINT SPACE$(1);
COLOR Gray
PRINT DarkBlock$;
PRINT MediumBlock$;
PRINT SPACE$(17);
COLOR White
PRINT Dot$;
PRINT SPACE$(1);
COLOR Gray
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT SPACE$(8);
COLOR HighWhite
PRINT Message4$;
PRINT SPACE$(4);
COLOR HighGreen
PRINT STRING$(4, UpDot%);
PRINT SPACE$(1);
COLOR Gray
PRINT DarkBlock$;
PRINT MediumBlock$;
PRINT DarkBlock$;
PRINT SPACE$(2);
PRINT Dot2$;
PRINT SPACE$(8);
PRINT DarkBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT MediumBlock$;
COLOR White
PRINT Dot$;
PRINT SPACE$(4);
COLOR Gray
PRINT MediumBlock$;
PRINT SPACE$(1);
PRINT MediumBlock$;
PRINT SPACE$(2);
COLOR White
PRINT Dot$;
PRINT SPACE$(3);
PRINT Dot2$;
PRINT SPACE$(3);
COLOR Gray
PRINT Dot$;
PRINT SPACE$(4);
PRINT DarkBlock$;
PRINT SPACE$(2);
PRINT DarkBlock$;
PRINT SPACE$(4);
PRINT DarkBlock$;
COLOR White
PRINT Dot2$;
PRINT SPACE$(3);
COLOR HighWhite
PRINT SPACE$(1);
PRINT Message5$
PRINT SPACE$(3);
COLOR HighGreen
PRINT UpDot$;
PRINT WhiteBlock$;
COLOR Green
PRINT LightBlock$;
COLOR HighGreen
PRINT STRING$(4, WhiteBlock%);
PRINT SPACE$(2);
COLOR Gray
PRINT DarkBlock$;
PRINT SPACE$(6);
COLOR White
PRINT Dot$;
PRINT SPACE$(5);
COLOR Gray
PRINT DarkBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT SPACE$(4);
PRINT DarkBlock$;
PRINT SPACE$(1);
PRINT LightBlock$;
PRINT SPACE$(13);
COLOR White
PRINT Dot$;
PRINT SPACE$(8);
COLOR Gray
PRINT Dot2$;
PRINT DarkBlock$;
PRINT MediumBlock$;
PRINT DarkBlock$;
PRINT SPACE$(1);
COLOR HighWhite
PRINT Dot2$;
PRINT SPACE$(2);
PRINT Message6$;
PRINT SPACE$(1)
COLOR White
PRINT SPACE$(2);
COLOR HighGreen
PRINT WhiteBlock$;
COLOR Green
PRINT LightBlock$;
COLOR HighGreen
PRINT WhiteBlock$;
PRINT WhiteBlock$;
COLOR Green
PRINT STRING$(4, LightBlock%);
COLOR HighGreen
PRINT WhiteBlock$;
COLOR HighWhite
PRINT SPACE$(4);
PRINT Dot$;
PRINT SPACE$(11);
COLOR Gray
PRINT MediumBlock$;
PRINT SPACE$(2);
COLOR White
PRINT Dot2$;
PRINT SPACE$(2);
COLOR Gray
PRINT DarkBlock$;
PRINT SPACE$(1);
PRINT LightBlock$;
PRINT Dot2$;
PRINT SPACE$(1);
COLOR White
PRINT Dot3$;
PRINT SPACE$(1);
COLOR HighWhite
PRINT Dot$;
PRINT SPACE$(3);
COLOR Brown
PRINT SPACE$(1);
PRINT DoubleBar$;
COLOR Red
PRINT WhiteBlock$;
PRINT WhiteBlock$;
PRINT UpDot$;
PRINT SPACE$(2);
COLOR White
PRINT UpDot$;
COLOR Brown
PRINT DoubleBar$;
COLOR Red
PRINT WhiteBlock$;
PRINT WhiteBlock$;
PRINT UpDot$;
PRINT SPACE$(9);
COLOR White
PRINT SPACE$(11)
PRINT SPACE$(1);
COLOR Green
PRINT STRING$(10, WhiteBlock%);
PRINT SPACE$(10);
COLOR White
PRINT Dot2$;
PRINT SPACE$(7);
COLOR Gray
PRINT MediumBlock$;
PRINT SPACE$(4);
PRINT DarkBlock$;
PRINT DarkBlock$;
PRINT MediumBlock$;
PRINT MediumBlock$;
PRINT SPACE$(6);
COLOR White
PRINT UpDot$;
COLOR Brown
PRINT DoubleBar$;
COLOR White
PRINT WhiteBlock$;
PRINT Bar$;
PRINT SPACE$(2);
PRINT WhiteBlock$;
PRINT WhiteBlock$;
COLOR Brown
PRINT DoubleBar$;
PRINT SPACE$(7);
PRINT DoubleBar$;
COLOR Red
PRINT WhiteBlock$;
PRINT UpDot$;
PRINT SPACE$(2);
COLOR Gray
PRINT DarkBlock$;
COLOR White
PRINT Dot2$
COLOR HighGreen
PRINT STRING$(13, WhiteBlock%);
PRINT SPACE$(17);
COLOR Gray
PRINT DarkBlock$;
PRINT MediumBlock$;
COLOR White
PRINT Dot2$;
PRINT SPACE$(4);
COLOR Gray
PRINT DarkBlock$;
PRINT SPACE$(3);
COLOR White
PRINT UpDot$;
PRINT UpDot$;
PRINT WhiteBlock$;
PRINT LightBlock$;
COLOR Brown
PRINT DoubleBar$;
COLOR White
PRINT LightBlock$;
PRINT WhiteBlock$;
PRINT SPACE$(1);
PRINT DarkBlock$;
PRINT LightBlock$;
PRINT WhiteBlock$;
PRINT LightBlock$;
PRINT UpDot$;
PRINT SPACE$(4);
PRINT UpDot$;
PRINT WhiteBlock$;
COLOR Brown
PRINT DoubleBar$;
PRINT SPACE$(3);
COLOR Gray
PRINT Dot2$;
PRINT DarkBlock$;
PRINT MediumBlock$;
PRINT DarkBlock$;
PRINT SPACE$(1);
COLOR White
PRINT Dot2$;
COLOR Gray
PRINT Dot2$;
PRINT SPACE$(1);
PRINT DarkBlock$;
PRINT SPACE$(1);
PRINT DarkBlock$;
COLOR White
PRINT Dot3$;
PRINT SPACE$(1);
COLOR HighWhite
PRINT Dot$
COLOR Green
PRINT WhiteBlock$;
PRINT WhiteBlock$;
PRINT STRING$(15, LightBlock%);
PRINT SPACE$(14);
COLOR Gray
PRINT DarkBlock$;
PRINT SPACE$(5);
PRINT DarkBlock$;
COLOR Brown
PRINT BackSlash$;
COLOR White
PRINT DownDot$;
PRINT WhiteBlock$;
PRINT WhiteBlock$;
PRINT LightBlock$;
PRINT WhiteBlock$;
PRINT WhiteBlock$;
COLOR Brown
PRINT DoubleBar$;
COLOR White
PRINT WhiteBlock$;
PRINT LightBlock$;
PRINT Bar$;
PRINT WhiteBlock$;
PRINT LightBlock$;
PRINT WhiteBlock$;
PRINT WhiteBlock$;
PRINT LightBlock$;
PRINT WhiteBlock$;
PRINT SPACE$(2);
PRINT WhiteBlock$;
PRINT LightBlock$;
PRINT WhiteBlock$;
COLOR Brown
PRINT DoubleBar$;
COLOR White
PRINT WhiteBlock$;
PRINT SPACE$(3);
COLOR HighWhite
PRINT Dot$;
PRINT SPACE$(7);
COLOR Gray
PRINT LightBlock$;
PRINT SPACE$(2);
COLOR White
PRINT Dot$;
PRINT SPACE$(1);
PRINT Dot3$
COLOR HighGreen
PRINT STRING$(7, LightBlock%);
PRINT MediumBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT SPACE$(8);
COLOR White
PRINT Dot2$;
PRINT SPACE$(5);
COLOR Gray
PRINT Dot2$;
PRINT SPACE$(5);
COLOR Brown
PRINT BackSlash$;
PRINT BackSlash$;
COLOR White
PRINT WhiteBlock$;
PRINT WhiteBlock$;
PRINT LightBlock$;
PRINT WhiteBlock$;
COLOR Brown
PRINT DoubleBar$;
COLOR White
PRINT DownDot$;
PRINT SPACE$(1);
COLOR White
PRINT BackSlash$;
PRINT DarkBlock$;
PRINT WhiteBlock$;
PRINT LightBlock$;
PRINT WhiteBlock$;
PRINT WhiteBlock$;
PRINT LightBlock$;
PRINT WhiteBlock$;
PRINT SPACE$(2);
PRINT WhiteBlock$;
PRINT WhiteBlock$;
COLOR Brown
PRINT DoubleBar$;
COLOR White
PRINT WhiteBlock$;
PRINT LightBlock$;
PRINT SPACE$(3);
COLOR HighWhite
PRINT Dot$;
PRINT SPACE$(3);
COLOR White
PRINT Dot3$;
PRINT SPACE$(4);
COLOR Gray
PRINT DarkBlock$;
PRINT SPACE$(1);
PRINT MediumBlock$
COLOR Green
PRINT STRING$(20, MediumBlock%);
PRINT DarkBlock$;
PRINT DarkBlock$;
PRINT DarkBlock$;
PRINT SPACE$(13);
COLOR White
PRINT Dot$;
PRINT SPACE$(3);
COLOR Brown
PRINT BackSlash$;
PRINT BackSlash$;
COLOR White
PRINT WhiteBlock$;
PRINT DownDot$;
PRINT SPACE$(1);
COLOR Brown
PRINT DoubleBar$;
COLOR White
PRINT BackSlash$;
PRINT SPACE$(2);
PRINT BackSlash$;
PRINT DownDot$;
PRINT WhiteBlock$;
PRINT WhiteBlock$;
PRINT LightBlock$;
PRINT WhiteBlock$;
PRINT WhiteBlock$;
PRINT WhiteBlock$;
PRINT Slash$;
PRINT SPACE$(1);
PRINT DownDot$;
COLOR Brown
PRINT DoubleBar$;
COLOR White
PRINT LightBlock$;
PRINT WhiteBlock$;
PRINT WhiteBlock$;
PRINT UpDot$;
PRINT SPACE$(9);
PRINT Dot2$;
PRINT SPACE$(3);
COLOR Gray
PRINT DarkBlock$
COLOR Green
PRINT MediumBlock$;
PRINT MediumBlock$;
PRINT DarkBlock$;
PRINT MediumBlock$;
PRINT DarkBlock$;
PRINT MediumBlock$;
PRINT STRING$(16, DarkBlock%);
COLOR Green
PRINT WhiteBlock$;
PRINT WhiteBlock$;
PRINT WhiteBlock$;
COLOR HighYellow
PRINT STRING$(4, WhiteBlock%);
PRINT SPACE$(8);
COLOR White
PRINT Dot$;
PRINT SPACE$(3);
COLOR Brown
PRINT BackSlash$;
COLOR Red
PRINT STRING$(4, MediumBlock%);
COLOR Brown
PRINT UnderScore$;
COLOR White
PRINT BackSlash$;
COLOR Brown
PRINT UnderScore$;
PRINT UnderScore$;
COLOR White
PRINT BackSlash$;
COLOR Brown
PRINT UnderScore$;
PRINT DoubleBar$;
COLOR White
PRINT DownDot$;
PRINT DownDot$;
PRINT DownDot$;
PRINT Slash$;
COLOR Brown
PRINT UnderScore$;
COLOR Red
PRINT MediumBlock$;
COLOR Brown
PRINT Square$;
COLOR Red
PRINT MediumBlock$;
COLOR Brown
PRINT Square$;
COLOR Red
PRINT MediumBlock$;
PRINT LightBlock$;
COLOR Brown, Red
PRINT Slash$;
COLOR HighWhite, Black
PRINT SPACE$(5);
PRINT Dot3$
COLOR Green
PRINT STRING$(6, DarkBlock%);
COLOR Green
PRINT WhiteBlock$;
COLOR HighGreen
PRINT DarkBlock$;
COLOR Green
PRINT WhiteBlock$;
COLOR HighGreen
PRINT DarkBlock$;
PRINT DarkBlock$;
PRINT DarkBlock$;
COLOR Green
PRINT WhiteBlock$;
COLOR HighGreen
PRINT DarkBlock$;
COLOR Green
PRINT WhiteBlock$;
COLOR HighGreen
PRINT DarkBlock$;
COLOR Green
PRINT STRING$(8, WhiteBlock%);
COLOR HighYellow
PRINT STRING$(7, WhiteBlock%);
COLOR Blue
PRINT STRING$(8, WhiteBlock%);
COLOR HighWhite, Blue
PRINT LightBlock$;
PRINT MediumBlock$;
COLOR Blue, Black
PRINT WhiteBlock$;
COLOR Red
PRINT MediumBlock$;
PRINT MediumBlock$;
PRINT MediumBlock$;
COLOR Brown
PRINT Square$;
COLOR Red
PRINT MediumBlock$;
COLOR Brown
PRINT Square$;
COLOR Red
PRINT MediumBlock$;
COLOR Brown
PRINT Square$;
COLOR Red
PRINT MediumBlock$;
COLOR Brown
PRINT Square$;
COLOR Red
PRINT MediumBlock$;
COLOR Brown
PRINT Square$;
COLOR Red
PRINT MediumBlock$;
COLOR Brown
PRINT Square$;
COLOR Red
PRINT MediumBlock$;
COLOR Brown
PRINT Square$;
COLOR Red
PRINT STRING$(4, MediumBlock%);
PRINT LightBlock$;
COLOR Brown, Red
PRINT Slash$;
COLOR Blue, Black
PRINT STRING$(16, WhiteBlock%);
COLOR Green
PRINT STRING$(21, WhiteBlock%);
COLOR HighYellow
PRINT WhiteBlock$;
PRINT WhiteBlock$;
PRINT LightBlock$;
PRINT WhiteBlock$;
PRINT WhiteBlock$;
COLOR Blue
PRINT MediumBlock$;
PRINT MediumBlock$;
PRINT STRING$(10, LightBlock%);
COLOR HighWhite, Blue
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT DarkBlock$;
COLOR Blue, Black
PRINT LightBlock$;
PRINT LightBlock$;
COLOR Red
PRINT STRING$(18, MediumBlock%);
PRINT LightBlock$;
COLOR Brown, Red
PRINT Slash$;
COLOR Blue, Black
PRINT STRING$(10, LightBlock%);
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
COLOR Green
PRINT STRING$(4, WhiteBlock%);
PRINT LightBlock$;
PRINT WhiteBlock$;
PRINT WhiteBlock$;
PRINT WhiteBlock$;
PRINT LightBlock$;
PRINT WhiteBlock$;
PRINT LightBlock$;
PRINT STRING$(6, WhiteBlock%);
COLOR HighYellow
PRINT STRING$(4, WhiteBlock%);
PRINT LightBlock$;
PRINT LightBlock$;
PRINT WhiteBlock$;
COLOR Blue
PRINT STRING$(7, MediumBlock%);
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
COLOR HighWhite, Blue
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT DarkBlock$;
PRINT MediumBlock$;
COLOR Blue, Black
PRINT MediumBlock$;
PRINT MediumBlock$;
PRINT MediumBlock$;
COLOR Red
PRINT STRING$(16, LightBlock%);
COLOR Brown, Red
PRINT Slash$;
COLOR Blue, Black
PRINT STRING$(18, MediumBlock%);
COLOR Green
PRINT WhiteBlock$;
PRINT WhiteBlock$;
PRINT STRING$(11, LightBlock%);
COLOR HighYellow
PRINT STRING$(4, LightBlock%);
PRINT WhiteBlock$;
PRINT LightBlock$;
PRINT WhiteBlock$;
PRINT STRING$(6, LightBlock%);
PRINT MediumBlock$;
COLOR Blue
PRINT DarkBlock$;
PRINT DarkBlock$;
PRINT DarkBlock$;
PRINT STRING$(7, MediumBlock%);
COLOR Blue, White
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT DarkBlock$;
COLOR HighWhite, Blue
PRINT LightBlock$;
COLOR Blue, Black
PRINT STRING$(23, MediumBlock%);
PRINT STRING$(4, DarkBlock%);
COLOR HighYellow
PRINT STRING$(11, LightBlock%);
COLOR Green
PRINT LightBlock$;
PRINT LightBlock$;
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT DarkBlock$;
PRINT LightBlock$;
COLOR HighYellow
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT MediumBlock$;
COLOR Blue
PRINT STRING$(5, DarkBlock%);
PRINT MediumBlock$;
PRINT DarkBlock$;
PRINT DarkBlock$;
COLOR HighWhite, Blue
PRINT DarkBlock$;
COLOR Blue, White
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
COLOR HighWhite, Blue
PRINT DarkBlock$;
COLOR Blue, Black
PRINT DarkBlock$;
PRINT DarkBlock$;
PRINT DarkBlock$;
PRINT MediumBlock$;
PRINT DarkBlock$;
PRINT MediumBlock$;
PRINT DarkBlock$;
PRINT MediumBlock$;
PRINT DarkBlock$;
PRINT DarkBlock$;
PRINT MediumBlock$;
PRINT MediumBlock$;
PRINT DarkBlock$;
PRINT MediumBlock$;
PRINT STRING$(5, DarkBlock%);
COLOR HighYellow
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT STRING$(6, LightBlock%);
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
COLOR Green
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT STRING$(9, MediumBlock%);
PRINT DarkBlock$;
COLOR HighYellow
PRINT STRING$(8, MediumBlock%);
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT MediumBlock$;
COLOR Blue
PRINT STRING$(4, DarkBlock%);
COLOR HighWhite, Blue
PRINT DarkBlock$;
COLOR Blue, White
PRINT LightBlock$;
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
COLOR HighWhite, Blue
PRINT DarkBlock$;
PRINT MediumBlock$;
COLOR Blue, Black
PRINT STRING$(13, DarkBlock%);
COLOR HighYellow
PRINT MediumBlock$;
PRINT MediumBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT LightBlock$;
PRINT STRING$(5, MediumBlock%);
PRINT LightBlock$;
PRINT STRING$(4, MediumBlock%);
COLOR Green
PRINT MediumBlock$;
PRINT MediumBlock$;
PRINT MediumBlock$;
PRINT DarkBlock$;
PRINT MediumBlock$;
PRINT DarkBlock$;
PRINT MediumBlock$;
PRINT DarkBlock$;
PRINT MediumBlock$;
PRINT STRING$(5, DarkBlock%);
COLOR HighYellow
PRINT MediumBlock$;
PRINT MediumBlock$;
PRINT DarkBlock$;
PRINT STRING$(12, MediumBlock%);
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT STRING$(4, MediumBlock%);
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT LightBlock$;
PRINT LightBlock$;
PRINT MediumBlock$;
PRINT LightBlock$;
PRINT STRING$(6, MediumBlock%);
PRINT LightBlock$;
PRINT STRING$(11, MediumBlock%);
PRINT DarkBlock$;
PRINT MediumBlock$;
PRINT MediumBlock$;
PRINT DarkBlock$;
PRINT DarkBlock$;
PRINT MediumBlock$;
COLOR Green
PRINT MediumBlock$;
PRINT DarkBlock$;
PRINT MediumBlock$;
PRINT DarkBlock$;
PRINT MediumBlock$;
PRINT STRING$(5, DarkBlock%);
COLOR HighYellow
PRINT DarkBlock$;
PRINT MediumBlock$;
PRINT STRING$(4, DarkBlock%);
PRINT MediumBlock$;
PRINT DarkBlock$;
PRINT MediumBlock$;
PRINT MediumBlock$;
PRINT DarkBlock$;
PRINT STRING$(4, MediumBlock%);
PRINT DarkBlock$;
PRINT MediumBlock$;
PRINT DarkBlock$;
PRINT MediumBlock$;
PRINT DarkBlock$;
PRINT STRING$(7, MediumBlock%);
PRINT DarkBlock$;
PRINT MediumBlock$;
PRINT DarkBlock$;
PRINT MediumBlock$;
PRINT DarkBlock$;
PRINT MediumBlock$;
PRINT MediumBlock$;
PRINT DarkBlock$;
PRINT STRING$(10, MediumBlock%);
PRINT DarkBlock$;
PRINT MediumBlock$;
PRINT DarkBlock$;
PRINT DarkBlock$;
PRINT STRING$(5, MediumBlock%);
PRINT DarkBlock$;
PRINT MediumBlock$;
PRINT MediumBlock$;
PRINT DarkBlock$;
PRINT MediumBlock$;
PRINT DarkBlock$;
PRINT MediumBlock$;
PRINT MediumBlock$;
PRINT STRING$(8, DarkBlock%)
COLOR White, Black ' reset color to HighWhite on black
LOCATE 23, 1, 1 ' reset cursor to screen bottom
PRINT SPACE$(80); ' clear the bottom line
END SUB

REM The Dungeon Help Utility v12.0 r3.0 Program Source
SUB Display.Help.Menu
' help function/0 key
DO ' loop to redisplay help screen
    GOSUB Display.Help.Screen ' display help screen
    DO ' loop to get key of function to display help on
        x$ = "" ' get keystroke
        WHILE x$ = "" ' wait for input
            _LIMIT 100
            x$ = INKEY$ ' get keystroke
        WEND ' end wait
        SELECT CASE LEN(x$)
            ' before any help is displayed,
            ' output clear screen.
            CASE 1
                SELECT CASE x$
                    CASE CHR$(1) TO CHR$(10) ' use staff keys
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen77
                        EXIT DO
                    CASE CHR$(17) TO CHR$(26) ' use ring keys
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen78
                        EXIT DO
                    CASE "+" ' info key
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen41
                        EXIT DO
                    CASE "1" ' fight key
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen42
                        EXIT DO
                    CASE "2" ' move south
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen43
                        EXIT DO
                    CASE "3" ' cast key
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen44
                        EXIT DO
                    CASE "4" ' move west
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen45
                        EXIT DO
                    CASE "5", CHR$(13) ' enter/accept keys
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen46
                        EXIT DO
                    CASE "6" ' move east
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen47
                        EXIT DO
                    CASE "7" ' evade key
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen48
                        EXIT DO
                    CASE "8" ' move north
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen49
                        EXIT DO
                    CASE "9" ' ignore key
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen50
                        EXIT DO
                    CASE "." ' stats key
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen51
                        EXIT DO
                    CASE "-" ' chants key
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen52
                        EXIT DO
                    CASE "x", "X" ' players symbol
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen53
                        EXIT DO
                    CASE "#" ' monster symbol
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen54
                        EXIT DO
                    CASE "~" ' scroll symbol
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen55
                        EXIT DO
                    CASE "$" ' gold symbol
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen56
                        EXIT DO
                    CASE CHR$(127) ' chest symbol
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen57
                        EXIT DO
                    CASE "^" ' trap symbol
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen58
                        EXIT DO
                    CASE "o", "O" ' pit symbol
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen59
                        EXIT DO
                    CASE "*" ' gem symbol
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen60
                        EXIT DO
                    CASE "!" ' teleporter symbol
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen61
                        EXIT DO
                    CASE "?" ' temple symbol
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen62
                        EXIT DO
                    CASE "0" ' help key
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen63
                        EXIT DO
                    CASE ">" ' stairs up symbol
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen64
                        EXIT DO
                    CASE "<" ' stairs down symbol
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen65
                        EXIT DO
                    CASE "\" ' trade key
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen66
                        EXIT DO
                    CASE ";" ' buy key
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen80
                        EXIT DO
                    CASE "=" ' change level key
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen67
                        EXIT DO
                    CASE "'" ' use magic item key
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen68
                        EXIT DO
                    CASE "/" ' inventory key
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen69
                        EXIT DO
                    CASE "|" ' wand symbol
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen70
                        EXIT DO
                    CASE "@" ' potion symbol
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen71
                        EXIT DO
                    CASE "%" ' staff symbol
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen74
                        EXIT DO
                    CASE "&" ' coins symbol
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen75
                        EXIT DO
                    CASE CHR$(34) ' ring symbol
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen76
                        EXIT DO
                    CASE "_" ' edit user
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen82
                        EXIT DO
                    CASE "`" ' user list
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen81
                        EXIT DO
                    CASE " " ' exit help
                        COLOR 7, 0
                        CLS
                        EXIT SUB ' exit routine
                END SELECT
            CASE 2
                sp = ASC(MID$(x$, 2)) ' convert extended acsii input key
                SELECT CASE sp ' secondary/extended key functions
                    CASE 81 ' PageDown, fight key
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen42
                        EXIT DO
                    CASE 80 ' Down, move player south
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen43
                        EXIT DO
                    CASE 79 ' End, cast key
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen44
                        EXIT DO
                    CASE 75 ' Left, move player west
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen45
                        EXIT DO
                    CASE 76 ' Center, acceptance key
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen46
                        EXIT DO
                    CASE 77 ' Right, move player east
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen47
                        EXIT DO
                    CASE 73 ' PageUp, evade key
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen48
                        EXIT DO
                    CASE 72 ' Up, move player north
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen49
                        EXIT DO
                    CASE 71 ' Home, ignore key
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen50
                        EXIT DO
                    CASE 82 ' Insert, help key
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen63
                        EXIT DO
                    CASE 116 ' Control-Right, info key
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen41
                        EXIT DO
                    CASE 83 ' Delete, stats key
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen51
                        EXIT DO
                    CASE 118 ' Control-PageDown, chants key
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen52
                        EXIT DO
                    CASE 119 ' Control-Home, trade key
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen66
                        EXIT DO
                    CASE 15 ' Shift-Tab, buy key
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen80
                        EXIT DO
                    CASE 117 ' Control-End, change level key
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen67
                        EXIT DO
                    CASE 115 ' Control-Left, inventory key
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen69
                        EXIT DO
                    CASE 130 ' Alt--, edit user
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen82
                        EXIT DO
                    CASE 131 ' Alt-=, user list
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen81
                        EXIT DO
                    CASE 132 ' Control-PageUp, use key
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen68
                        EXIT DO
                    CASE 30, 48, 46, 32, 18, 33, 34, 35, 23, 36 ' use wand keys
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen72
                        EXIT DO
                    CASE 16, 19, 31, 20, 22, 47, 17, 45, 21, 44 ' use potion keys
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen73
                        EXIT DO
                    CASE 120 TO 129 ' use power globe keys
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen79
                        EXIT DO
                    CASE 147 ' Ctrl-Delete, DOS shell key
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen83
                        EXIT DO
                    CASE 152 ' Alt-Up, Map Edit key
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen84
                        EXIT DO
                    CASE 155 ' Alt-Left, Power Globe key
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen85
                        EXIT DO
                    CASE 133 ' F11
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen86
                        EXIT DO
                    CASE 134 ' F12
                        COLOR 14, 1: CLS
                        GOSUB Help.Screen87
                        EXIT DO
                    CASE 59 TO 68 ' spell type 1, F1 to F10
                        sp = sp - 58 ' calculate spell number 1 to 10
                        SELECT CASE sp ' display spell help screen
                            CASE 1
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen1
                                EXIT DO
                            CASE 2
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen2
                                EXIT DO
                            CASE 3
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen3
                                EXIT DO
                            CASE 4
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen4
                                EXIT DO
                            CASE 5
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen5
                                EXIT DO
                            CASE 6
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen6
                                EXIT DO
                            CASE 7
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen7
                                EXIT DO
                            CASE 8
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen8
                                EXIT DO
                            CASE 9
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen9
                                EXIT DO
                            CASE 10
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen10
                                EXIT DO
                        END SELECT
                    CASE 94 TO 103 ' spell type 2, Control-F1 to Control-F10
                        sp = sp - 93 ' calculate spell number 1 to 10
                        SELECT CASE sp ' display spell help screen
                            CASE 1
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen11
                                EXIT DO
                            CASE 2
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen12
                                EXIT DO
                            CASE 3
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen13
                                EXIT DO
                            CASE 4
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen14
                                EXIT DO
                            CASE 5
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen15
                                EXIT DO
                            CASE 6
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen16
                                EXIT DO
                            CASE 7
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen17
                                EXIT DO
                            CASE 8
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen18
                                EXIT DO
                            CASE 9
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen19
                                EXIT DO
                            CASE 10
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen20
                                EXIT DO
                        END SELECT
                    CASE 84 TO 93 ' spell type 3, Shift-F1 to Shift-F10
                        sp = sp - 83 ' calculate spell number 1 to 10
                        SELECT CASE sp ' display spell help screen
                            CASE 1
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen21
                                EXIT DO
                            CASE 2
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen22
                                EXIT DO
                            CASE 3
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen23
                                EXIT DO
                            CASE 4
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen24
                                EXIT DO
                            CASE 5
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen25
                                EXIT DO
                            CASE 6
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen26
                                EXIT DO
                            CASE 7
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen27
                                EXIT DO
                            CASE 8
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen28
                                EXIT DO
                            CASE 9
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen29
                                EXIT DO
                            CASE 10
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen30
                                EXIT DO
                        END SELECT
                    CASE 104 TO 113 ' spell type 4, Alt-F1 to Alt-F10
                        sp = sp - 103 ' calculate spell number 1 to 10
                        SELECT CASE sp ' display spell help screen
                            CASE 1
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen31
                                EXIT DO
                            CASE 2
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen32
                                EXIT DO
                            CASE 3
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen33
                                EXIT DO
                            CASE 4
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen34
                                EXIT DO
                            CASE 5
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen35
                                EXIT DO
                            CASE 6
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen36
                                EXIT DO
                            CASE 7
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen37
                                EXIT DO
                            CASE 8
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen38
                                EXIT DO
                            CASE 9
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen39
                                EXIT DO
                            CASE 10
                                COLOR 14, 1: CLS
                                GOSUB Help.Screen40
                                EXIT DO
                        END SELECT
                END SELECT
        END SELECT
    LOOP
LOOP
COLOR 7, 0
CLS
EXIT SUB ' exit routine

Help.Screen1:
LOCATE 10, 15, 0
PRINT "The heal spell restores hit points to your player."
LOCATE 11, 15, 0
PRINT "The amount restored is equal to your constitution."
GOSUB Key.Pressed
RETURN

Help.Screen2:
LOCATE 10, 15, 0
PRINT "The protection from evil spell repells a monster's"
LOCATE 11, 15, 0
PRINT "attacks for one round. The number of hits repelled"
LOCATE 12, 15, 0
PRINT "is equal to your dexterity."
GOSUB Key.Pressed
RETURN

Help.Screen3:
LOCATE 10, 15, 0
PRINT "The magic missile spell causes a dartlike object to"
LOCATE 11, 15, 0
PRINT "fly from your player's hand.  This missile hits the"
LOCATE 12, 15, 0
PRINT "monster with exact accuracy every time. The amount"
LOCATE 13, 15, 0
PRINT "of damage normally done is doubled."
GOSUB Key.Pressed
RETURN

Help.Screen4:
LOCATE 10, 15, 0
PRINT "The shield spell generates an invisible force field"
LOCATE 11, 15, 0
PRINT "between your player and the monster.  Your player's"
LOCATE 12, 15, 0
PRINT "total armor plus is increased  by one armor factor."
GOSUB Key.Pressed
RETURN

Help.Screen5:
LOCATE 10, 15, 0
PRINT "The strength spell doubles your player's strength"
LOCATE 11, 15, 0
PRINT "for the duration of one round."
GOSUB Key.Pressed
RETURN

Help.Screen6:
LOCATE 10, 15, 0
PRINT "The sleep spell binds the monster in a helpless"
LOCATE 11, 15, 0
PRINT "state for the duration of one round."
GOSUB Key.Pressed
RETURN

Help.Screen7:
LOCATE 10, 15, 0
PRINT "The locate spell  reveals nearby traps and pits."
LOCATE 11, 15, 0
PRINT "This spell works only for the immediate vicinity"
LOCATE 12, 15, 0
PRINT "and has no duration."
GOSUB Key.Pressed
RETURN

Help.Screen8:
LOCATE 10, 15, 0
PRINT "The pyrotechnics spell causes a bolt of fire to fly"
LOCATE 11, 15, 0
PRINT "from your player's hands. The amount of damage done"
LOCATE 12, 15, 0
PRINT "normally is doubled."
GOSUB Key.Pressed
RETURN

Help.Screen9:
LOCATE 10, 15, 0
PRINT "The firestorm spell actuates a small blast of flames"
LOCATE 11, 15, 0
PRINT "to engulf the monster. The amount of damage normally"
LOCATE 12, 15, 0
PRINT "done is doubled."
GOSUB Key.Pressed
RETURN

Help.Screen10:
LOCATE 10, 15, 0
PRINT "The transmute spell restructures the surrounding air"
LOCATE 11, 15, 0
PRINT "into gold. This gold can then be picked up."
GOSUB Key.Pressed
RETURN

Help.Screen11:
LOCATE 10, 15, 0
PRINT "The restoration spell restores HIT POINTS to your"
LOCATE 11, 15, 0
PRINT "player.  The  amount  restored is equal to double"
LOCATE 12, 15, 0
PRINT "your constitution."
GOSUB Key.Pressed
RETURN

Help.Screen12:
LOCATE 10, 15, 0
PRINT "The scare spell will repell a monster's attacks for"
LOCATE 11, 15, 0
PRINT "two rounds. The amount of hits repelled is equal to"
LOCATE 12, 15, 0
PRINT "your dexterity."
GOSUB Key.Pressed
RETURN

Help.Screen13:
LOCATE 10, 15, 0
PRINT "The fireball spell causes a fireball to fly from your"
LOCATE 11, 15, 0
PRINT "player's hands. The amount of damage normally done to"
LOCATE 12, 15, 0
PRINT "the monster is tripled."
GOSUB Key.Pressed
RETURN

Help.Screen14:
LOCATE 10, 15, 0
PRINT "The obscurement spell creates an opaque force field"
LOCATE 11, 15, 0
PRINT "between your player and the monster.  Your player's"
LOCATE 12, 15, 0
PRINT "total armor plus is increased by two armor factors."
GOSUB Key.Pressed
RETURN

Help.Screen15:
LOCATE 10, 15, 0
PRINT "The power shield spell doubles your player's strength"
LOCATE 11, 15, 0
PRINT "for the duration of two rounds."
GOSUB Key.Pressed
RETURN

Help.Screen16:
LOCATE 10, 15, 0
PRINT "The wall of will spell renders the monster helpless"
LOCATE 11, 15, 0
PRINT "for the duration of two rounds."
GOSUB Key.Pressed
RETURN

Help.Screen17:
LOCATE 10, 15, 0
PRINT "The light spell illuminates an extended area around"
LOCATE 11, 15, 0
PRINT "your player. This spell has no duration."
GOSUB Key.Pressed
RETURN

Help.Screen18:
LOCATE 10, 15, 0
PRINT "The levitate spell causes your player to float above"
LOCATE 11, 15, 0
PRINT "the dungeon walls uncontrollably for the duration of"
LOCATE 12, 15, 0
PRINT "two to five rounds."
GOSUB Key.Pressed
RETURN

Help.Screen19:
LOCATE 10, 15, 0
PRINT "The monster summon spell calls up one to four"
LOCATE 11, 15, 0
PRINT "monsters of the current monster level."
GOSUB Key.Pressed
RETURN

Help.Screen20:
LOCATE 10, 15, 0
PRINT "The finger of death spell instantly destroys the"
LOCATE 11, 15, 0
PRINT "monster.  The monster's treasure remains intact."
GOSUB Key.Pressed
RETURN

Help.Screen21:
LOCATE 10, 15, 0
PRINT "The regenerate spell restores hit points to your"
LOCATE 11, 15, 0
PRINT "player.  The amount  restored is equal to triple"
LOCATE 12, 15, 0
PRINT "your constitution."
GOSUB Key.Pressed
RETURN

Help.Screen22:
LOCATE 10, 15, 0
PRINT "The forget spell will  repell a monster's attacks for"
LOCATE 11, 15, 0
PRINT "three rounds. The amount of hits repelled is equal to"
LOCATE 12, 15, 0
PRINT "your dexterity."
GOSUB Key.Pressed
RETURN

Help.Screen23:
LOCATE 10, 15, 0
PRINT "The lightning bolt  spell causes bolts of electricity"
LOCATE 11, 15, 0
PRINT "to fly from your player's hands. The amount of damage"
LOCATE 12, 15, 0
PRINT "normally done to the  monster is multiplied  by four."
GOSUB Key.Pressed
RETURN

Help.Screen24:
LOCATE 10, 15, 0
PRINT "The phantasmal force spell creates a solid wall between"
LOCATE 11, 15, 0
PRINT "your player and the monster.  Your player's total armor"
LOCATE 12, 15, 0
PRINT "plus is increased by three armor factors."
GOSUB Key.Pressed
RETURN

Help.Screen25:
LOCATE 10, 15, 0
PRINT "The wall of force spell doubles your player's strength."
LOCATE 11, 15, 0
PRINT "This spell has a duration of three rounds."
GOSUB Key.Pressed
RETURN

Help.Screen26:
LOCATE 10, 15, 0
PRINT "The charm monster spell renders the monster helpless."
LOCATE 11, 15, 0
PRINT "This spell has a duration of three rounds."
GOSUB Key.Pressed
RETURN

Help.Screen27:
LOCATE 10, 15, 0
PRINT "The continual light spell illuminates an extended area"
LOCATE 11, 15, 0
PRINT "around your player.  This spell has a duration of four"
LOCATE 12, 15, 0
PRINT "rounds."
GOSUB Key.Pressed
RETURN

Help.Screen28:
LOCATE 10, 15, 0
PRINT "The passwall spell allows your player to travel"
LOCATE 11, 15, 0
PRINT "through the dungeon walls. This spell will stay"
LOCATE 12, 15, 0
PRINT "in effect while in a wall. It has a duration of"
LOCATE 13, 15, 0
PRINT "two to four rounds."
GOSUB Key.Pressed
RETURN

Help.Screen29:
LOCATE 10, 15, 0
PRINT "The demon summon spell calls up one to three demons"
LOCATE 11, 15, 0
PRINT "of level twenty to fifty. Demons have been found to"
LOCATE 12, 15, 0
PRINT "have vast amounts of treasure."
GOSUB Key.Pressed
RETURN

Help.Screen30:
LOCATE 10, 15, 0
PRINT "The doomkill spell  instantly demolishes all monsters"
LOCATE 11, 15, 0
PRINT "surrounding you. The monster's treasures will remain."
LOCATE 12, 15, 0
PRINT "This spell has no duration."
GOSUB Key.Pressed
RETURN

Help.Screen31:
LOCATE 10, 15, 0
PRINT "The resurrection spell restores hit points to your"
LOCATE 11, 15, 0
PRINT "player. The amount restored is equal to four times"
LOCATE 12, 15, 0
PRINT "your constitution."
GOSUB Key.Pressed
RETURN

Help.Screen32:
LOCATE 10, 15, 0
PRINT "The fear spell will repell a monster's attacks for"
LOCATE 11, 15, 0
PRINT "four rounds.  The amount of hits repelled is equal"
LOCATE 12, 15, 0
PRINT "to your dexterity."
GOSUB Key.Pressed
RETURN

Help.Screen33:
LOCATE 10, 15, 0
PRINT "The disintegrate spell causes energy to be drained"
LOCATE 11, 15, 0
PRINT "from the nearest attacking monster.  The amount of"
LOCATE 12, 15, 0
PRINT "damaged done normally is multiplied by five."
GOSUB Key.Pressed
RETURN

Help.Screen34:
LOCATE 10, 15, 0
PRINT "The blade barrier spell  creates a rotating  wall of"
LOCATE 11, 15, 0
PRINT "swords  between your  player and  the monster.  Your"
LOCATE 12, 15, 0
PRINT "player's total armor plus is increased by four armor"
LOCATE 13, 15, 0
PRINT "factors."
GOSUB Key.Pressed
RETURN

Help.Screen35:
LOCATE 10, 15, 0
PRINT "The enchanted weapon spell doubles your weapon plus."
LOCATE 11, 15, 0
PRINT "This spell has a duration of three rounds."
GOSUB Key.Pressed
RETURN

Help.Screen36:
LOCATE 10, 15, 0
PRINT "The hold monster spell renders the monster helpless."
LOCATE 11, 15, 0
PRINT "This spell lasts up to four rounds."
GOSUB Key.Pressed
RETURN

Help.Screen37:
LOCATE 10, 15, 0
PRINT "The teleport spell  causes your player to reappear in"
LOCATE 11, 15, 0
PRINT "another area of the dungeon level. Your player has no"
LOCATE 12, 15, 0
PRINT "control over teleportation."
GOSUB Key.Pressed
RETURN

Help.Screen38:
LOCATE 10, 15, 0
PRINT "The polymorph spell instantly devastates all monsters"
LOCATE 11, 15, 0
PRINT "within three squares of your player. This spell lasts"
LOCATE 12, 15, 0
PRINT "for one round."
GOSUB Key.Pressed
RETURN

Help.Screen39:
LOCATE 10, 15, 0
PRINT "The time stop spell  nullifies all monster's  attacks"
LOCATE 11, 15, 0
PRINT "around your player. This spell lasts for five rounds."
GOSUB Key.Pressed
RETURN

Help.Screen40:
LOCATE 10, 15, 0
PRINT "The genocide spell annihilates  all monsters within"
LOCATE 11, 15, 0
PRINT "three squares of your player. This spell's duration"
LOCATE 12, 15, 0
PRINT "lasts for three rounds."
GOSUB Key.Pressed
RETURN

Help.Screen41:
LOCATE 10, 15, 0
PRINT "The info key displays current monster hits and experience,"
LOCATE 11, 15, 0
PRINT "average, maximum, and minimum  statistics for  monsters at"
LOCATE 12, 15, 0
PRINT "the current dungeon level, attack formulas, and your total"
LOCATE 13, 15, 0
PRINT "armor and weapon plus."
GOSUB Key.Pressed
RETURN

Help.Screen42:
LOCATE 10, 15, 0
PRINT "The fight key causes your player to attack a monster."
LOCATE 11, 15, 0
PRINT "While fighting monsters,  many factors are considered"
LOCATE 12, 15, 0
PRINT "such as your armor plus, your weapon plus,  and other"
LOCATE 13, 15, 0
PRINT "conditions created by casted spells."
GOSUB Key.Pressed
RETURN

Help.Screen43:
LOCATE 10, 15, 0
PRINT "This key moves your player to the south one dungeon"
LOCATE 11, 15, 0
PRINT "square. Occupied squares next to your player create"
LOCATE 12, 15, 0
PRINT "conditions such as opening chests, retreiving gold,"
LOCATE 13, 15, 0
PRINT "and engaging monsters in combat."
GOSUB Key.Pressed
RETURN

Help.Screen44:
LOCATE 10, 15, 0
PRINT "The cast key causes your player to cast a spell chant."
LOCATE 11, 15, 0
PRINT "In order to cast spells chants, they must be collected"
LOCATE 12, 15, 0
PRINT "and memorized during the Adventure."
GOSUB Key.Pressed
RETURN

Help.Screen45:
LOCATE 10, 15, 0
PRINT "This key moves your player to the west one dungeon"
LOCATE 11, 15, 0
PRINT "square. Moving on a square creates certain unknown"
LOCATE 12, 15, 0
PRINT "conditions such as triggering a trap, falling in a"
LOCATE 13, 15, 0
PRINT "pit, and discovering scrolls."
GOSUB Key.Pressed
RETURN

Help.Screen46:
LOCATE 10, 15, 0
PRINT "The enter key on the numeric keypad is used as an"
LOCATE 11, 15, 0
PRINT "'acceptance' key for various prompts such opening"
LOCATE 12, 15, 0
PRINT "and storing inventory from chests, and picking up"
LOCATE 13, 15, 0
PRINT "treasure."
GOSUB Key.Pressed
RETURN

Help.Screen47:
LOCATE 10, 15, 0
PRINT "This key moves your player to the east one dungeon"
LOCATE 11, 15, 0
PRINT "square. Your player can move to other parts of the"
LOCATE 12, 15, 0
PRINT "the dungeon level with certain spells. Among these"
LOCATE 13, 15, 0
PRINT "spells are levitate, passwall, teleport."
GOSUB Key.Pressed
RETURN

Help.Screen48:
LOCATE 10, 15, 0
PRINT "The evade key causes your player to dodge the"
LOCATE 11, 15, 0
PRINT "current monster (if your player is in combat)"
LOCATE 12, 15, 0
PRINT "and move to an adjacent square."
GOSUB Key.Pressed
RETURN

Help.Screen49:
LOCATE 10, 15, 0
PRINT "This key moves your player to the north one dungeon"
LOCATE 11, 15, 0
PRINT "square.  To change dungeon levels, move your player"
LOCATE 12, 15, 0
PRINT "through the upper and lower exits."
GOSUB Key.Pressed
RETURN

Help.Screen50:
LOCATE 10, 15, 0
PRINT "The ignore key responds as a 'refuse' key for some"
LOCATE 11, 15, 0
PRINT "prompts. These prompts include opening and storing"
LOCATE 12, 15, 0
PRINT "inventory from chests and picking up treasure."
GOSUB Key.Pressed
RETURN

Help.Screen51:
LOCATE 8, 16, 0
PRINT "The stats key displays your player character's"
LOCATE 9, 16, 0
PRINT "attributes, armor pluses, and weapon plus. The"
LOCATE 10, 16, 0
PRINT "stats and their  abbreviations are as follows:"
LOCATE 12, 11, 0
PRINT "STR - Strength       INT - Intelligence   WIS - Wisdom"
LOCATE 13, 11, 0
PRINT "CON - Constitution   DEX - Dexterity      CHA - Charisma"
LOCATE 14, 11, 0
PRINT "WE  - Weapon         AR  - Armor          SH  - Shield"
LOCATE 15, 11, 0
PRINT "CL  - Cloak          HE  - Helmet         BO  - Boots"
GOSUB Key.Pressed
RETURN

Help.Screen52:
LOCATE 10, 15, 0
PRINT "The chants key displays all the chants your player has"
LOCATE 11, 15, 0
PRINT "remembered. Your player can read spell chants found on"
LOCATE 12, 15, 0
PRINT "scrolls hidden in the dungeon."
GOSUB Key.Pressed
RETURN

Help.Screen53:
LOCATE 10, 15, 0
PRINT "The X is your player.  Move your player with the"
LOCATE 11, 15, 0
PRINT "numeric keypad cursor keys. You will be prompted"
LOCATE 12, 15, 0
PRINT "for  an action when you move your player next to"
LOCATE 13, 15, 0
PRINT "a symbol. For example, you move your player next"
LOCATE 14, 15, 0
PRINT "to a scroll and are asked to take or  ignore it."
GOSUB Key.Pressed
RETURN

Help.Screen54:
LOCATE 10, 15, 0
PRINT "The # symbol is a monster. Monsters usually carry some"
LOCATE 11, 15, 0
PRINT "treasure, such as gold, gems, or chests. Monsters have"
LOCATE 12, 15, 0
PRINT "statistics like your player, including experience, hit"
LOCATE 13, 15, 0
PRINT "points, and strength."
GOSUB Key.Pressed
RETURN

Help.Screen55:
LOCATE 10, 15, 0
PRINT "The ~ symbol is a scroll. Your player can read and"
LOCATE 11, 15, 0
PRINT "memorize scrolls for later use. In order to cast a"
LOCATE 12, 15, 0
PRINT "spell chant learned from a scroll use the '-' key."
GOSUB Key.Pressed
RETURN

Help.Screen56:
LOCATE 10, 15, 0
PRINT "The $ symbol is gold.  Taking gold increases your hoard."
LOCATE 11, 15, 0
PRINT "Gold is needed for reaching the next level of expertise."
GOSUB Key.Pressed
RETURN

Help.Screen57:
LOCATE 10, 15, 0
PRINT "The " + CHR$(127) + " symbol is a chest. Chests can be opened, and usually"
LOCATE 11, 15, 0
PRINT "contain  armor  and  weapons.  It is said there was once a"
LOCATE 12, 15, 0
PRINT "Globe of extreme power locked in a chest. This Globe could"
LOCATE 13, 15, 0
PRINT "unleash the immense power of the  past and  allow the user"
LOCATE 14, 15, 0
PRINT "to control whatever he desired."
GOSUB Key.Pressed
RETURN

Help.Screen58:
LOCATE 10, 15, 0
PRINT "The ^ symbol represents a trap. Traps are not visible"
LOCATE 11, 15, 0
PRINT "until  triggered or Located (with the locate spell)."
LOCATE 12, 15, 0
PRINT "Traps in  higher dungeon levels  cause more damage to"
LOCATE 13, 15, 0
PRINT "your player. Traps block dungeon passages."
GOSUB Key.Pressed
RETURN

Help.Screen59:
LOCATE 10, 15, 0
PRINT "The o symbol is a pit. Pits become visible when your"
LOCATE 11, 15, 0
PRINT "player has fallen in.  Pits in higher dungeon levels"
LOCATE 12, 15, 0
PRINT "cause more damage to your player.  Some spells allow"
LOCATE 13, 15, 0
PRINT "your player  to move around pits,  such as levitate,"
LOCATE 14, 15, 0
PRINT "passwall, and teleport."
GOSUB Key.Pressed
RETURN

Help.Screen60:
LOCATE 10, 15, 0
PRINT "The * symbol are gems. When you land on gems then your"
LOCATE 11, 15, 0
PRINT "player picks  them up.  They are  converted to gold in"
LOCATE 12, 15, 0
PRINT "your  inventory.  Gold is  needed  to increase  to the"
LOCATE 13, 15, 0
PRINT "next level when you have enough experience."
GOSUB Key.Pressed
RETURN

Help.Screen61:
LOCATE 10, 15, 0
PRINT "The ! symbol is a teleporter. You can either land on"
LOCATE 11, 15, 0
PRINT "a teleporter or ignore it. Entering a teleporter can"
LOCATE 12, 15, 0
PRINT "move  your player to  another dungeon  level or some"
LOCATE 13, 15, 0
PRINT "place  in the  current level.  Teleporters disappear"
LOCATE 14, 15, 0
PRINT "after they have been entered."
GOSUB Key.Pressed
RETURN

Help.Screen62:
LOCATE 10, 15, 0
PRINT "The ? symbol is a temple. You can either pray at the"
LOCATE 11, 15, 0
PRINT "temple,  pillage it or ignore it.  Temples disappear"
LOCATE 12, 15, 0
PRINT "after they  are acted on.  If you pillage it you can"
LOCATE 13, 15, 0
PRINT "be hit for damage or rewarded with treasure.  If you"
LOCATE 14, 15, 0
PRINT "pray either  nothing may happen or gold will appear."
GOSUB Key.Pressed
RETURN

Help.Screen63:
LOCATE 10, 15, 0
PRINT "The help command displays the help screens.  You can"
LOCATE 11, 15, 0
PRINT "get  help for  every spell by pressing the  function"
LOCATE 12, 15, 0
PRINT "key F1 through F10 with shift key,  for every symbol"
LOCATE 13, 15, 0
PRINT "in  the dungeon,  or meanings  for the cursor keypad"
LOCATE 14, 15, 0
PRINT "functions. Exit help by pressing the spacebar."
GOSUB Key.Pressed
RETURN

Help.Screen64:
LOCATE 10, 15, 0
PRINT "The > symbol is a stair leading up. When you enter"
LOCATE 11, 15, 0
PRINT "a stair up  from level one  your player  exits the"
LOCATE 12, 15, 0
PRINT "dungeon.  Stairs up from  lower levels lead to the"
LOCATE 13, 15, 0
PRINT "next higher dungeon you were in."
GOSUB Key.Pressed
RETURN

Help.Screen65:
LOCATE 10, 15, 0
PRINT "The < symbol is a stair leading down. There is no"
LOCATE 11, 15, 0
PRINT "limit to the dungeon.  Lower levels have more and"
LOCATE 12, 15, 0
PRINT "more difficult monsters with more treasure."
GOSUB Key.Pressed
RETURN

Help.Screen66:
LOCATE 10, 15, 0
PRINT "The trade command allows  your player to trade gold"
LOCATE 11, 15, 0
PRINT "for  statistics.  Your  remaining gold is displayed"
LOCATE 12, 15, 0
PRINT "with the average cost per statistic. You can select"
LOCATE 13, 15, 0
PRINT "the statistic to purchase with the number keys 1-6."
GOSUB Key.Pressed
RETURN

Help.Screen67:
LOCATE 10, 15, 0
PRINT "The level command  lets you change the dungeon level"
LOCATE 11, 15, 0
PRINT "to another level.  Your number of  stored  levels is"
LOCATE 12, 15, 0
PRINT "shown. You can select any level to continue play in."
GOSUB Key.Pressed
RETURN

Help.Screen68:
LOCATE 10, 15, 0
PRINT "The use command uses a charge  from a wand,  potion,"
LOCATE 11, 15, 0
PRINT "staff, or ring.  You must first collect the items in"
LOCATE 12, 15, 0
PRINT "the dungeon. You must enter the alphabetic number of"
LOCATE 13, 15, 0
PRINT "the magic item in your inventory to use them."
GOSUB Key.Pressed
RETURN

Help.Screen69:
LOCATE 10, 15, 0
PRINT "The inventory command displays the wands, potions, staffs"
LOCATE 11, 15, 0
PRINT "and rings you have collected.  There are ten of each item"
LOCATE 12, 15, 0
PRINT "in the dungeon.  They are used with the Use command or by"
LOCATE 13, 15, 0
PRINT "the extended Alt- and Control- letter key commands."
GOSUB Key.Pressed
RETURN

Help.Screen70:
LOCATE 10, 15, 0
PRINT "The | symbol is a wand.  There are ten different wands in"
LOCATE 11, 15, 0
PRINT "the dungeon. When you use a wand with the Use command the"
LOCATE 12, 15, 0
PRINT "charge from  the  wand  will cast a  spell onto  monsters"
LOCATE 13, 15, 0
PRINT "nearby.  When all the charges are used the wand vanishes."
GOSUB Key.Pressed
RETURN

Help.Screen71:
LOCATE 10, 15, 0
PRINT "The @ symbol is a potion. There are ten different potions"
LOCATE 11, 15, 0
PRINT "in the dungeon. When you use potions with the Use command"
LOCATE 12, 15, 0
PRINT "the  charge from  the potion  will  cast a healing spell."
LOCATE 13, 15, 0
PRINT "When the potion is used up it vanishes."
GOSUB Key.Pressed
RETURN

Help.Screen72:
LOCATE 10, 25, 0
PRINT "The Alt-A to Alt-J are the wand keys:"
LOCATE 12, 20, 0
PRINT "Alt-A = magic missile     Alt-F = lightning bolt"
LOCATE 13, 20, 0
PRINT "Alt-B = pyrotechnics      Alt-G = doomkill"
LOCATE 14, 20, 0
PRINT "Alt-C = firestorm         Alt-H = disintegrate"
LOCATE 15, 20, 0
PRINT "Alt-D = fireball          Alt-I = blade barrier"
LOCATE 16, 20, 0
PRINT "Alt-E = finger of death   Alt-J = genocide"
GOSUB Key.Pressed
RETURN

Help.Screen73:
LOCATE 10, 25, 0
PRINT "The Alt-Q to Alt-Z are the potion keys:"
LOCATE 12, 20, 0
PRINT "Alt-Q = heal          Alt-V = power shield"
LOCATE 13, 20, 0
PRINT "Alt-R = protection    Alt-W = wall of will"
LOCATE 14, 20, 0
PRINT "Alt-S = shield        Alt-X = regeneration"
LOCATE 15, 20, 0
PRINT "Alt-T = strength      Alt-Y = phantasmal force"
LOCATE 16, 20, 0
PRINT "Alt-U = restoration   Alt-Z = resurrection"
GOSUB Key.Pressed
RETURN

Help.Screen74:
LOCATE 10, 15, 0
PRINT "The % symbol is a staff.  There's ten different staffs in"
LOCATE 11, 15, 0
PRINT "the dungeon. When you use staffs with the Use command the"
LOCATE 12, 15, 0
PRINT "charge from  the staff will  cast a spell.  When  all the"
LOCATE 13, 15, 0
PRINT "charges are used the staff vanishes."
GOSUB Key.Pressed
RETURN

Help.Screen75:
LOCATE 10, 15, 0
PRINT "The & symbol are some coins. The number of coins you will"
LOCATE 11, 15, 0
PRINT "find  in the dungeon  are according  to the dungeon level"
LOCATE 12, 15, 0
PRINT "you are at. Each coin is worth 100 gold peices."
GOSUB Key.Pressed
RETURN

Help.Screen76:
LOCATE 10, 15, 0
PRINT "The " + CHR$(34) + " symbol is a ring.  There are ten different rings in"
LOCATE 11, 15, 0
PRINT "the dungeon. When you use a ring with the Use command the"
LOCATE 12, 15, 0
PRINT "charge from  the  ring will  cast a spell.  When  all the"
LOCATE 13, 15, 0
PRINT "charges are used the ring vanishes."
GOSUB Key.Pressed
RETURN

Help.Screen77:
LOCATE 10, 20, 0
PRINT "The Control-A to Control-J are the staff keys:"
LOCATE 12, 15, 0
PRINT "Control-A = locate            Control-F = passwall"
LOCATE 13, 15, 0
PRINT "Control-B = transmute         Control-G = fear"
LOCATE 14, 15, 0
PRINT "Control-C = light             Control-H = enchanted weapon"
LOCATE 15, 15, 0
PRINT "Control-D = levitate          Control-I = teleport"
LOCATE 16, 15, 0
PRINT "Control-E = continual light   Control-J = time stop"
GOSUB Key.Pressed
RETURN

Help.Screen78:
LOCATE 10, 20, 0
PRINT "The Control-Q to Control-Z are the ring keys:"
LOCATE 12, 15, 0
PRINT "Control-Q = sleep             Control-V = wall of force"
LOCATE 13, 15, 0
PRINT "Control-R = scare             Control-W = charm monster"
LOCATE 14, 15, 0
PRINT "Control-S = obscurement       Control-X = demon summon"
LOCATE 15, 15, 0
PRINT "Control-T = monster summon    Control-Y = hold monster"
LOCATE 16, 15, 0
PRINT "Control-U = forget            Control-Z = polymorph"
GOSUB Key.Pressed
RETURN

Help.Screen79:
LOCATE 10, 15, 0
PRINT "The Alt-0 to Alt-9 are the Power Globe keys:"
LOCATE 12, 15, 0
PRINT "Alt-0 = clear level       Alt-5 = use all wands"
LOCATE 13, 15, 0
PRINT "Alt-1 = replenish level   Alt-6 = use all potions"
LOCATE 14, 15, 0
PRINT "Alt-2 = down 10 levels    Alt-7 = use all staffs"
LOCATE 15, 15, 0
PRINT "Alt-3 = up 10 levels      Alt-8 = use all rings"
LOCATE 16, 15, 0
PRINT "Alt-4 = collect all       Alt-9 = trade all"
GOSUB Key.Pressed
RETURN

Help.Screen80:
LOCATE 10, 15, 0
PRINT "The  buy  key allows your  player to buy inventory."
LOCATE 11, 15, 0
PRINT "Your  remaining gold  is displayed  along  with the"
LOCATE 12, 15, 0
PRINT "average cost per inventory item. You can select the"
LOCATE 13, 15, 0
PRINT "item to purchase with the number keys 1-6."
GOSUB Key.Pressed
RETURN

Help.Screen81:
LOCATE 10, 15, 0
PRINT "The user list key displays the levels and statistics"
LOCATE 11, 15, 0
PRINT "of all the players in the dungeon game.  The utility"
LOCATE 12, 15, 0
PRINT "program is shelled from the main program."
GOSUB Key.Pressed
RETURN

Help.Screen82:
LOCATE 10, 15, 0
PRINT "The user edit key  allows the author to edit players"
LOCATE 11, 15, 0
PRINT "statistics in the player character file. To use this"
LOCATE 12, 15, 0
PRINT "feature you must have the password."
GOSUB Key.Pressed
RETURN

Help.Screen83:
LOCATE 10, 15, 0
PRINT "The DOS shell key  allows the player to exit to  DOS"
LOCATE 11, 15, 0
PRINT "and perform command line DOS commands.  To return to"
LOCATE 12, 15, 0
PRINT "the Dungeon, enter EXIT and press return."
GOSUB Key.Pressed
RETURN

Help.Screen84:
LOCATE 10, 15, 0
PRINT "The Map Edit  key  allows  the player  to  enter the"
LOCATE 11, 15, 0
PRINT "map editing utility where each level of any player's"
LOCATE 12, 15, 0
PRINT "dungeon level and dungeon symbol can be edited."
GOSUB Key.Pressed
RETURN

Help.Screen85:
LOCATE 10, 15, 0
PRINT "The Power Globe key places the globe symbol  next to"
LOCATE 11, 15, 0
PRINT "the player.  This allows  the most powerful  item in"
LOCATE 12, 15, 0
PRINT "the game to be located without searching for it."
GOSUB Key.Pressed
RETURN

Help.Screen86:
LOCATE 10, 15, 0
PRINT "The F11 displays all symbols in the dungeon level in"
LOCATE 11, 15, 0
PRINT "yellow. This is useful for searching for stairs,  or"
LOCATE 12, 15, 0
PRINT "other symbols the player needs to find. Also try the"
LOCATE 13, 15, 0
PRINT "Shift-F11 to display stairs, Shift-F12 display wands"
LOCATE 14, 15, 0
PRINT "Ctrl-F11 display potion symbols, Ctrl-F12 to display"
LOCATE 15, 15, 0
PRINT "staffs, Alt-F11 displays rings, and Alt-F12 displays"
LOCATE 16, 15, 0
PRINT "chest symbols."
GOSUB Key.Pressed
RETURN

Help.Screen87:
LOCATE 10, 15, 0
PRINT "The F12 key clears the highlighted yellow symbols on"
LOCATE 11, 15, 0
PRINT "the screen displayed from F11. The symbols displayed"
LOCATE 12, 15, 0
PRINT "in the dungeon remain after being cleared."
GOSUB Key.Pressed
RETURN

' routine to display main help screen
Display.Help.Screen:
COLOR 14, 1: CLS
COLOR 10
LOCATE 1, 1, 0
PRINT "While playing,  type 0 for help,  or Q for quit.  The 24th status line displays"
LOCATE 2, 1, 0
PRINT "the following information:  Dlvl  is the  dungeon level,  Lvl is your  player's"
LOCATE 3, 1, 0
PRINT "level,  Hp is your player's hit points,  Exp is your player's experience,  Gold"
LOCATE 4, 1, 0
PRINT "is  the amount  of  gold  hoarded,  and  Nm the  number  of monsters remaining."
COLOR 11
LOCATE 5, 1, 0
PRINT "Function Keys";
COLOR 14
LOCATE 6, 1, 0
PRINT CHR$(201); STRING$(12, CHR$(205)); CHR$(187);
LOCATE 7, 1, 0
PRINT CHR$(186);
COLOR 12
PRINT "FN Key ";
COLOR 10
PRINT "Spell";
COLOR 14
PRINT CHR$(186);
LOCATE 8, 1, 0
PRINT CHR$(186);
COLOR 12
PRINT "Shift  ";
COLOR 10
PRINT "Level";
COLOR 14
PRINT CHR$(186);
LOCATE 9, 1, 0
PRINT CHR$(186); STRING$(12, CHR$(196)); CHR$(186);
LOCATE 10, 1, 0
PRINT CHR$(186);
COLOR 12
PRINT "Normal ";
COLOR 11
PRINT "> ";
COLOR 10
PRINT " 1 ";
COLOR 14
PRINT CHR$(186);
LOCATE 11, 1, 0
PRINT CHR$(186);
COLOR 12
PRINT "Ctrl-  ";
COLOR 11
PRINT "> ";
COLOR 10
PRINT " 2 ";
COLOR 14
PRINT CHR$(186);
LOCATE 12, 1, 0
PRINT CHR$(186);
COLOR 12
PRINT "Shift- ";
COLOR 11
PRINT "> ";
COLOR 10
PRINT " 3 ";
COLOR 14
PRINT CHR$(186);
LOCATE 13, 1, 0
PRINT CHR$(186);
COLOR 12
PRINT "Alt-   ";
COLOR 11
PRINT "> ";
COLOR 10
PRINT " 4 ";
COLOR 14
PRINT CHR$(186);
COLOR 14
LOCATE 14, 1, 0
PRINT CHR$(186); STRING$(12, CHR$(196)); CHR$(186);
LOCATE 15, 1, 0
PRINT CHR$(186);
COLOR 12
PRINT "Key   ";
COLOR 10
PRINT "Spell ";
COLOR 14
PRINT CHR$(186);
LOCATE 16, 1, 0
PRINT CHR$(186);
COLOR 12
PRINT "Shift ";
COLOR 10
PRINT "Number";
COLOR 14
PRINT CHR$(186);
LOCATE 17, 1, 0
PRINT CHR$(186); STRING$(12, CHR$(196)); CHR$(186);
LOCATE 18, 1, 0
PRINT CHR$(186);
COLOR 12
PRINT "Alt-  ";
COLOR 10
PRINT "A ";
COLOR 11
PRINT "-";
COLOR 10
PRINT " J ";
COLOR 14
PRINT CHR$(186);
LOCATE 19, 1, 0
PRINT CHR$(186);
COLOR 12
PRINT "Alt-  ";
COLOR 10
PRINT "Q ";
COLOR 11
PRINT "-";
COLOR 10
PRINT " Z ";
COLOR 14
PRINT CHR$(186);
LOCATE 20, 1, 0
PRINT CHR$(186);
COLOR 12
PRINT "Ctrl- ";
COLOR 10
PRINT "A ";
COLOR 11
PRINT "-";
COLOR 10
PRINT " J ";
COLOR 14
PRINT CHR$(186);
LOCATE 21, 1, 0
PRINT CHR$(186);
COLOR 12
PRINT "Ctrl- ";
COLOR 10
PRINT "Q ";
COLOR 11
PRINT "-";
COLOR 10
PRINT " Z ";
COLOR 14
PRINT CHR$(186);
LOCATE 22, 1, 0
PRINT CHR$(186); STRING$(12, CHR$(196)); CHR$(186);
LOCATE 23, 1, 0
PRINT CHR$(186);
COLOR 12
PRINT "Alt-  ";
COLOR 10
PRINT "0 ";
COLOR 11
PRINT "-";
COLOR 10
PRINT " 9 ";
COLOR 14
PRINT CHR$(186);
LOCATE 24, 1, 0
PRINT CHR$(200); STRING$(12, CHR$(205)); CHR$(188);
LOCATE 5, 16, 0
COLOR 11
PRINT "Numeric Keypad";
LOCATE 6, 16, 0
COLOR 14
PRINT CHR$(201); STRING$(47, CHR$(205)); CHR$(187);
LOCATE 7, 16, 0
PRINT CHR$(186);
COLOR 12
PRINT "7";
COLOR 11
PRINT " : ";
COLOR 10
PRINT "Evade  ";
COLOR 12
PRINT "8";
COLOR 11
PRINT " : ";
COLOR 10
PRINT "North  ";
COLOR 12
PRINT "9";
COLOR 11
PRINT " : ";
COLOR 10
PRINT "Ignore     ";
COLOR 12
PRINT "'";
COLOR 11
PRINT " : ";
COLOR 10
PRINT "Use   ";
COLOR 14
PRINT CHR$(186);
LOCATE 8, 16, 0
PRINT CHR$(186);
COLOR 12
PRINT "4";
COLOR 11
PRINT " : ";
COLOR 10
PRINT "West   ";
COLOR 12
PRINT "5";
COLOR 11
PRINT " : ";
COLOR 10
PRINT "<cr>   ";
COLOR 12
PRINT "6";
COLOR 11
PRINT " : ";
COLOR 10
PRINT "East       ";
COLOR 12
PRINT "-";
COLOR 11
PRINT " : ";
COLOR 10
PRINT "Chants";
COLOR 14
PRINT CHR$(186);
LOCATE 9, 16, 0
PRINT CHR$(186);
COLOR 12
PRINT "1";
COLOR 11
PRINT " : ";
COLOR 10
PRINT "Fight  ";
COLOR 12
PRINT "2";
COLOR 11
PRINT " : ";
COLOR 10
PRINT "South  ";
COLOR 12
PRINT "3";
COLOR 11
PRINT " : ";
COLOR 10
PRINT "Cast       ";
COLOR 12
PRINT "+";
COLOR 11
PRINT " : ";
COLOR 10
PRINT "Info  ";
COLOR 14
PRINT CHR$(186);
LOCATE 10, 16, 0
PRINT CHR$(186);
COLOR 12
PRINT "0";
COLOR 11
PRINT " : ";
COLOR 10
PRINT "Help   ";
COLOR 12
PRINT ".";
COLOR 11
PRINT " : ";
COLOR 10
PRINT "Stats  ";
COLOR 12
PRINT "/";
COLOR 11
PRINT " : ";
COLOR 10
PRINT "Inven      ";
COLOR 12
PRINT ";";
COLOR 11
PRINT " : ";
COLOR 10
PRINT "Buy   ";
COLOR 14
PRINT CHR$(186);
LOCATE 11, 16, 0
PRINT CHR$(186);
COLOR 12
PRINT "\";
COLOR 11
PRINT " : ";
COLOR 10
PRINT "Trade  ";
COLOR 12
PRINT "=";
COLOR 11
PRINT " : ";
COLOR 10
PRINT "Level  ";
COLOR 12
PRINT "`";
COLOR 11
PRINT " : ";
COLOR 10
PRINT "Users      ";
COLOR 12
PRINT "_";
COLOR 11
PRINT " : ";
COLOR 10
PRINT "Edit  ";
COLOR 14
PRINT CHR$(186);
LOCATE 12, 16, 0
PRINT CHR$(200); STRING$(47, CHR$(205)); CHR$(188);
LOCATE 14, 16, 0
COLOR 11
PRINT "Cursor Keypad";
LOCATE 15, 16, 0
COLOR 14
PRINT CHR$(201); STRING$(47, CHR$(205)); CHR$(187);
LOCATE 16, 16, 0
PRINT CHR$(186);
COLOR 12
PRINT "Home";
COLOR 11
PRINT ":";: COLOR 10
PRINT "Evade ";
COLOR 12
PRINT "Up ";
COLOR 11
PRINT ":";
COLOR 10
PRINT "North ";
COLOR 12
PRINT "PgUp";
COLOR 11
PRINT ":";
COLOR 10
PRINT "Ignore ";
COLOR 12
PRINT "CtlPgUp";
COLOR 11
PRINT ":";
COLOR 10
PRINT "Use   ";
COLOR 14
PRINT CHR$(186);
LOCATE 17, 16, 0
PRINT CHR$(186);
COLOR 12
PRINT "Lft ";
COLOR 11
PRINT ":";
COLOR 10
PRINT "West  ";
COLOR 12
PRINT "Rtn";
COLOR 11
PRINT ":";
COLOR 10
PRINT "<cr>  ";
COLOR 12
PRINT "Rgt ";
COLOR 11
PRINT ":";
COLOR 10
PRINT "East   ";
COLOR 12
PRINT "CtlPgDn";
COLOR 11
PRINT ":";
COLOR 10
PRINT "Chants";
COLOR 14
PRINT CHR$(186);
LOCATE 18, 16, 0
PRINT CHR$(186);
COLOR 12
PRINT "End ";
COLOR 11
PRINT ":";
COLOR 10
PRINT "Fight ";
COLOR 12
PRINT "Dn ";
COLOR 11
PRINT ":";
COLOR 10
PRINT "South ";
COLOR 12
PRINT "PgDn";
COLOR 11
PRINT ":";
COLOR 10
PRINT "Cast   ";
COLOR 12
PRINT "CtlRgt ";
COLOR 11
PRINT ":";
COLOR 10
PRINT "Info  ";
COLOR 14
PRINT CHR$(186);
LOCATE 19, 16, 0
PRINT CHR$(186);
COLOR 12
PRINT "Ins ";
COLOR 11
PRINT ":";
COLOR 10
PRINT "Help  ";
COLOR 12
PRINT "Del";
COLOR 11
PRINT ":";
COLOR 10
PRINT "Stats ";
COLOR 12
PRINT "CtlLft";
COLOR 11
PRINT ":";
COLOR 10
PRINT "Inv  ";
COLOR 12
PRINT "ShftTab";
COLOR 11
PRINT ":";
COLOR 10
PRINT "Buy   ";
COLOR 14
PRINT CHR$(186);
LOCATE 20, 16, 0
PRINT CHR$(186);
COLOR 12
PRINT "CtlHm";
COLOR 11
PRINT ":";
COLOR 10
PRINT "Trade ";
COLOR 12
PRINT "CtlEnd";
COLOR 11
PRINT ":";
COLOR 10
PRINT "Level ";
COLOR 12
PRINT "Alt-=";
COLOR 11
PRINT ":";
COLOR 10
PRINT "Users ";
COLOR 12
PRINT "Alt--";
COLOR 11
PRINT ":";
COLOR 10
PRINT "Edit";
COLOR 14
PRINT CHR$(186);
LOCATE 21, 16, 0
PRINT CHR$(200); STRING$(47, CHR$(205)); CHR$(188);
LOCATE 5, 66, 0
COLOR 11
PRINT "Dungeon Runes";
LOCATE 6, 66, 0
COLOR 14
PRINT CHR$(201); STRING$(12, CHR$(205)); CHR$(187);
LOCATE 7, 66, 0
PRINT CHR$(186);
COLOR 12
PRINT "X";
COLOR 11
PRINT "=";
COLOR 10
PRINT "Player    ";
COLOR 14
PRINT CHR$(186);
LOCATE 8, 66, 0
PRINT CHR$(186);
COLOR 12
PRINT "#";
COLOR 11
PRINT "=";
COLOR 10
PRINT "Monster   ";
COLOR 14
PRINT CHR$(186);
LOCATE 9, 66, 0
PRINT CHR$(186);
COLOR 12
PRINT "$";
COLOR 11
PRINT "=";
COLOR 10
PRINT "Gold      ";
COLOR 14
PRINT CHR$(186);
LOCATE 10, 66, 0
PRINT CHR$(186);
COLOR 12
PRINT "~";
COLOR 11
PRINT "=";
COLOR 10
PRINT "Scroll    ";
COLOR 14
PRINT CHR$(186);
LOCATE 11, 66, 0
PRINT CHR$(186);
COLOR 12
PRINT "^";
COLOR 11
PRINT "=";
COLOR 10
PRINT "Trap      ";
COLOR 14
PRINT CHR$(186);
LOCATE 12, 66, 0
PRINT CHR$(186);
COLOR 12
PRINT "o";
COLOR 11
PRINT "=";
COLOR 10
PRINT "Pit       ";
COLOR 14
PRINT CHR$(186);
LOCATE 13, 66, 0
PRINT CHR$(186);
COLOR 12
PRINT "*";
COLOR 11
PRINT "=";
COLOR 10
PRINT "Gem       ";
COLOR 14
PRINT CHR$(186);
LOCATE 14, 66, 0
PRINT CHR$(186);
COLOR 12
PRINT "!";
COLOR 11
PRINT "=";
COLOR 10
PRINT "Teleporter";
COLOR 14
PRINT CHR$(186);
LOCATE 15, 66, 0
PRINT CHR$(186);
COLOR 12
PRINT "?";
COLOR 11
PRINT "=";
COLOR 10
PRINT "Temple    ";
COLOR 14
PRINT CHR$(186);
LOCATE 16, 66, 0
PRINT CHR$(186);
COLOR 12
PRINT CHR$(127);
COLOR 11
PRINT "=";
COLOR 10
PRINT "Chest     ";
COLOR 14
PRINT CHR$(186);
LOCATE 17, 66, 0
PRINT CHR$(186);
COLOR 12
PRINT ">";
COLOR 11
PRINT "=";
COLOR 10
PRINT "Stair Up  ";
COLOR 14
PRINT CHR$(186);
LOCATE 18, 66, 0
PRINT CHR$(186);
COLOR 12
PRINT "<";
COLOR 11
PRINT "=";
COLOR 10
PRINT "Stair Down";
COLOR 14
PRINT CHR$(186);
LOCATE 19, 66, 0
PRINT CHR$(186);
COLOR 12
PRINT "|";
COLOR 11
PRINT "=";
COLOR 10
PRINT "Wand      ";
COLOR 14
PRINT CHR$(186);
LOCATE 20, 66, 0
PRINT CHR$(186);
COLOR 12
PRINT "@";
COLOR 11
PRINT "=";
COLOR 10
PRINT "Potion    ";
COLOR 14
PRINT CHR$(186);
LOCATE 21, 66, 0
PRINT CHR$(186);
COLOR 12
PRINT "%";
COLOR 11
PRINT "=";
COLOR 10
PRINT "Staff     ";
COLOR 14
PRINT CHR$(186);
LOCATE 22, 66, 0
PRINT CHR$(186);
COLOR 12
PRINT "&";
COLOR 11
PRINT "=";
COLOR 10
PRINT "Coins     ";
COLOR 14
PRINT CHR$(186);
LOCATE 23, 66, 0
PRINT CHR$(186);
COLOR 12
PRINT CHR$(34);
COLOR 11
PRINT "=";
COLOR 10
PRINT "Ring      ";
COLOR 14
PRINT CHR$(186);
LOCATE 24, 66, 0
PRINT CHR$(200); STRING$(12, CHR$(205)); CHR$(188);
LOCATE 22, 16, 0
COLOR 12
PRINT "Ctrl-Delete";
COLOR 11
PRINT " = ";
COLOR 10
PRINT "DOS Shell    ";
COLOR 12
PRINT "Alt-Up";
COLOR 11
PRINT " = ";
COLOR 10
PRINT "Map Editor   ";
LOCATE 23, 16, 0
COLOR 12
PRINT "Alt-Left";
COLOR 11
PRINT " = ";
COLOR 10
PRINT "Power Globe  ";
COLOR 12
PRINT "F11/F12";
COLOR 11
PRINT " = ";
COLOR 10
PRINT "Dungeon Symbols";
COLOR 13
LOCATE 24, 27, 0
PRINT "Press a listed key for Help";
LOCATE 25, 21, 0
COLOR 11
PRINT "Press <space> to Return to the dungeon";
RETURN

' routine to display message then wait for keystroke
Key.Pressed:
LOCATE 25, 34, 0
PRINT "Press a key..";
Z1 = 0 ' reset keystroke first byte
Z2 = 0 ' reset keystroke second byte
DO ' loop until key pressed
    x$ = "" ' reset buffer
    WHILE x$ = "" ' loop until buffer full
        _LIMIT 100
        x$ = INKEY$ ' store keystroke
    WEND ' end buffer loop
    IF LEN(x$) = 2 THEN ' determine key type
        Z1 = ASC(LEFT$(x$, 1)) ' store first byte
        Z2 = ASC(RIGHT$(x$, 1)) ' store second byte
    ELSE ' compare key type
        Z1 = 0 ' store null byte
        Z2 = ASC(x$) ' store single byte
    END IF ' end byte compare
    ' check if control-break returned two null bytes
    IF Z1 > 0 OR Z2 > 0 THEN ' compare byte values
        EXIT DO ' exit from key pressed
    END IF ' end compare bytes
    Z1 = 0 ' reset byte value
    Z2 = 0 ' reset byte value
LOOP ' end key press loop
RETURN ' exit from routine
END SUB

REM The Dungeon Player List Utility v12.0 r3.0 Program Source
SUB User.List.Menu
CLOSE #3
OPEN Util.Filename FOR OUTPUT SHARED AS #3
GOSUB Header1 ' display header one
Page.Length = 5 ' reset page length
FOR User.Number = 1 TO LOF(1) / LEN(UserRecord) ' loop through all users in file
    Page.Length = Page.Length + 1 ' increment page length
    IF Page.Length = 24 THEN ' check page length
        Page.Length = 6 ' reset page length
        GOSUB Press.Key ' get a keypress
        GOSUB Header1 ' redisplay header one
    END IF ' end check page length
    GET 1, User.Number, UserRecord ' read next user record
    User.Name$ = fndecrypt$(UserRecord.CodeName) ' store decypted user name
    MID$(User.Name$, 1, 1) = UCASE$(MID$(User.Name$, 1, 1)) ' uppercase name
    Dungeon.Level$ = STR$(GetUserRecordStats1(1)) ' store dungeon levels
    Dungeon.Level$ = Dungeon.Level$ + SPACE$(14 - LEN(Dungeon.Level$)) ' pad spaces
    Player.Level$ = STR$(GetUserRecordStats1(2)) ' store player level
    Player.Level$ = Player.Level$ + SPACE$(14 - LEN(Player.Level$)) ' pad spaces
    Display.Line$ = User.Name$ + Dungeon.Level$ + " " + Player.Level$ ' append
    Wands = 0 ' reset wand counter
    FOR Count = 1 TO 10 ' count number of wands
        IF GetUserRecordWand(Count) > 0 THEN ' check wands
            Wands = Wands + 1 ' increment wands
        END IF ' end check wands
    NEXT ' end count wands
    Display.Field$ = STR$(Wands) ' store number of wands
    Display.Field$ = Display.Field$ + SPACE$(7 - LEN(Display.Field$)) ' pad spaces
    Display.Line$ = Display.Line$ + Display.Field$ ' append to display line
    Potions = 0 ' reset potion counter
    FOR Count = 1 TO 10 ' count number of potions
        IF GetUserRecordPotion(Count) > 0 THEN ' check potions
            Potions = Potions + 1 ' increment potions
        END IF ' end check potions
    NEXT ' end count potions
    Display.Field$ = STR$(Potions) ' store number ofp otions
    Display.Field$ = Display.Field$ + SPACE$(9 - LEN(Display.Field$)) ' pad spaces
    Display.Line$ = Display.Line$ + Display.Field$ ' append to display line
    Staffs = 0 ' reset staff counter
    FOR Count = 1 TO 10 ' count number of staffs
        IF GetUserRecordStaff(Count) > 0 THEN ' check staffs
            Staffs = Staffs + 1 ' increment staffs
        END IF ' end check staffs
    NEXT ' end counter staffs
    Display.Field$ = STR$(Staffs) ' store number of staffs
    Display.Field$ = Display.Field$ + SPACE$(8 - LEN(Display.Field$)) ' pad spaces
    Display.Line$ = Display.Line$ + Display.Field$ ' append to display line
    Rings = 0 ' reset ring counter
    FOR Count = 1 TO 10 ' count number of rings
        IF GetUserRecordRing(Count) > 0 THEN ' check rings
            Rings = Rings + 1 ' increment rings
        END IF ' end check rings
    NEXT ' end count rings
    Display.Field$ = STR$(Rings) ' store number of rings
    Display.Line$ = Display.Line$ + Display.Field$ ' append to display line
    COLOR 13 ' color magenta
    IF Futil = 0 THEN
        PRINT Display.Line$ ' display report line on screen
    END IF
    PRINT #3, Display.Line$ ' print display line to file
NEXT ' end loop through user file
PRINT #3, "" ' print linefeed to file
GOSUB Press.Key ' get a keypress
GOSUB Header2 ' display header two
Page.Length = 5 ' reste page length
FOR User.Number = 1 TO LOF(1) / LEN(UserRecord) ' loop through all users in file
    Page.Length = Page.Length + 1 ' increment page length
    IF Page.Length = 24 THEN ' check page length
        Page.Length = 6 ' reset page length
        GOSUB Press.Key ' get a keypress
        GOSUB Header2 ' redisplay header two
    END IF ' end check page length
    GET 1, User.Number, UserRecord ' read next user record
    User.Name$ = fndecrypt$(UserRecord.CodeName) ' store decrypted user name
    MID$(User.Name$, 1, 1) = UCASE$(MID$(User.Name$, 1, 1)) ' uppercase name
    Strength$ = STR$(GetUserRecordStats2(1)) ' store strength statistic
    Strength$ = Strength$ + SPACE$(9 - LEN(Strength$)) ' pad spaces
    Wisdom$ = STR$(GetUserRecordStats2(2)) ' store wisdom statistic
    Wisdom$ = Wisdom$ + SPACE$(7 - LEN(Wisdom$)) ' pad spaces
    Intellect$ = STR$(GetUserRecordStats2(3)) ' store intellect statistic
    Intellect$ = Intellect$ + SPACE$(10 - LEN(Intellect$)) ' pad spaces
    Constitution$ = STR$(GetUserRecordStats2(4)) ' store consitution statistic
    Constitution$ = Constitution$ + SPACE$(13 - LEN(Constitution$)) ' pad spaces
    Dexterity$ = STR$(GetUserRecordStats2(5)) ' store dexterity statistic
    Dexterity$ = Dexterity$ + SPACE$(10 - LEN(Dexterity$)) ' pad spaces
    Charisma$ = STR$(GetUserRecordStats2(6)) ' store charisma statistic
    COLOR 13 ' color magenta
    ' append all statistics to display line
    Display.Line$ = User.Name$ ' store user name
    Display.Line$ = Display.Line$ + Strength$ ' append strength
    Display.Line$ = Display.Line$ + Wisdom$ ' append wisdom
    Display.Line$ = Display.Line$ + Intellect$ ' append intellect
    Display.Line$ = Display.Line$ + Constitution$ ' append constitution
    Display.Line$ = Display.Line$ + Dexterity$ ' sppend dexterity
    Display.Line$ = Display.Line$ + Charisma$ ' append charisma
    IF Futil = 0 THEN
        PRINT Display.Line$ ' display report line on screen
    END IF
    PRINT #3, Display.Line$ ' print display line to file
NEXT ' end loop through user file
PRINT #3, "" ' print linefeed to file
GOSUB Press.Key ' get a keypress
GOSUB Header3 ' display header three
Page.Length = 5 ' reset page length
FOR User.Number = 1 TO LOF(1) / LEN(UserRecord) ' loop through all users in file
    Page.Length = Page.Length + 1 ' increment page length
    IF Page.Length = 24 THEN ' check page length
        Page.Length = 6 ' reset page length
        GOSUB Press.Key ' get a keypress
        GOSUB Header3 ' redisplay header three
    END IF ' end check page length
    GET 1, User.Number, UserRecord ' read next user record
    User.Name$ = fndecrypt$(UserRecord.CodeName) ' store decrypted user name
    MID$(User.Name$, 1, 1) = UCASE$(MID$(User.Name$, 1, 1)) ' uppercase name
    Experience$ = STR$(GetUserRecordStats1(3)) ' store experience
    Experience$ = Experience$ + SPACE$(22 - LEN(Experience$)) ' pad spaces
    Gold$ = STR$(GetUserRecordStats1(5)) ' store gold
    Gold$ = Gold$ + SPACE$(22 - LEN(Gold$)) ' pad spaces
    Hit.Points$ = STR$(GetUserRecordStats1(6)) ' store hit points
    COLOR 13 ' color magenta
    ' append all statistics to display line
    Display.Line$ = User.Name$ ' store user name
    Display.Line$ = Display.Line$ + Experience$ ' append experience
    Display.Line$ = Display.Line$ + Gold$ ' append gold
    Display.Line$ = Display.Line$ + Hit.Points$ ' append hit points
    IF Futil = 0 THEN
        PRINT Display.Line$ ' display report line on screen
    END IF
    PRINT #3, Display.Line$ ' print report line to file
NEXT ' end loop through user file
PRINT #3, "" ' print linefeed to file
GOSUB Press.Key ' get a keypress
GOSUB Header4 ' display header four
Page.Length = 5 ' reset page length
FOR User.Number = 1 TO LOF(1) / LEN(UserRecord) ' loop through all users in file
    Page.Length = Page.Length + 1 ' increment page length
    IF Page.Length = 24 THEN ' check page length
        Page.Length = 6 ' reset page length
        GOSUB Press.Key ' get a keypress
        GOSUB Header4 ' redisplay header four
    END IF ' end check page length
    GET 1, User.Number, UserRecord ' read next user record
    User.Name$ = fndecrypt$(UserRecord.CodeName) ' store decrypted user name
    MID$(User.Name$, 1, 1) = UCASE$(MID$(User.Name$, 1, 1)) ' uppercase name
    Display.Line$ = User.Name$ ' store user name
    FOR Count = 7 TO 12 ' loop through remaining statistics
        Display.Field$ = STR$(GetUserRecordStats2(Count)) ' store statistic
        Display.Field$ = Display.Field$ + SPACE$(7 - LEN(Display.Field$)) ' pad spaces
        Display.Line$ = Display.Line$ + Display.Field$ ' append to display line
    NEXT ' end loop through remaining statistics
    COLOR 13 ' color magenta
    IF Futil = 0 THEN
        PRINT Display.Line$ ' display report line on screen
    END IF
    PRINT #3, Display.Line$ ' print report line to file
NEXT ' end loop through user file
PRINT #3, "" ' print linefeed to file
GOSUB Press.Key ' get a keypress
COLOR 7, 0 ' color white on black
CLS ' clear screen
EXIT SUB ' quit program

' routine to get a keypress
Press.Key:
IF Futil THEN
    RETURN
END IF
LOCATE 25, 1, 0 ' set cursor to bottom of screen
COLOR 10 ' color cyan
PRINT "Press a key to continue:"; ' display message prompt
WHILE INKEY$ = "" ' loop until keypress
    _LIMIT 100
WEND ' end keypress loop
RETURN ' exit routine

' routine to display header one to screen and file
Header1:
GOSUB Page.Header ' display main header
COLOR 14 ' color yellow
' make header one
Header$ = "Player Name          Dungeon Level  Player Level  Wands  Potions  Staffs  Rings"
IF Futil = 0 THEN
    PRINT Header$ ' display header
END IF
PRINT #3, Header$ ' print header to file
Header$ = STRING$(79, "-") ' make header underline
IF Futil = 0 THEN
    PRINT Header$ ' display header underline
END IF
PRINT #3, Header$ ' print header underline to file
RETURN ' exit routine

' routine to display header two to screen and file
Header2:
GOSUB Page.Header ' display main header
COLOR 14 ' color yellow
' make header two
Header$ = "Player Name          Strength Wisdom Intellect Constitution Dexterity Charisma"
IF Futil = 0 THEN
    PRINT Header$ ' display header
END IF
PRINT #3, Header$ ' print header to file
Header$ = STRING$(78, "-") ' make header underline
IF Futil = 0 THEN
    PRINT Header$ ' display header underline
END IF
PRINT #3, Header$ ' print header underline to file
RETURN ' exit routine

' routine to display header three to screen and file
Header3:
GOSUB Page.Header ' display main header
COLOR 14 ' color yellow
' make header three
Header$ = "Player Name          Experience            Gold                  Hit Points"
IF Futil = 0 THEN
    PRINT Header$ ' display header
END IF
PRINT #3, Header$ ' print header to file
Header$ = STRING$(74, "-") ' make header underline
IF Futil = 0 THEN
    PRINT Header$ ' display header underline
END IF
PRINT #3, Header$ ' print header underline to file
RETURN ' exit routine

' routine to display header four to screen and file
Header4:
GOSUB Page.Header ' display main header
COLOR 14 ' color yellow
' make header four
Header$ = "Player Name          Weapon Shield Armor  Cloak  Helmet Boots"
IF Futil = 0 THEN
    PRINT Header$ ' display header
END IF
PRINT #3, Header$ ' print header to file
Header$ = STRING$(61, "-") ' make header underline
IF Futil = 0 THEN
    PRINT Header$ ' display header underline
END IF
PRINT #3, Header$ ' print header underline to file
RETURN ' exit routine

' display the main page header
Page.Header:
IF Futil THEN
    RETURN
END IF
COLOR 14, 1 ' color yellow on blue
CLS ' clear screen
COLOR 15 ' color white
PRINT "The Dungeon Player List v12.0 r3.0 Utility" ' display header
PRINT STRING$(42, "=") ' display header underline
PRINT ' linefeed
RETURN ' exit routine
END SUB ' quit program

REM The Dungeon Player Edit Utility v12.0 r3.0 Program Source
SUB Edit.User.Menu

' main program
User.Number = 1 ' reset user number being edited
DO ' loop through all users in data file
    GOSUB Page.Header ' display edit page header
    COLOR 14 ' color yellow
    GET 1, User.Number, UserRecord ' read the next user record from data file
    User.Name$ = fndecrypt$(UserRecord.CodeName) ' read encrypted user name
    MID$(User.Name$, 1, 1) = UCASE$(MID$(User.Name$, 1, 1)) ' uppercase name
    User.Name$ = RTRIM$(User.Name$) ' trim right side of user name
    PRINT User.Name$; ": Edit(Y/N/Q/#)? "; ' prompt to edit user name
    LINE INPUT Input.Line$ ' get keyboard input
    New.User = INT(VAL(Input.Line$)) ' convert string to value
    ' check if value is in user file bounds
    IF New.User >= 1 AND New.User <= LOF(1) / LEN(UserRecord) THEN
        User.Number = New.User ' reset user number
        GOSUB Edit.User.Items ' edit user
    ELSE ' check value
        SELECT CASE UCASE$(Input.Line$) ' determine input key pressed
            CASE "N" ' input no
                User.Number = User.Number + 1 ' increment to next user
            CASE "Y" ' input yes
                GOSUB Edit.User.Items ' edit user
                User.Number = User.Number + 1 ' increment to next user
            CASE "Q" ' input quit
                EXIT DO ' exit editing loop
        END SELECT ' end input key determine
    END IF
    ' check user number to end of user data file
LOOP WHILE User.Number <= LOF(1) / LEN(UserRecord)
GOSUB Press.Key ' get a key press
COLOR 7, 0 ' white on black
CLS ' clear screen
EXIT SUB ' quit program

' define routine to prompt for key press
Press.Key:
LOCATE 25, 1, 0 ' locate cursor to bottom of screen
COLOR 10 ' color magenta
PRINT "Press a key to continue:"; ' display prompt message
WHILE INKEY$ = "" ' wait for key press
    _LIMIT 100
WEND ' end wait for key press
RETURN ' exit routine

' routine to input edit values for user
Edit.User.Items:
COLOR 11 ' color cyan
PRINT "Strength(1-99)"; ' display input prompt
INPUT Input.Value% ' get numeric input value
IF INT(Input.Value%) > 0 AND INT(Input.Value%) < 100 THEN ' validate numeric range
    CALL PutUserRecordStats2(1, INT(Input.Value%)) ' store valid numeric entry
END IF ' end validate numeric range
PRINT "Wisdom(1-99)"; ' display input prompt
INPUT Input.Value% ' get numeric input value
IF INT(Input.Value%) > 0 AND INT(Input.Value%) < 100 THEN ' validate numeric range
    CALL PutUserRecordStats2(2, INT(Input.Value%)) ' store valid numeric entry
END IF ' end validate numeric range
PRINT "Intellect(1-99)"; ' display input prompt
INPUT Input.Value% ' get numeric input value
IF INT(Input.Value%) > 0 AND INT(Input.Value%) < 100 THEN ' validate numeric range
    CALL PutUserRecordStats2(3, INT(Input.Value%)) ' store valid numeric entry
END IF ' end validate numeric range
PRINT "Constitution(1-99)"; ' display input prompt
INPUT Input.Value% ' get numeric input value
IF INT(Input.Value%) > 0 AND INT(Input.Value%) < 100 THEN ' validate numeric range
    CALL PutUserRecordStats2(4, INT(Input.Value%)) ' store valid numeric entry
END IF ' end validate numeric range
PRINT "Dexterity(1-99)"; ' display input prompt
INPUT Input.Value% ' get numeric input value
IF INT(Input.Value%) > 0 AND INT(Input.Value%) < 100 THEN ' validate numeric range
    CALL PutUserRecordStats2(5, INT(Input.Value%)) ' store valid numeric entry
END IF ' end validate numeric range
PRINT "Charisma(1-99)"; ' display input prompt
INPUT Input.Value% ' get numeric input value
IF INT(Input.Value%) > 0 AND INT(Input.Value%) < 100 THEN ' validate numeric range
    CALL PutUserRecordStats2(6, INT(Input.Value%)) ' store valid numeric entry
END IF ' end validate numeric range
PRINT "Weapon(1-99)"; ' display input prompt
INPUT Input.Value% ' get numeric input value
IF INT(Input.Value%) > 0 AND INT(Input.Value%) < 100 THEN ' validate numeric range
    CALL PutUserRecordStats2(7, INT(Input.Value%)) ' store valid numeric entry
END IF ' end validate numeric range
PRINT "Shield(1-99)"; ' display input prompt
INPUT Input.Value% ' get numeric input value
IF INT(Input.Value%) > 0 AND INT(Input.Value%) < 100 THEN ' validate numeric range
    CALL PutUserRecordStats2(8, INT(Input.Value%)) ' store valid numeric entry
END IF ' end validate numeric range
PRINT "Armor(1-99)"; ' display input prompt
INPUT Input.Value% ' get numeric input value
IF INT(Input.Value%) > 0 AND INT(Input.Value%) < 100 THEN ' validate numeric range
    CALL PutUserRecordStats2(9, INT(Input.Value%)) ' store valid numeric entry
END IF ' end validate numeric range
PRINT "Cloak(1-99)"; ' display input prompt
INPUT Input.Value% ' get numeric input value
IF INT(Input.Value%) > 0 AND INT(Input.Value%) < 100 THEN ' validate numeric range
    CALL PutUserRecordStats2(10, INT(Input.Value%)) ' store valid numeric entry
END IF ' end validate numeric range
PRINT "Helmet(1-99)"; ' display input prompt
INPUT Input.Value% ' get numeric input value
IF INT(Input.Value%) > 0 AND INT(Input.Value%) < 100 THEN ' validate numeric range
    CALL PutUserRecordStats2(11, INT(Input.Value%)) ' store valid input entry
END IF ' end validate numeric range
PRINT "Boots(1-99)"; ' display input prompt
INPUT Input.Value% ' get numeric input value
IF INT(Input.Value%) > 0 AND INT(Input.Value%) < 100 THEN ' validate numeric range
    CALL PutUserRecordStats2(12, INT(Input.Value%)) ' store valid input entry
END IF ' end validate numeric range
PRINT "Experience"; ' display input prompt
INPUT Input.Value# ' get numeric input value
CALL PutUserRecordStats1(3, CDBL(INT(Input.Value#))) ' store double precision input entry
PRINT "Gold"; ' display input prompt
INPUT Input.Value# ' get numeric input value
CALL PutUserRecordStats1(5, CDBL(INT(Input.Value#))) ' store double precision input entry
PUT 1, User.Number, UserRecord ' write user number record to data file
RETURN ' exit routine

' display the edit page header on screen
Page.Header:
COLOR 14, 1 ' yellow on blue
CLS ' clear screen
COLOR 15 ' white foreground
PRINT "The Dungeon Player Edit v12.0 r3.0 Utility" ' display header
PRINT STRING$(42, "=") ' display header underline
RETURN ' exit routine
END SUB

REM The Dungeon Map Edit Utility v12.0 r3.0 Program Source
SUB Map.Editor

' main routine loop
User.Number = 1 ' reset the user number being edited
DO ' loop through the user data file
    GET 1, User.Number, UserRecord ' read the next user record
    User.Name$ = fndecrypt$(UserRecord.CodeName) ' get the decrypted user name
    MID$(User.Name$, 1, 1) = UCASE$(MID$(User.Name$, 1, 1)) ' uppercase name
    User.Name$ = RTRIM$(User.Name$) ' trim blanks from the right
    COLOR 14, 1 ' color yellow on blue
    CLS ' clear screen
    PRINT User.Name$; ": Edit(Y/N/Q/#)? "; ' display input prompt
    LINE INPUT Input.Line$ ' get keyboard input
    New.User = INT(VAL(Input.Line$)) ' convert string to value
    ' check if value is in user file bounds
    IF New.User >= 1 AND New.User <= LOF(1) / LEN(UserRecord) THEN
        User.Number = New.User ' reset user number
        GOSUB Edit.Map ' edit the current user number
    ELSE ' check value
        SELECT CASE UCASE$(Input.Line$) ' determine input character
            CASE "N" ' input no
                User.Number = User.Number + 1 ' increment to the next user record
            CASE "Y" ' input yes
                GOSUB Edit.Map ' edit the current user number
                User.Number = User.Number + 1 ' increment to the next user record
            CASE "Q" ' input quit
                EXIT DO ' exit the edit loop
        END SELECT ' end determine input character
    END IF
    ' loop until the user index number exceeds the data file length
LOOP WHILE User.Number <= LOF(1) / LEN(UserRecord)
COLOR 7, 0 ' color white on black
CLS ' clear screen
EXIT SUB ' quit program

' routine to edit the user dungeon maps
Edit.Map:
GOSUB OpenDataFile ' open the user dungeon map data file
Dungeon.Level# = 1# ' reset the dungeon level being edited
' loop through all dungeon map levels
' until the dungeon level exceeds the data file length
DO WHILE Dungeon.Level# <= LOF(2) / LEN(DungeonRecord)
    COLOR 14, 1 ' color yellow on blue
    CLS ' clear screen
    PRINT "Edit Dungeon Level"; Dungeon.Level#; "(Y/N/Q/#)? "; ' make input prompt
    LINE INPUT Input.Line$ ' get a key board character
    New.Level# = INT(VAL(Input.Line$)) ' convert string to value
    ' check if value is in dungeon file bounds
    IF New.Level# >= 1 AND New.Level# <= LOF(2) / LEN(DungeonRecord) THEN
        Dungeon.Level# = New.Level# ' reset level
        GOSUB Edit.Level ' edit the dungeon level
        Dungeon.Level# = Dungeon.Level# + 1# ' increment level
    ELSE ' check value
        SELECT CASE UCASE$(Input.Line$) ' determine input character
            CASE "N" ' input no
                Dungeon.Level# = Dungeon.Level# + 1# ' increment dungeon level
            CASE "Y" ' input yes
                GOSUB Edit.Level ' edit the current dungeon level
                Dungeon.Level# = Dungeon.Level# + 1# ' increment dungeon level
            CASE "Q" ' input quit
                EXIT DO ' exit dungeon map edit loop
        END SELECT ' end determine input character
    END IF ' end check value
LOOP ' end dungeon map level edit loop
RETURN ' exit routine

' routine to edit a dungeon level
Edit.Level:
GOSUB Display.Level ' display the dungeon level map
Status.Line = -1 ' reset status line toggle
GOSUB Display.Status.Line ' display the status line
DO ' loop while the map is edited
    Input.Char$ = "" ' reset input string
    WHILE Input.Char$ = "" ' loop until key entered
        _LIMIT 100
        Input.Char$ = INKEY$ ' read the keyboard
    WEND ' end key input loop
    SELECT CASE LEN(Input.Char$) ' determine input scan code
        CASE 1 ' input length is single key scan code
            SELECT CASE ASC(UCASE$(Input.Char$)) ' determine single keypress
                CASE 9 ' tab
                    ' try to tab as far right until column 80
                    IF Dungeon.Y + 8 < 80 THEN ' next tab stop is within screen edge
                        Dungeon.Y = Dungeon.Y + 8 ' increment the map Y cursor value
                        COLOR 15, 1 ' color white on blue
                        LOCATE Dungeon.X, Dungeon.Y, 1 ' set cursor to new position
                    ELSE ' next tab stop is beyond screen
                        IF Dungeon.Y < 79 THEN ' check rightmost column
                            Dungeon.Y = 79 ' set map Y cursor value
                            COLOR 15, 1 ' color white on blue
                            LOCATE Dungeon.X, Dungeon.Y, 1 ' set cursor to new position
                        END IF ' end check rightmost column
                    END IF ' end check next tab stop
                CASE 27 ' escape
                    PUT 2, Dungeon.Level#, DungeonRecord ' write current dungeon level
                    EXIT DO ' exit dungeon level edit loop
                CASE ASC("-") ' minus
                    ' toggle the status line
                    IF Status.Line = 0 THEN ' check current status toggle
                        Status.Line = -1 ' toggle status line
                        GOSUB Display.Status.Line ' display status line
                    END IF ' end check current status toggle
                CASE ASC("+") ' plus
                    ' toggle the status line
                    IF Status.Line THEN ' check current status toggle
                        Status.Line = 0 ' toggle status line
                        GOSUB Display.Status.Line ' display status line
                    END IF ' end check current status toggle
                CASE 48 TO 57, 65 TO 70, 79, 88 ' character is a dungeon cell symbol
                    GOSUB Change.Cell ' routine to change that cell
            END SELECT ' end determine key scan type
        CASE 2 ' key scan type is double byte
            Key.Scan = ASC(RIGHT$(Input.Char$, 1)) ' store second key scan byte
            SELECT CASE Key.Scan ' determine key scan byte value
                CASE 15 ' shift-tab
                    ' try to tab left as far as column 1
                    IF Dungeon.Y - 8 > 0 THEN ' next left tab stop is within screen edge
                        Dungeon.Y = Dungeon.Y - 8 ' decrement the map Y cursor value
                        COLOR 15, 1 ' color white on blue
                        LOCATE Dungeon.X, Dungeon.Y, 1 ' set the cursor to new position
                    ELSE ' next left tab stop is beyond screen
                        IF Dungeon.Y > 1 THEN ' check leftmost column
                            Dungeon.Y = 1 ' set map Y cursor value
                            COLOR 15, 1 ' color white on blue
                            LOCATE Dungeon.X, Dungeon.Y, 1 ' set the cursor to new position
                        END IF ' end check leftmost column
                    END IF ' end check left tab stop
                CASE 79 ' end
                    ' move cursor to end of screen
                    IF Dungeon.Y <> 79 THEN ' check is cursor already at screen edge
                        Dungeon.Y = 79 ' set the map Y cursor value
                        COLOR 15, 1 ' color white on blue
                        LOCATE Dungeon.X, Dungeon.Y, 1 ' set the cursor to new psotion
                    END IF ' end check cursor position
                CASE 80 ' down
                    ' move cursor down one row
                    IF Dungeon.X < 23 THEN ' check is cursor already at screen bottom
                        Dungeon.X = Dungeon.X + 1 ' set the map X cursor value
                        COLOR 15, 1 ' color white on blue
                        LOCATE Dungeon.X, Dungeon.Y, 1 ' set the cursor to new position
                    END IF ' end check cursor position
                CASE 81 ' page down
                    ' move the cursor to bottom of screen
                    IF Dungeon.X <> 23 THEN ' check is cursor already at screen bottom
                        Dungeon.X = 23 ' set the map X cursor value
                        COLOR 15, 1 ' color white on blue
                        LOCATE Dungeon.X, Dungeon.Y, 1 ' set the cursor to new position
                    END IF ' end check cursor position
                CASE 75 ' left
                    ' move the cursor left one column
                    IF Dungeon.Y > 1 THEN ' check is cursor already at screen edge
                        Dungeon.Y = Dungeon.Y - 1 ' set the map Y cursor value
                        COLOR 15, 1 ' color white on blue
                        LOCATE Dungeon.X, Dungeon.Y, 1 ' set the cursor to new position
                    END IF ' end check cursor position
                CASE 77 ' right
                    ' move the cursor right one column
                    IF Dungeon.Y < 79 THEN ' check is cursor already at screen edge
                        Dungeon.Y = Dungeon.Y + 1 ' set the map Y cursor value
                        COLOR 15, 1 ' color white on blue
                        LOCATE Dungeon.X, Dungeon.Y, 1 ' set the cursor to new position
                    END IF ' end check cursor position
                CASE 71 ' home
                    ' move the cursor to the left side of screen
                    IF Dungeon.Y <> 1 THEN ' check is cursor already at left side
                        Dungeon.Y = 1 ' set the map Y cursor value
                        COLOR 15, 1 ' color white on blue
                        LOCATE Dungeon.X, Dungeon.Y, 1 ' set the cursor to new position
                    END IF ' end check cursor position
                CASE 72 ' up
                    ' move cursor up one row
                    IF Dungeon.X > 1 THEN ' check is cursor already at top edge
                        Dungeon.X = Dungeon.X - 1 ' set the map X cursor value
                        COLOR 15, 1 ' color white on blue
                        LOCATE Dungeon.X, Dungeon.Y, 1 ' set the cursor to new position
                    END IF ' end check cursor position
                CASE 73 ' page up
                    ' move the cursor to top edge
                    IF Dungeon.X <> 1 THEN ' check is cursor already at top edge
                        Dungeon.X = 1 ' set the map X cursor value
                        COLOR 15, 1 ' color white on blue
                        LOCATE Dungeon.X, Dungeon.Y, 1 ' set the cursor to new position
                    END IF ' end check cursor position
            END SELECT ' end determine key scan code
    END SELECT ' end determine key scan type
LOOP ' end dungeon map edit loop
RETURN ' exit routine

' routine displays the current dungeon level map
Display.Level:
CLS ' clear screen
GET 2, Dungeon.Level#, DungeonRecord ' read the dungeon level data record
' loop through all dungeon record level cells
FOR Dungeon.Row = 1 TO 23 ' rows by
    FOR Dungeon.Column = 1 TO 79 ' columns
        LOCATE Dungeon.Row, Dungeon.Column, 0 ' locate at dungeon xy
        ' get ascii value of dungeon coordinate
        Dungeon.Cell = GetDungeonRecordLevelData(Dungeon.Row, Dungeon.Column)
        ' display character in dungeon screen array on screen
        IF Dungeon.Cell = 46 OR Dungeon.Cell = 176 THEN ' cell is dungeon wall
            COLOR 7 ' color low white
        ELSE ' compare dungeon cell type
            ' check dungeon cell is room border
            IF Dungeon.Cell > 21 AND Dungeon.Cell < 26 THEN ' compare dot
                Dungeon.Cell = 176 ' reset dungeon character to wall block
                COLOR 15 ' dungeon room border
            ELSE ' color dot
                COLOR 7 ' dungeon dot/wall
            END IF ' end compare dungeon cell type
        END IF ' end check dungeon cell type
        PRINT CHR$(Dungeon.Cell); ' print ascii character
    NEXT ' end dungeon column loop
    COLOR 7 ' display rightmost column
    PRINT CHR$(176); ' print ascii character
NEXT ' end dungeon row loop
Dungeon.X = 1 ' reset edit X cursor
Dungeon.Y = 1 ' reset edit Y cursor
COLOR 15 ' color white
LOCATE 1, 1, 1 ' reset editing cursor
RETURN ' exit routine

' routine to display one of two status lines
Display.Status.Line:
IF Status.Line THEN ' check status line toggle
    GOSUB Display.Status.Line1 ' display the first status line
ELSE ' check status line toggle
    GOSUB Display.Status.Line2 ' display the second status line
END IF ' end check status line toggle
LOCATE Dungeon.X, Dungeon.Y, 1 ' reset the cursor at the dungeon cell
RETURN ' exit routine

' routine to display status line one
Display.Status.Line1:
GOSUB Clear.Status.Line ' clear the status line area
LOCATE 24, 1 ' locate cursor at status line
' display status line part one
PRINT "1 = ~,  2 = $,  3 = " + CHR$(254) + ",  4 = ^,  5 = o,  6 = *,  7 = !,  8 = ?,  9 = >,  (+)Next";
LOCATE 25, 1 ' locate cursor at status line
' display status line part two
PRINT "Scroll, Gold,   Chest,  Trap,   Pit,    Gem,    Tele,   Temple, Stairs Up.";
RETURN ' exit routine

' routine to display status line two
Display.Status.Line2:
GOSUB Clear.Status.Line ' clear the status line area
LOCATE 24, 1 ' locate cursor at status line
' display status line part one
PRINT "A = <,  B = &,  C = |,  D = @,  E = %,  F = " + CHR$(34) + ",  0 = #,   X = .,         (-)Prev";
LOCATE 25, 1 ' locate cursor at status line
' display status line part two
PRINT "Down,   Coins,  Wand,   Potion, Staff,  Ring,   Monster, Empty Cell, <esc>=quit";
RETURN ' exit routine

' routine to clear status lines with blanks
Clear.Status.Line:
LOCATE 24, 1 ' locate cursor at status line
PRINT SPACE$(79); ' display line of blanks
LOCATE 25, 1 ' locate cursor at status line
PRINT SPACE$(79); ' display line of blanks
COLOR 7 ' color white
RETURN ' exit routine

' routine changes the dungeon cell in the current dungeon level map
' at the current dungeon x, y coordinates at the cursor location.
Change.Cell:
Current.Cell = GetDungeonRecordLevelData(Dungeon.X, Dungeon.Y)
SELECT CASE Current.Cell ' check dungeon cell type to wall, room border
    CASE 21, 22, 23, 24, 25, 26, 176 ' dungeon cell wall, room border values
        RETURN ' exit routine without changing cell
END SELECT ' end check dungeon cell type
SELECT CASE UCASE$(Input.Char$) ' determine cell type to change
    ' store current cell to change to, and current cell type.
    CASE "O" ' monster
        Current.Cell = ASC("#") ' cell symbol
        Cell.Type = 6 ' cell type
    CASE "1" ' scroll
        Current.Cell = ASC("~") ' cell symbol
        Cell.Type = 7 ' cell type
    CASE "2" ' gold
        Current.Cell = ASC("$") ' cell symbol
        Cell.Type = 11 ' cell type
    CASE "3" ' chest
        Current.Cell = 254 ' block character (chest)
        Cell.Type = 23 ' cell type
    CASE "4" ' trap
        Current.Cell = ASC("^") ' cell symbl
        Cell.Type = 9 ' cell type
    CASE "5" ' pit
        Current.Cell = ASC("o") ' cell symbol
        Cell.Type = 13 ' cell type
    CASE "6" ' gem
        Current.Cell = ASC("*") ' cell symbol
        Cell.Type = 19 ' cell type
    CASE "7" ' teleporter
        Current.Cell = ASC("!") ' cell symbol
        Cell.Type = 15 ' cell type
    CASE "8" ' temple
        Current.Cell = ASC("?") ' cell symbol
        Cell.Type = 17 ' cell type
    CASE "9" ' stairs down
        Current.Cell = ASC(">") ' cell symbol
        Cell.Type = 0 ' cell type
    CASE "A" ' stairs up
        Current.Cell = ASC("<") ' cell symbol
        Cell.Type = 0 ' cell type
    CASE "B" ' coins
        Current.Cell = ASC("&") ' cell symbol
        Cell.Type = 21 ' cell type
    CASE "C" ' wand
        Current.Cell = ASC("|") ' cell symbol
        Cell.Type = 25 ' cell type
    CASE "D" ' potion
        Current.Cell = ASC("@") ' cell symbol
        Cell.Type = 27 ' cell type
    CASE "E" ' staff
        Current.Cell = ASC("%") ' cell symbol
        Cell.Type = 29 ' cell type
    CASE "F" ' ring
        Current.Cell = 34 ' double quote
        Cell.Type = 31 ' cell type
    CASE "X" ' dot
        Current.Cell = ASC(".") ' dot/non-symbol (hallway)
        Cell.Type = -1 ' cell type (decrement if changing other symbol)
    CASE ELSE ' some other synbol
        Current.Cell = ASC(".") ' default to dot/non-symbol
        Cell.Type = 0 ' any cell type
END SELECT ' end determine cell type to change
' check the cell change type
SELECT CASE Cell.Type ' determine change type
    CASE IS > 0 ' cell is an additional symbol
        ' increment the value of the number of symbols in dungeon
        z1 = GetDungeonRecordStats(Cell.Type)
        z1 = z1 + 1
        CALL PutDungeonRecordStats(Cell.Type, z1)
    CASE -1 ' new cell is dot/non-symbol,
        ' otherwise cell is changed to dot/non-symbol,
        GOSUB Update.Counters ' decrement the number of symbols in dungeon
END SELECT ' end determine change type
' store the new cell type in the dungeon level data array
CALL PutDungeonRecordLevelData(Dungeon.X, Dungeon.Y, Current.Cell)
' check is dungeon cell was uncovered during game play
IF GetDungeonRecordScreenData(Dungeon.X, Dungeon.Y) <> 32 THEN ' cell not space
    CALL PutDungeonRecordScreenData(Dungeon.X, Dungeon.Y, Current.Cell) ' add cell type
END IF ' end check cell already uncovered
COLOR 7 ' color white
LOCATE Dungeon.X, Dungeon.Y, 1 ' set cursor at x, y position on screen
PRINT CHR$(Current.Cell); ' display the new cell value
LOCATE Dungeon.X, Dungeon.Y, 1 ' reset cursor at x, y position on screen
RETURN ' exit routine

' routine to decrement the total number of a specific dungeon cell symbol
' if the cell is replaced with a dot/non-symbol.
Update.Counters:
' get the current dungeon cell before changing,
Previous.Cell = GetDungeonRecordLevelData(Dungeon.X, Dungeon.Y)
SELECT CASE Previous.Cell ' determine the cell symbol to replace
    CASE ASC("#") ' monster
        Previous.Cell.Type = 6 ' cell type
    CASE ASC("~") ' scroll
        Previous.Cell.Type = 7 ' cell type
    CASE ASC("$") ' gold
        Previous.Cell.Type = 11 ' cell type
    CASE 254 ' block character (chest)
        Previous.Cell.Type = 23 ' cell type
    CASE ASC("^") ' trap
        Previous.Cell.Type = 9 ' cell type
    CASE ASC("o") ' pit
        Previous.Cell.Type = 13 ' cell type
    CASE ASC("*") ' gem
        Previous.Cell.Type = 19 ' cell type
    CASE ASC("!") ' teleporter
        Previous.Cell.Type = 15 ' cell type
    CASE ASC("?") ' temple
        Previous.Cell.Type = 17 ' cell type
    CASE ASC("&") ' coins
        Previous.Cell.Type = 21 ' cell type
    CASE ASC("|") ' wand
        Previous.Cell.Type = 25 ' cell type
    CASE ASC("@") ' potion
        Previous.Cell.Type = 27 ' cell type
    CASE ASC("%") ' staff
        Previous.Cell.Type = 29 ' cell type
    CASE 34 ' double quote (ring)
        Previous.Cell.Type = 31 ' cell type
    CASE ELSE ' some other symbol
        Previous.Cell.Type = 0 ' other cell type
END SELECT ' end determine replacement cell type
IF Previous.Cell.Type > 0 THEN ' check cell type
    ' decrement the total number of cell type being replaced
    z1 = GetDungeonRecordStats(Previous.Cell.Type)
    z1 = z1 - 1
    CALL PutDungeonRecordStats(Previous.Cell.Type, z1)
END IF ' end check cell type
RETURN ' exit routine

' open the data file, length of dungeon record
OpenDataFile:
CLOSE 2 ' close file 2
FileName$ = "datafile." + RIGHT$(STR$(User.Number + 1000), 3)
OPEN FileName$ FOR RANDOM SHARED AS #2 LEN = LEN(DungeonRecord)
RETURN ' exit routine
END SUB

